import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;

public class JDPLineLayout implements LayoutManager {
   int hgap = 3;
   Component[] leftc;
   int lastleftpos = -1;
   boolean staticLayout;

   public Dimension minimumLayoutSize(Container var1) {
      Dimension var2 = new Dimension(0, 0);

      for(int var3 = 0; var3 <= this.lastleftpos; ++var3) {
         if (this.leftc[var3] != null && this.leftc[var3].isVisible()) {
            Dimension var4 = this.leftc[var3].minimumSize();
            var2.height = Math.max(var2.height, var4.height);
            var2.width += var4.width + this.hgap;
         }
      }

      Insets var5 = var1.insets();
      var2.width += var5.left + var5.right;
      var2.height += var5.top + var5.bottom;
      return var2;
   }

   public String toString() {
      return this.getClass().getName() + "[hgap=" + this.hgap + "]";
   }

   public void removeLayoutComponent(Component var1) {
      for(int var2 = 0; var2 <= this.lastleftpos; ++var2) {
         if (var1.equals(this.leftc[var2])) {
            if (this.staticLayout) {
               this.leftc[var2] = null;
               return;
            }

            while(var2 < this.lastleftpos) {
               this.leftc[var2] = this.leftc[var2 + 1];
               ++var2;
            }

            this.lastleftpos += -1;
            this.leftc[var2] = null;
            return;
         }
      }

   }

   public JDPLineLayout() {
      this.leftc = new Component[500];
   }

   public JDPLineLayout(int var1) {
      this.hgap = var1;
      this.leftc = new Component[500];
   }

   public JDPLineLayout(int var1, boolean var2) {
      this.hgap = var1;
      this.staticLayout = var2;
      this.leftc = new Component[500];
   }

   public Dimension preferredLayoutSize(Container var1) {
      Dimension var2 = new Dimension(0, 0);

      for(int var3 = 0; var3 <= this.lastleftpos; ++var3) {
         if (this.leftc[var3] != null && this.leftc[var3].isVisible()) {
            Dimension var4 = this.leftc[var3].preferredSize();
            var2.height = Math.max(var2.height, var4.height);
            var2.width += var4.width + this.hgap;
         }
      }

      Insets var5 = var1.insets();
      var2.width += var5.left + var5.right;
      var2.height += var5.top + var5.bottom;
      return var2;
   }

   public void layoutContainer(Container var1) {
      Insets var2 = var1.insets();
      Dimension var3 = var1.preferredSize();
      int var4 = var2.top;
      int var5 = var2.left;

      for(int var6 = 0; var6 <= this.lastleftpos; ++var6) {
         if (this.leftc[var6] != null && this.leftc[var6].isVisible()) {
            Dimension var7 = this.leftc[var6].preferredSize();
            this.leftc[var6].reshape(var5, var4, var7.width, var7.height);
            var5 += var7.width + this.hgap;
         }
      }

   }

   public void addLayoutComponent(String var1, Component var2) {
      if (!this.staticLayout) {
         this.leftc[++this.lastleftpos] = var2;
      } else {
         for(int var3 = 0; var3 < this.leftc.length; ++var3) {
            if (this.leftc[var3] == null) {
               this.leftc[var3] = var2;
               ++this.lastleftpos;
               return;
            }
         }

      }
   }
}
