import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;

public class JDPSubsystemMgr extends Thread {
   static final int DISABLED = 0;
   static final int ENTRY_LEVEL = 1;
   static final int STANDARD = 2;
   static final int HIGH_END = 3;
   static int enterpriseServerLevel;
   static int enterpriseJobsAllowed;
   static int purgeJobNameAfterDays = 7;
   JaggServer server;
   JDPJobQueueMgr queueMgr;
   JDPJob systemJob;
   JDPJob schedulerJob;
   JDPJaggUserMgr userMgr;
   JDPJaggCxnMgr connectionMgr;
   ThreadGroup mainThreadGroup;
   JDPUtils utils;
   static String dbtype;
   Vector subsystems;
   Vector jobs;
   Vector notifyQueue;
   Vector systems;
   Vector prevJobNames;
   Vector entryLevelTasksDisallowed;
   boolean shutdownRequested;
   boolean updateRequested;
   boolean sleeping;
   boolean serverRunning;
   JDPJagg jaggSQL;
   JDPJagg jaggSQL1;
   Vector eventTypeCode;
   Vector eventTypeDesc;
   String JDPDomain;
   String JDPDirectory = "";
   String JDPJaggDir;
   String JDPCompiler;
   String JDPClasses;
   String NDSPath;
   String JDPJaggPath;
   String JDPJaggDSN;
   String JDPJaggCSTR;
   String JDPJaggJdbcDriver;
   String JDPJaggJdbcPrefix;
   String JDPJaggJdbcSuffix;
   String JDPJaggJdbcUserid;
   String JDPJaggJdbcPassword;
   String[] systemMsg = new String[]{"Could not submit '%j' to the Enterprise Server. Please purchase the Enterprise Server license from BulletProof.", "Could not submit '%j'. Max jobs of %d exceeded. Please purchase the %s from BulletProof.", "Job configuration settings are corrupted or missing. Please contact BulletProof.", "You must purchase an Enterprise Server license from BulletProof in order to perform this function.", "You must purchase an upgrade to the Standard or High End Enterprise Server in order to perform this function.", "This function may not be performed due to a server configuration error. Please contact BulletProof."};

   String addDelimiter(String var1) {
      String var2 = var1.substring(var1.length() - 1);
      if (!var2.equals("/") && !var2.equals("\\")) {
         if (var1.indexOf("/") >= 0) {
            var1 = var1 + "/";
         } else {
            var1 = var1 + "\\";
         }
      }

      return var1;
   }

   String checkTaskAllowed(String var1) {
      if (enterpriseServerLevel == 3) {
         return null;
      } else if (enterpriseServerLevel == 2) {
         return null;
      } else if (enterpriseServerLevel == 0) {
         return this.systemMsg[3];
      } else if (enterpriseServerLevel == 1) {
         return this.entryLevelTasksDisallowed.indexOf(var1) >= 0 ? this.systemMsg[4] : null;
      } else {
         return this.systemMsg[5];
      }
   }

   public String getClassForName(String var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getJobName().equals(var1) && var3.activeJob != null && var3.getInstance() != null) {
            return var3.getObject();
         }
      }

      return null;
   }

   public String getJobsForName(String var1) {
      String var2 = "";

      for(int var3 = 0; var3 < this.jobs.size(); ++var3) {
         JDPJob var4 = (JDPJob)this.jobs.elementAt(var3);
         if (var4.getJobName().equals(var1) && var4.getInstance() != null) {
            var2 = var2 + Integer.toString(var4.getJobNumber()) + ";";
         }
      }

      return var2;
   }

   void loadDisallowedTaskList() {
      this.entryLevelTasksDisallowed = new Vector();
      this.entryLevelTasksDisallowed.addElement("setMaxJobs");
      this.entryLevelTasksDisallowed.addElement("setSubsystemPriority");
      this.entryLevelTasksDisallowed.addElement("setSubsystemHeld");
      this.entryLevelTasksDisallowed.addElement("createSubsystem");
      this.entryLevelTasksDisallowed.addElement("removeSubsystem");
      this.entryLevelTasksDisallowed.addElement("setJobQueueSubsystem");
      this.entryLevelTasksDisallowed.addElement("setJobQueueHeld");
      this.entryLevelTasksDisallowed.addElement("createJobQueue");
      this.entryLevelTasksDisallowed.addElement("removeJobQueue");
      this.entryLevelTasksDisallowed.addElement("setJobJobQueue");
      this.entryLevelTasksDisallowed.addElement("setJobPriority");
      this.entryLevelTasksDisallowed.addElement("setJobQueuePriority");
      this.entryLevelTasksDisallowed.addElement("setJobHeldWhenActive");
   }

   public synchronized String createJobQueue(String var1) {
      String var2 = this.checkTaskAllowed("createJobQueue");
      if (var2 != null) {
         return var2;
      } else {
         Vector var3 = new Vector();
         String var4 = "SELECT QueueName FROM JDPQueue WHERE QueueName = '" + var1 + "'";
         int var5 = this.jaggSQL.execSQL(var4, var3);
         if (var5 == 1) {
            return "Job Queue already exists.";
         } else {
            var4 = "INSERT INTO JDPQueue (QueueName, QueueDesc, SubSysName, QueueHeld) " + "VALUES('" + var1 + "','','SBATCH','N')";
            var5 = this.jaggSQL.execSQL(var4, var3);
            if (var5 != 1) {
               return "Failed to create Job Queue " + var1 + ". See server log for details.";
            } else {
               JDPJobQueue var6 = new JDPJobQueue(this.queueMgr, var1);
               var6.setJobQueueDesc("");
               var6.setSubsystem("SBATCH");
               var6.setQueueHeld(false);
               this.queueMgr.queues.addElement(var6);
               return "Job Queue " + var1 + " successfully created with subsystem set to SBATCH.";
            }
         }
      }
   }

   public synchronized String removeJobQueue(String var1) {
      String var2 = this.checkTaskAllowed("removeJobQueue");
      if (var2 != null) {
         return var2;
      } else {
         JDPJobQueue var3 = this.getJobQueue(var1);
         if (var3.getQueuedJobCount() > 0) {
            return "Job Queue may not be removed because " + Integer.toString(var3.getQueuedJobCount()) + " job(s) are still queued.";
         } else {
            Vector var4 = new Vector();
            String var5 = "DELETE FROM JDPQueue WHERE QueueName = '" + var1 + "'";
            int var6 = this.jaggSQL.execSQL(var5, var4);
            if (var6 != 1) {
               return "Failed to remove Job Queue " + var1 + ". See server log for details.";
            } else {
               this.queueMgr.queues.removeElement(var3);
               return null;
            }
         }
      }
   }

   boolean check4Table(String var1) {
      this.jaggSQL.setFCTN("SQLTables");
      this.jaggSQL.setFCTNP(";;" + var1 + ";TABLE;");
      if (dbtype.indexOf(JDPJagg.ORACLE) >= 0 || dbtype.indexOf(JDPJagg.POSTGRES) >= 0 || dbtype.indexOf(JDPJagg.CLOUDSCAPE) >= 0) {
         this.jaggSQL.setFCTNP(";;" + var1.toUpperCase() + ";TABLE;");
      }

      Vector var2 = new Vector();
      int var3 = this.jaggSQL.execSQL("", var2);
      this.jaggSQL.setFCTN("");
      this.jaggSQL.setFCTNP("");
      if (var3 > 0) {
         return true;
      } else {
         if (dbtype.indexOf(JDPJagg.ORACLE) < 0 && dbtype.indexOf(JDPJagg.POSTGRES) < 0) {
            if (var1.length() > 8) {
               this.jaggSQL.setFCTN("SQLTables");
               this.jaggSQL.setFCTNP(";;" + var1.substring(0, 8) + ";TABLE;");
               var2 = new Vector();
               var3 = this.jaggSQL.execSQL("", var2);
               this.jaggSQL.setFCTN("");
               this.jaggSQL.setFCTNP("");
               if (var3 > 0) {
                  return true;
               }
            }
         } else {
            this.jaggSQL.setFCTN("SQLColumns");
            this.jaggSQL.setFCTNP(";;" + var1.toUpperCase() + ";;");
            var3 = this.jaggSQL.execSQL("", var2);
            this.jaggSQL.setFCTN("");
            this.jaggSQL.setFCTNP("");
            if (var3 > 0) {
               return true;
            }
         }

         return false;
      }
   }

   public synchronized String setSubsystemHeld(String var1, boolean var2) {
      String var3 = this.checkTaskAllowed("setSubsystemHeld");
      if (var3 != null) {
         return var3;
      } else {
         JDPSubsystem var4 = this.getSubsystem(var1);
         var4.setSubsystemHeld(var2);
         if (var2) {
            ((Thread)var4).suspend();
         } else {
            ((Thread)var4).resume();
         }

         Vector var5 = new Vector();
         String var6 = "UPDATE JDPSubs SET SubSysHeld = '" + (var2 ? "Y" : "N") + "' WHERE SubSysName = '" + var1 + "'";
         this.jaggSQL.execSQL(var6, var5);
         this.queueMgr.reOrderQueue();
         return null;
      }
   }

   public String getClassForNumber(int var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getJobNumber() == var1 && var3.activeJob != null) {
            return var3.getObject();
         }
      }

      return null;
   }

   public void interruptScheduler() {
      this.schedulerJob.getActiveJob().interrupt();
   }

   void loadPrevJobNames() {
      this.prevJobNames = new Vector();

      try {
         FileInputStream var1 = new FileInputStream(this.JDPDirectory + "System" + File.separator + "Server.cfg");
         byte[] var2 = new byte[var1.available()];
         var1.read(var2);

         for(int var3 = 0; var3 < var2.length; ++var3) {
            var2[var3] = (byte)(var2[var3] - 15);
         }

         String var8 = new String(var2, 0);
         StringTokenizer var4 = new StringTokenizer(var8, "\u0005");

         while(var4.hasMoreTokens()) {
            String var5 = var4.nextToken();
            if (var5.startsWith("NDS:")) {
               this.server.NDSusername = var5.substring(4, var5.indexOf(";"));
               this.server.NDSpassword = var5.substring(var5.indexOf(";") + 1);
            } else {
               StringTokenizer var6 = new StringTokenizer(var5, "|", true);
               if (var6.countTokens() != 5) {
                  this.prevJobNames = null;
                  this.println(this.systemMsg[2]);
                  var1.close();
                  if (enterpriseServerLevel == 0 || enterpriseServerLevel == 3) {
                     this.prevJobNames = null;
                  }

                  return;
               }

               this.prevJobNames.addElement(var5);
            }
         }

         var1.close();
         this.removeOldJobs();
      } catch (Exception var7) {
      }

      if (enterpriseServerLevel == 0 || enterpriseServerLevel == 3) {
         this.prevJobNames = null;
      }

   }

   public String setJobPriority(int var1, int var2, String var3) {
      String var4 = this.checkTaskAllowed("setJobPriority");
      if (var4 != null) {
         return var4;
      } else {
         JDPJob var5 = this.getJob(var1);
         var5.appendJobLog("Job Priority changed from " + Integer.toString(var5.getJobPriority()) + " to " + Integer.toString(var2) + " by: " + var3 + " on " + (new Date()).toString());
         var5.getActiveJob().setPriority(var2);
         var5.setJobPriority(var2);
         return null;
      }
   }

   synchronized void unloadJobs() {
      new Vector();
      Date var3 = new Date();
      long var4 = var3.getTime();
      var3.setHours(0);
      var3.setMinutes(0);
      var3.setSeconds(0);
      var4 -= var3.getTime();
      if (this.systemJob.getJobStatus() == 4) {
         this.systemJob.setQueuedDate(var3);
         this.systemJob.setQueuedTime(var4);
      }

      for(int var6 = 0; var6 < this.jobs.size(); ++var6) {
         JDPJob var7 = (JDPJob)this.jobs.elementAt(var6);
         String var8 = this.utils.replace(var7.getJobLog(), "'", "''");
         if (dbtype.indexOf(JDPJagg.INFORMIX) >= 0) {
            var8 = var8.replace('\n', '|');
            var8 = var8.replace('\r', '|');
            var8 = var8.replace('\t', ' ');
            if (var8.length() > 255) {
               var8 = var8.substring(0, 255);
            }
         }

         String var9 = "Job Log contains invalid characters that could not be inserted into log table";
         if (var7.getSkipJobLog()) {
            var8 = var9;
         }

         String var10 = var7.getObjectConstructor();
         if (var10.length() > 240) {
            var10 = var10.substring(0, 240);
         }

         var10 = this.utils.replace(var10, "'", "''");
         String var11 = var7.getObjectParm();
         if (var11.length() > 240) {
            var11 = var11.substring(0, 240);
         }

         var11 = this.utils.replace(var11, "'", "''");
         if (var7.getLogEventNow()) {
            String var12 = "-1";
            String var13 = "";
            if (var7.getNotifyNow()) {
               var12 = Integer.toString(var7.getNotifyUser());
               var13 = var7.getNotifyVia();
            }

            long var14 = var7.getEndDate().getTime() + var7.getEndTime() - var7.getStartDate().getTime() - var7.getStartTime();
            String var2 = "INSERT INTO JDPEvtLo (EventId, JDPSystem, EventName, EvtSource, Category, " + "UserName, Computer, QueueDate, QueueTime, JobQueue, SubSystem, StartDate, StartTime, " + "EndDate, EndTime, ObjectType, Object, ObjectParm, ObjectCons, Notified, NotifyVia, " + "TotalTime, FinalSts, EventLog) " + "VALUES(" + var7.getJobNumber() + ", " + "'" + this.utils.replace(var7.getJDPSystem(), "'", "''") + "', " + "'" + this.utils.replace(var7.getJobName(), "'", "''") + "', " + "'JDPSubsystemMgr', " + "'" + (String)this.eventTypeDesc.elementAt(this.eventTypeCode.indexOf(var7.getEventType())) + "', " + "'" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "'" + this.JDPDomain + "', " + "" + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "" + Long.toString(var7.getQueuedTime()) + ", " + "'" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "'" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "" + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "" + Long.toString(var7.getStartTime()) + ", " + "" + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "" + Long.toString(var7.getEndTime()) + ", " + "'" + var7.getObjectType() + "', " + "'" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "'" + var11 + "', " + "'" + var10 + "', " + "" + var12 + ", " + "'" + var13 + "', " + "" + Long.toString(var14) + ", " + "" + Integer.toString(var7.getJobStatus()) + ", " + "'" + var8 + "'" + ")";
            Vector var1 = new Vector();
            int var16 = this.jaggSQL1.execSQL(var2, var1);
            if (var16 < 0) {
               var2 = "INSERT INTO JDPEvtLo (EventId, JDPSystem, EventName, EvtSource, Category, " + "UserName, Computer, QueueDate, QueueTime, JobQueue, SubSystem, StartDate, StartTime, " + "EndDate, EndTime, ObjectType, Object, ObjectParm, ObjectCons, Notified, NotifyVia, " + "TotalTime, FinalSts, EventLog) " + "VALUES(" + var7.getJobNumber() + ", " + "'" + this.utils.replace(var7.getJDPSystem(), "'", "''") + "', " + "'" + this.utils.replace(var7.getJobName(), "'", "''") + "', " + "'JDPSubsystemMgr', " + "'" + (String)this.eventTypeDesc.elementAt(this.eventTypeCode.indexOf(var7.getEventType())) + "', " + "'" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "'" + this.JDPDomain + "', " + "" + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "" + Long.toString(var7.getQueuedTime()) + ", " + "'" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "'" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "" + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "" + Long.toString(var7.getStartTime()) + ", " + "" + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "" + Long.toString(var7.getEndTime()) + ", " + "'" + var7.getObjectType() + "', " + "'" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "'" + var11 + "', " + "'" + var10 + "', " + "" + var12 + ", " + "'" + var13 + "', " + "" + Long.toString(var14) + ", " + "" + Integer.toString(var7.getJobStatus()) + ", " + "'" + var9 + "'" + ")";
               var1 = new Vector();
               this.jaggSQL1.execSQL(var2, var1);
            }

            var7.setLogEventNow(false);
         }

         if (var7.getNotifyNow()) {
            this.notifyQueue.addElement(var7);
            var7.setNotifyNow(false);
         }

         if (var7.getJobChanged() || var7.equals(this.systemJob)) {
            var7.setJobChanged(false);
            if (var7.getJobStatus() != 10 && var7.getJobStatus() != 9) {
               String var25 = "UPDATE JDPJobA SET JobOwner = '" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "EventType = '" + var7.getEventType() + "', " + "QueueName = '" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "SubSysName = '" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "JobStatus = " + Integer.toString(var7.getJobStatus()) + ", " + "JobPriorty = " + Integer.toString(var7.getJobPriority()) + ", " + "QuePriorty = " + Integer.toString(var7.getJobQueuePriority()) + ", " + "ObjectType = '" + var7.getObjectType() + "', " + "Object = '" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "ObjectParm = '" + var11 + "', " + "ObjectCons = '" + var10 + "', " + "NotifyUser = " + Integer.toString(var7.getNotifyUser()) + ", " + "NotifyVia = '" + var7.getNotifyVia() + "', " + "NotifyOn = '" + var7.getNotifyOn() + "', " + "LogEvent = '" + var7.getLogEvent() + "', " + "QueueDate = " + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "QueueTime = " + Long.toString(var7.getQueuedTime()) + ", " + "StartDate = " + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "StartTime = " + Long.toString(var7.getStartTime()) + ", " + "EndDate = " + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "EndTime = " + Long.toString(var7.getEndTime()) + ", " + "JobLog = '" + var8 + "'" + " WHERE JobNumber = " + var7.getJobNumber();
               Vector var19 = new Vector();
               int var34 = this.jaggSQL1.execSQL(var25, var19);
               if (var34 < 0) {
                  var7.setSkipJobLog(true);
                  var25 = "UPDATE JDPJobA SET JobOwner = '" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "EventType = '" + var7.getEventType() + "', " + "QueueName = '" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "SubSysName = '" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "JobStatus = " + Integer.toString(var7.getJobStatus()) + ", " + "JobPriorty = " + Integer.toString(var7.getJobPriority()) + ", " + "QuePriorty = " + Integer.toString(var7.getJobQueuePriority()) + ", " + "ObjectType = '" + var7.getObjectType() + "', " + "Object = '" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "ObjectParm = '" + var11 + "', " + "ObjectCons = '" + var10 + "', " + "NotifyUser = " + Integer.toString(var7.getNotifyUser()) + ", " + "NotifyVia = '" + var7.getNotifyVia() + "', " + "NotifyOn = '" + var7.getNotifyOn() + "', " + "LogEvent = '" + var7.getLogEvent() + "', " + "QueueDate = " + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "QueueTime = " + Long.toString(var7.getQueuedTime()) + ", " + "StartDate = " + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "StartTime = " + Long.toString(var7.getStartTime()) + ", " + "EndDate = " + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "EndTime = " + Long.toString(var7.getEndTime()) + ", " + "JobLog = '" + var9 + "'" + " WHERE JobNumber = " + var7.getJobNumber();
                  var19 = new Vector();
                  this.jaggSQL1.execSQL(var25, var19);
               } else if (var34 == 0) {
                  var25 = "INSERT INTO JDPJobA (JDPSystem, JobName, JobNumber, EventType, JobOwner, QueueName, " + "SubSysName, JobStatus, JobPriorty, QuePriorty, ObjectType, Object, ObjectParm, ObjectCons, " + "NotifyUser, NotifyVia, NotifyOn, LogEvent, " + "QueueDate, QueueTime, StartDate, StartTime, EndDate, EndTime, JobLog) " + "VALUES('" + var7.getJDPSystem() + "', " + "'" + this.utils.replace(var7.getJobName(), "'", "''") + "', " + "" + var7.getJobNumber() + ", " + "'" + var7.getEventType() + "', " + "'" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "'" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "'" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "" + Integer.toString(var7.getJobStatus()) + ", " + "" + Integer.toString(var7.getJobPriority()) + ", " + "" + Integer.toString(var7.getJobQueuePriority()) + ", " + "'" + var7.getObjectType() + "', " + "'" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "'" + var11 + "', " + "'" + var10 + "', " + "" + Integer.toString(var7.getNotifyUser()) + ", " + "'" + var7.getNotifyVia() + "', " + "'" + var7.getNotifyOn() + "', " + "'" + var7.getLogEvent() + "', " + "" + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "" + Long.toString(var7.getQueuedTime()) + ", " + "" + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "" + Long.toString(var7.getStartTime()) + ", " + "" + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "" + Long.toString(var7.getEndTime()) + ", " + "'" + var8 + "'" + ")";
                  var19 = new Vector();
                  var34 = this.jaggSQL1.execSQL(var25, var19);
                  if (var34 < 0) {
                     var7.setSkipJobLog(true);
                     var25 = "INSERT INTO JDPJobA (JDPSystem, JobName, JobNumber, EventType, JobOwner, QueueName, " + "SubSysName, JobStatus, JobPriorty, QuePriorty, ObjectType, Object, ObjectParm, ObjectCons, " + "NotifyUser, NotifyVia, NotifyOn, LogEvent, " + "QueueDate, QueueTime, StartDate, StartTime, EndDate, EndTime, JobLog) " + "VALUES('" + var7.getJDPSystem() + "', " + "'" + this.utils.replace(var7.getJobName(), "'", "''") + "', " + "" + var7.getJobNumber() + ", " + "'" + var7.getEventType() + "', " + "'" + this.utils.replace(var7.getJobOwner(), "'", "''") + "', " + "'" + this.utils.replace(var7.getJobQueue(), "'", "''") + "', " + "'" + this.utils.replace(var7.getSubsystem(), "'", "''") + "', " + "" + Integer.toString(var7.getJobStatus()) + ", " + "" + Integer.toString(var7.getJobPriority()) + ", " + "" + Integer.toString(var7.getJobQueuePriority()) + ", " + "'" + var7.getObjectType() + "', " + "'" + this.utils.replace(var7.getObject(), "'", "''") + "', " + "'" + var11 + "', " + "'" + var10 + "', " + "" + Integer.toString(var7.getNotifyUser()) + ", " + "'" + var7.getNotifyVia() + "', " + "'" + var7.getNotifyOn() + "', " + "'" + var7.getLogEvent() + "', " + "" + JDPDate.formatForSql(var7.getQueuedDate(), dbtype) + ", " + "" + Long.toString(var7.getQueuedTime()) + ", " + "" + JDPDate.formatForSql(var7.getStartDate(), dbtype) + ", " + "" + Long.toString(var7.getStartTime()) + ", " + "" + JDPDate.formatForSql(var7.getEndDate(), dbtype) + ", " + "" + Long.toString(var7.getEndTime()) + ", " + "'" + var9 + "'" + ")";
                     var19 = new Vector();
                     this.jaggSQL1.execSQL(var25, var19);
                  }
               }
            } else {
               String var24 = "DELETE FROM JDPJobA " + " WHERE JobNumber = " + var7.getJobNumber();
               Vector var18 = new Vector();
               this.jaggSQL1.execSQL(var24, var18);
            }
         }
      }

      this.removeCompletedJobs();
   }

   int getTrialDaysRemaining() {
      this.jaggSQL.getSEP();
      Vector var1 = new Vector();
      int var2 = 0;
      if (this.JDPJaggDir != null) {
         String var3 = this.JDPJaggDir + "JDPJagg.exe CheckLicense";
         this.jaggSQL.setFCTN("System");
         this.jaggSQL.setFCTNP("");
         var2 = this.jaggSQL.execSQL(var3, var1);
         if (var2 <= 0) {
            System.out.println(this.jaggSQL.getError());
         }
      } else {
         this.jaggSQL.setFCTN("UnixDaysRemaining");
         this.jaggSQL.setFCTNP("");
         var2 = this.jaggSQL.execSQL("", var1);
      }

      this.jaggSQL.setFCTN("");
      if (var2 < 0) {
         var2 = 0;
      }

      return var2;
   }

   public synchronized String setJobQueueHeld(String var1, boolean var2) {
      String var3 = this.checkTaskAllowed("setJobQueueHeld");
      if (var3 != null) {
         return var3;
      } else {
         JDPJobQueue var4 = this.getJobQueue(var1);
         var4.setQueueHeld(var2);
         Vector var5 = new Vector();
         String var6 = "UPDATE JDPQueue SET QueueHeld = '" + (var2 ? "Y" : "N") + "' WHERE QueueName = '" + var1 + "'";
         this.jaggSQL.execSQL(var6, var5);
         return null;
      }
   }

   public Object getJobRunningClass(String var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getObject().equals(var1) && var3.activeJob != null) {
            return var3.activeJob.thisInstance;
         }
      }

      return null;
   }

   JDPJob submitJob(String var1, String var2, String var3, String var4, String var5, String var6, String var7, String var8, String var9, int var10, int var11, boolean var12, int var13, String var14, String var15, String var16) {
      String var17 = this.checkJobSubmissionAllowed(var5, var3, var7);
      if (var17 != null) {
         return null;
      } else {
         var8 = this.utils.replace(var8, " ", "");
         JDPJob var18 = this.queueMgr.submitJob(var1, var2, var3, var4, var5, var6, var7, var8, var9, var10, var11, var12, var13, var14, var15, var16);
         if (var18 == null) {
            return null;
         } else {
            this.addJobName(var5, var3, var7);
            return var18;
         }
      }
   }

   String submitJob(String var1, String var2, String var3, String var4, String var5, String var6, String var7, String var8, String var9, boolean var10, int var11, int var12, int var13, String var14, String var15, String var16) {
      String var17 = this.checkJobSubmissionAllowed(var5, var3, var7);
      if (var17 != null) {
         return var17;
      } else {
         var8 = this.utils.replace(var8, " ", "");
         JDPJob var18 = this.queueMgr.submitJob(var1, var2, var3, var4, var5, var6, var7, var8, var9, var11, var12, var10, var13, var14, var15, var16);
         if (var18 == null) {
            return "Failed to submit job because Job Queue not found: " + var1;
         } else {
            this.addJobName(var5, var3, var7);
            return null;
         }
      }
   }

   public String submitJob(String var1, String var2, String var3, String var4, String var5, String var6, String var7, String var8, int var9, int var10, boolean var11, int var12, String var13, String var14, String var15) {
      String var16 = this.checkJobSubmissionAllowed(var4, var3, var6);
      if (var16 != null) {
         return var16;
      } else {
         var7 = this.utils.replace(var7, " ", "");
         JDPJob var17 = this.queueMgr.submitJob(var1, var2, var3, "D", var4, var5, var6, var7, var8, var9, var10, var11, var12, var13, var14, var15);
         if (var17 == null) {
            return "Failed to submit job because Job Queue not found: " + var1;
         } else {
            this.addJobName(var4, var3, var6);
            return "Jobnumber:" + Integer.toString(var17.getJobNumber());
         }
      }
   }

   void registerJob(JDPJob var1) {
      this.jobs.addElement(var1);
   }

   void removeCompletedJobs() {
      boolean var1 = false;
      int var2 = this.jobs.size();

      for(int var3 = 0; var3 < var2; ++var3) {
         JDPJob var4 = (JDPJob)this.jobs.elementAt(var3);
         if ((var4.getJobStatus() == 10 || var4.getJobStatus() == 9) && !var4.getLogEventNow() && !var4.getJobChanged() && !var4.getNotifyNow()) {
            this.jobs.removeElement(var4);
            --var3;
            --var2;
            var1 = true;
         }
      }

      System.gc();
      if (var1) {
         System.runFinalization();
      }

   }

   public Object getJobForName(String var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getJobName().equals(var1) && var3.activeJob != null) {
            return var3.activeJob.thisInstance;
         }
      }

      return null;
   }

   String checkJobSubmissionAllowed(String var1, String var2, String var3) {
      if (this.isValidSystemUser(var1)) {
         return null;
      } else if (enterpriseServerLevel == 3) {
         return null;
      } else if (enterpriseServerLevel == 0) {
         String var10 = this.utils.replace(this.systemMsg[0], "%j", var2);
         this.println(var10);
         return var10;
      } else {
         if (var3.startsWith("Prelim")) {
            var3 = var2;
         }

         this.removeOldJobs();
         if (enterpriseJobsAllowed > this.prevJobNames.size()) {
            return null;
         } else if (this.prevJobNames == null) {
            this.println(this.systemMsg[2]);
            return this.systemMsg[2];
         } else {
            for(int var4 = 0; var4 < this.prevJobNames.size(); ++var4) {
               String var5 = (String)this.prevJobNames.elementAt(var4);
               if (var5.startsWith(var2 + "|" + var3)) {
                  return null;
               }
            }

            String var6 = this.systemMsg[1];
            var6 = this.utils.replace(var6, "%j", var2);
            var6 = this.utils.replace(var6, "%d", Integer.toString(enterpriseJobsAllowed));
            String var11 = "Standard or High End Server";
            if (enterpriseServerLevel == 2) {
               var11 = "High End Server";
            }

            var6 = this.utils.replace(var6, "%s", var11);
            this.println(var6);
            return var6;
         }
      }
   }

   public JDPSubsystemMgr(JaggServer var1) {
      this.server = var1;
      enterpriseServerLevel = this.server.enterpriseServerLevel;
      enterpriseJobsAllowed = this.server.enterpriseJobsAllowed;
      purgeJobNameAfterDays = this.server.purgeJobNameAfterDays;
      JDPJagg.activated = true;
      JDPButton.activated = true;
      JDPJagg.version = JDesignerPro.version;
      this.mainThreadGroup = new ThreadGroup("JDPSubsystemMgr");
      this.connectionMgr = new JDPJaggCxnMgr(this, var1);
      this.connectionMgr.start();
      this.userMgr = new JDPJaggUserMgr(this);
      this.userMgr.start();
      this.jobs = new Vector();
      this.notifyQueue = new Vector();
      boolean var2 = this.loadJaggSettings();
      if (var2) {
         ((Thread)this).start();
      }
   }

   void logException(Exception var1) {
      if (this.systemJob != null) {
         this.systemJob.logException(var1);
      } else {
         ((Throwable)var1).printStackTrace();
      }
   }

   public Object getJobForNumber(int var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getJobNumber() == var1 && var3.activeJob != null) {
            return var3.activeJob.thisInstance;
         }
      }

      return null;
   }

   public String getJDPSystemSetup(String var1) {
      for(int var2 = 0; var2 < this.systems.size(); ++var2) {
         String var3 = (String)this.systems.elementAt(var2);
         if (var3.startsWith(var1)) {
            return var3.substring(5);
         }
      }

      return null;
   }

   public String getJobLog(int var1) {
      return this.getJob(var1).getJobLog();
   }

   public void run() {
      if (!this.serverRunning) {
         while(!this.loadJaggSettings()) {
            this.waitFor(30);
         }

         this.serverRunning = true;
         this.loadPrevJobNames();
         this.loadDisallowedTaskList();
         String var1 = this.getJDPSystemSetup("01001");
         this.jaggSQL = this.setupJaggInstance(var1, "01001");
         if (this.getTrialDaysRemaining() == 0) {
            System.out.println("Trial Period Expired. Please purchase a License from BulletProof.");
            System.out.println("http://www.bulletproof.com. (800)505-0105 (408)374-2323 sales@bulletproof.com.");
         } else {
            this.utils = new JDPUtils((JDPUser)null);
            this.utils.jaggSQL = new JDPJagg(this.JDPJaggPath);
            dbtype = this.utils.getDataSourceType(this.jaggSQL);
            this.jaggSQL1 = this.setupJaggInstance(var1);

            while(!this.check4Table("JDPJobNo")) {
               this.waitFor(30);
            }

            this.loadEventTypes();

            while(!this.createSubsystems()) {
               this.waitFor(30);
            }

            this.startSystemJobs();

            while(!this.shutdownRequested) {
               this.unloadJobs();
               if (!this.updateRequested) {
                  try {
                     this.sleeping = true;
                     Thread.sleep(60000L);
                  } catch (Exception var2) {
                  }
               }

               this.updateRequested = false;
               this.sleeping = false;
            }

         }
      }
   }

   public boolean triggerEvent(String var1, String var2) {
      JDPScheduler var3 = (JDPScheduler)this.schedulerJob.getInstance();
      if (var3 != null) {
         return var3.triggerEvent(var1, var2);
      } else {
         this.println("Failed to get handle to Scheduler to trigger event: " + var2);
         return false;
      }
   }

   public String setJobJobQueue(int var1, String var2, String var3) {
      String var4 = this.checkTaskAllowed("setJobJobQueue");
      if (var4 != null) {
         return var4;
      } else {
         JDPJob var5 = this.getJob(var1);
         var5.appendJobLog("Job Queue changed from " + var5.getJobQueue() + " to " + var2 + " by: " + var3 + " on " + (new Date()).toString());
         JDPJobQueue var6 = this.queueMgr.getJobQueue(var5.getJobQueue());
         var6.removeJob(var5);
         var6 = this.queueMgr.getJobQueue(var2);
         var6.submitJob(var5);
         return null;
      }
   }

   public String getSubsystems() {
      String var1 = " " + this.jaggSQL.getSEP();
      StringBuffer var2 = new StringBuffer();

      try {
         if (this.subsystems != null) {
            for(int var3 = 0; var3 < this.subsystems.size(); ++var3) {
               JDPSubsystem var4 = (JDPSubsystem)this.subsystems.elementAt(var3);
               String var5 = var4.getSubsystemDesc();
               if (var5 == null) {
                  var5 = "";
               }

               var2.append(var4.getSubsystemName());
               var2.append(var1);
               var2.append(var5);
               var2.append(var1);
               var2.append(Integer.toString(var4.getMaxJobs()));
               var2.append(var1);
               var2.append(var4.getSubsystemHeld() ? "Y" : "N");
               var2.append(var1);
               var2.append(Integer.toString(((Thread)var4).getPriority()));
               var2.append(var1);
            }
         }
      } catch (Exception var7) {
         this.logException(var7);
      }

      return var2.toString();
   }

   public String[] getSystems() {
      String[] var1 = new String[this.systems.size()];

      for(int var2 = 0; var2 < this.systems.size(); ++var2) {
         var1[var2] = (String)this.systems.elementAt(var2);
      }

      return var1;
   }

   void println(Object var1) {
      if (this.systemJob != null) {
         this.systemJob.appendJobLog(var1.toString());
      } else {
         System.out.println(var1);
      }
   }

   public String setJobHeld(int var1, boolean var2, String var3) {
      JDPJob var4 = this.getJob(var1);
      if (var2) {
         if (var4.getJobStatus() == 4) {
            String var5 = this.checkTaskAllowed("setJobHeldWhenActive");
            if (var5 != null) {
               return var5;
            }

            var4.getActiveJob().suspend();
            var4.setJobStatus(5);
         }

         if (var4.getJobStatus() == 2) {
            String var6 = this.checkTaskAllowed("setJobHeldWhenQueued");
            if (var6 != null) {
               return var6;
            }

            var4.setJobStatus(3);
            this.queueMgr.reOrderQueue();
         }

         var4.appendJobLog("Job held by: " + var3 + " on " + (new Date()).toString());
      } else {
         if (var4.getJobStatus() == 5) {
            String var7 = this.checkTaskAllowed("setJobReleasedWhenActive");
            if (var7 != null) {
               return var7;
            }

            var4.getActiveJob().resume();
            var4.setJobStatus(4);
         }

         if (var4.getJobStatus() == 3) {
            String var8 = this.checkTaskAllowed("setJobReleasesWhenQueued");
            if (var8 != null) {
               return var8;
            }

            var4.setJobStatus(2);
            this.queueMgr.reOrderQueue();
         }

         var4.appendJobLog("Job released by: " + var3 + " on " + (new Date()).toString());
      }

      return null;
   }

   synchronized void loadEventTypes() {
      Vector var1 = new Vector();
      String var2 = this.jaggSQL.getSEP();
      this.eventTypeCode = new Vector();
      this.eventTypeDesc = new Vector();
      int var3 = this.jaggSQL.execSQL("SELECT EventType, TypeDesc FROM JDPEvtTy", var1);
      if (var3 >= 1) {
         for(int var4 = 0; var4 < var1.size(); ++var4) {
            StringTokenizer var5 = new StringTokenizer((String)var1.elementAt(var4), var2);
            this.eventTypeCode.addElement(var5.nextToken().trim());
            this.eventTypeDesc.addElement(var5.nextToken().trim());
         }

      }
   }

   void savePrevJobNames() {
      try {
         FileOutputStream var1 = new FileOutputStream(this.JDPDirectory + "System" + File.separator + "Server.cfg");
         String var2 = "";
         if (this.NDSPath != null) {
            var2 = "NDS:" + this.server.NDSusername + ";" + this.server.NDSpassword + "\u0005";
         }

         if (this.prevJobNames != null) {
            for(int var3 = 0; var3 < this.prevJobNames.size(); ++var3) {
               var2 = var2 + (String)this.prevJobNames.elementAt(var3) + "\u0005";
            }
         }

         byte[] var7 = new byte[var2.length()];
         var2.getBytes(0, var7.length, var7, 0);

         for(int var4 = 0; var4 < var7.length; ++var4) {
            var7[var4] = (byte)(var7[var4] + 15);
         }

         var1.write(var7);
         var1.close();
      } catch (Exception var6) {
         this.logException(var6);
      }
   }

   JDPSubsystem getSubsystem(String var1) {
      for(int var2 = 0; var2 < this.subsystems.size(); ++var2) {
         JDPSubsystem var3 = (JDPSubsystem)this.subsystems.elementAt(var2);
         if (var3.getSubsystemName().equals(var1)) {
            return var3;
         }
      }

      return null;
   }

   JDPJagg setupJaggInstanceFromSystem(String var1) {
      String var2 = this.getJDPSystemSetup("01001");
      return this.setupJaggInstance(var2);
   }

   public JDPJob getJob(int var1) {
      for(int var2 = 0; var2 < this.jobs.size(); ++var2) {
         JDPJob var3 = (JDPJob)this.jobs.elementAt(var2);
         if (var3.getJobNumber() == var1) {
            return var3;
         }
      }

      return null;
   }

   public String startSystemJob(String var1, String var2) {
      JDPSubsystem var3 = this.getSubsystem(var1);
      if (var3 != null) {
         if (var3.getJob(var2) == null) {
            if (var2.equals("JDPSubsystemMgr")) {
               String var4 = "S";
               String var5 = "JDPSubsystemMgr";
               String var6 = "";
               String var7 = "";
               this.systemJob = this.submitJob("QSYSTEM", "01001", "JDPSubsystemMgr", "A", "JDPSubsystemMgr", var4, var5, var6, var7, 6, 5, false, -1, "", "", "B");
            }

            if (var2.equals("JaggServer")) {
               String var9 = "S";
               String var12 = "JaggServer";
               String var15 = "";
               String var18 = "";
               JaggServerCxn.jaggServerJob = this.submitJob("QSYSTEM", "01001", "JaggServer", "A", "JDPSubsystemMgr", var9, var12, var15, var18, 6, 5, false, -1, "", "", "B");
            }

            if (var2.equals("JDPScheduler")) {
               String var10 = "C";
               String var13 = "JDPScheduler";
               String var16 = "(JDPSubsystemMgr,JDPJob)";
               String var19 = "JDPSubsystemMgr,JDPJob";
               this.schedulerJob = this.submitJob("QSYSTEM", "01001", "JDPScheduler", "A", "JDPSubsystemMgr", var10, var13, var16, var19, 5, 5, false, -1, "", "", "B");
               if (this.schedulerJob == null) {
                  this.println("Failed to start JDPScheduler");
               }
            }

            if (var2.equals("JDPNotificationMgr")) {
               String var11 = "C";
               String var14 = "JDPNotificationMgr";
               String var17 = "(JDPSubsystemMgr,JDPJob)";
               String var20 = "JDPSubsystemMgr,JDPJob";
               JDPJob var8 = this.submitJob("QSYSTEM", "01001", "JDPNotificationMgr", "A", "JDPSubsystemMgr", var11, var14, var17, var20, 5, 5, false, -1, "", "", "B");
               if (var8 == null) {
                  this.println("Failed to start JDPScheduler");
               }
            }

            return var2 + " successfully queued.";
         } else {
            return var2 + " is already started.";
         }
      } else {
         this.println("System Error: Subsystem " + var1 + " not found.");
         return "System Error: Subsystem " + var1 + " not found.";
      }
   }

   public String killJob(int var1, String var2) {
      String var3 = this.checkTaskAllowed("killJob");
      if (var3 != null) {
         return var3;
      } else {
         JDPJob var4 = this.getJob(var1);
         if (var4.getJobStatus() != 4 && var4.getJobStatus() != 5) {
            return "Job no longer Active. Check Job Log Inquiry";
         } else {
            var4.setJobStatus(4);
            var4.getActiveJob().stop();
            var4.getActiveJob().completeJob(false);
            var4.appendJobLog("Job terminated by: " + var2 + " on " + (new Date()).toString());
            var4.setJobStatus(9);
            return null;
         }
      }
   }

   public void requestUpdate() {
      this.updateRequested = true;
      if (this.sleeping) {
         ((Thread)this).interrupt();
      }

   }

   void startSystemJobs() {
      this.queueMgr = new JDPJobQueueMgr(this);
      this.queueMgr.setPriority(5);
      this.queueMgr.start();
      this.startSystemJob("SSYSTEM", "JDPSubsystemMgr");
      this.startSystemJob("SSYSTEM", "JaggServer");
      this.startSystemJob("SSYSTEM", "JDPScheduler");
      this.startSystemJob("SSYSTEM", "JDPNotificationMgr");
   }

   boolean isValidSystemUser(String var1) {
      return var1.equals("JDPSubsystemMgr");
   }

   void addJobName(String var1, String var2, String var3) {
      if (!this.isValidSystemUser(var1)) {
         if (enterpriseServerLevel != 3) {
            if (enterpriseServerLevel != 0) {
               if (this.prevJobNames != null) {
                  if (var3.startsWith("Prelim")) {
                     var3 = var2;
                  }

                  this.removeOldJobs();
                  int var4 = -1;

                  for(int var5 = 0; var5 < this.prevJobNames.size(); ++var5) {
                     String var6 = (String)this.prevJobNames.elementAt(var5);
                     if (var6.startsWith(var2 + "|" + var3)) {
                        var4 = var5;
                        break;
                     }
                  }

                  String var7 = Long.toString((new Date()).getTime());
                  if (var4 == -1) {
                     if (this.prevJobNames.size() < enterpriseJobsAllowed) {
                        if (var3.equals("")) {
                           var3 = " ";
                        }

                        this.prevJobNames.addElement(var2 + "|" + var3 + "|" + var7);
                     }
                  } else {
                     if (var3.equals("")) {
                        var3 = " ";
                     }

                     this.prevJobNames.setElementAt(var2 + "|" + var3 + "|" + var7, var4);
                  }

                  this.savePrevJobNames();
               }

            }
         }
      }
   }

   boolean createSubsystems() {
      String var2 = this.jaggSQL.getSEP();
      Vector var3 = new Vector();
      String var4 = "SELECT SubSysName, SubSysDesc, maxJobs, priority, SubSysHeld " + "FROM JDPSubs ORDER BY SubSysName";
      int var5 = this.jaggSQL.execSQL(var4, var3);
      if (var5 == -1) {
         return false;
      } else {
         this.subsystems = new Vector();

         for(int var6 = 0; var6 < var5; ++var6) {
            StringTokenizer var1 = new StringTokenizer((String)var3.elementAt(var6), var2);
            JDPSubsystem var7 = new JDPSubsystem(this, var1.nextToken().trim());
            var7.setSubsystemDesc(var1.nextToken().trim());
            var7.setMaxJobs(Integer.parseInt(var1.nextToken().trim()));
            int var8 = Integer.parseInt(var1.nextToken().trim());
            if (var8 <= 10 && var8 >= 1) {
               try {
                  ((Thread)var7).setPriority(var8);
               } catch (Exception var10) {
                  this.println("Failed to adjust thread priority to " + var8);
               }
            }

            String var9 = var1.nextToken().trim();
            var7.setSubsystemHeld(var9.equals("Y"));
            this.subsystems.addElement(var7);
            ((Thread)var7).start();
         }

         return true;
      }
   }

   public boolean loadJaggSettings() {
      this.systems = new Vector();
      Object var1 = null;

      try {
         FileInputStream var3 = new FileInputStream("JDesignerPro.ini");
         InputStream var8 = (InputStream)var3;
         DataInputStream var2 = new DataInputStream(var8);
         boolean var4 = false;

         for(String var5 = var2.readLine(); var5 != null; var5 = var2.readLine()) {
            if (var5.length() > 10) {
               if (var5.substring(0, 7).compareTo("DOMAIN=") == 0) {
                  this.JDPDomain = this.addDelimiter(var5.substring(7));
               }

               if (var5.substring(0, 7).compareTo("JDPDIR=") == 0) {
                  this.JDPDirectory = this.addDelimiter(var5.substring(7));
               }

               if (var5.substring(0, 8).compareTo("JAGGDIR=") == 0) {
                  this.JDPJaggDir = this.addDelimiter(var5.substring(8));
               }

               if (var5.substring(0, 9).compareTo("COMPILER=") == 0) {
                  this.JDPCompiler = var5.substring(9);
               }

               if (var5.substring(0, 8).compareTo("CLASSES=") == 0) {
                  this.JDPClasses = var5.substring(8);
               }

               if (var5.substring(0, 8).compareTo("NDSPATH=") == 0 && var5.length() > 8) {
                  this.NDSPath = var5.substring(8);
               }

               if (var5.length() > 15 && var5.substring(0, 15).compareTo("JAGGSERVERPORT=") == 0) {
                  JDPJagg.setDefaultPort(Integer.parseInt(var5.substring(15)));
               }

               if (var5.length() > 5 && var4) {
                  this.systems.addElement(var5);
               }

               if (var5.startsWith("[JDesignerPro Systems]")) {
                  var4 = true;
               }
            }
         }

         var8.close();
      } catch (Exception var7) {
         this.logException(var7);
         this.println("Could not open JDesignerPro.ini file.");
         this.println("Check that ini file is in the JDesignerPro directory.");
         return false;
      }

      return this.getJDPSystemSetup("01001") != null;
   }

   synchronized int getNextJobNumber() {
      if (!this.serverRunning) {
         return 0;
      } else {
         Vector var1 = new Vector();
         int var2 = this.jaggSQL.execSQL("UPDATE JDPJobNo SET NextJobNum = NextJobNum + 1", var1);
         if (var2 != 1) {
            return -1;
         } else {
            var1 = new Vector();
            var2 = this.jaggSQL.execSQL("SELECT NextJobNum FROM JDPJobNo", var1);
            if (var2 != 1) {
               return -1;
            } else {
               StringTokenizer var3 = new StringTokenizer((String)var1.elementAt(0), this.jaggSQL.getSEP());

               try {
                  int var4 = Integer.parseInt(var3.nextToken().trim());
                  return var4;
               } catch (Exception var5) {
                  return -1;
               }
            }
         }
      }
   }

   public synchronized String setSubsystemPriority(String var1, int var2) {
      String var3 = this.checkTaskAllowed("setSubsystemPriority");
      if (var3 != null) {
         return var3;
      } else {
         try {
            this.getSubsystem(var1).setPriority(var2);
         } catch (Exception var7) {
            this.logException(var7);
         }

         Vector var4 = new Vector();
         String var5 = "UPDATE JDPSubs SET priority = " + Integer.toString(var2) + " WHERE SubSysName = '" + var1 + "'";
         this.jaggSQL.execSQL(var5, var4);
         return null;
      }
   }

   JDPJobQueue getJobQueue(String var1) {
      return this.queueMgr.getJobQueue(var1);
   }

   public synchronized String createSubsystem(String var1) {
      String var2 = this.checkTaskAllowed("createSubsystem");
      if (var2 != null) {
         return var2;
      } else {
         Vector var3 = new Vector();
         String var4 = "SELECT SubSysName FROM JDPSubs WHERE SubSysName = '" + var1 + "'";
         int var5 = this.jaggSQL.execSQL(var4, var3);
         if (var5 == 1) {
            return "Subsystem already exists.";
         } else {
            var4 = "INSERT INTO JDPSubs (SubSysName, SubSysDesc, maxJobs, priority, SubSysHeld) " + "VALUES('" + var1 + "','',3," + Integer.toString(5) + ",'N')";
            var5 = this.jaggSQL.execSQL(var4, var3);
            if (var5 != 1) {
               return "Failed to create subsystem " + var1 + ". See server log for details.";
            } else {
               JDPSubsystem var6 = new JDPSubsystem(this, var1);
               var6.setSubsystemDesc("");
               var6.setMaxJobs(3);

               try {
                  ((Thread)var6).setPriority(5);
               } catch (Exception var7) {
                  this.println("Failed to adjust thread priority to " + 5);
               }

               var6.setSubsystemHeld(false);
               this.subsystems.addElement(var6);
               ((Thread)var6).start();
               return "Subsystem " + var1 + " successfully created with maxJobs set to 3.";
            }
         }
      }
   }

   public synchronized String removeSubsystem(String var1) {
      String var2 = this.checkTaskAllowed("removeSubsystem");
      if (var2 != null) {
         return var2;
      } else {
         JDPSubsystem var3 = this.getSubsystem(var1);

         for(int var4 = 0; var4 < this.queueMgr.queues.size(); ++var4) {
            JDPJobQueue var5 = (JDPJobQueue)this.queueMgr.queues.elementAt(var4);
            if (var5.getSubsystem().equals(var1)) {
               return "Subsystem may not be removed because queue " + var5.getJobQueueName() + " is using it.";
            }
         }

         if (var3.getActiveJobCount() > 0) {
            return "Subsystem may not be removed as jobs are still active.";
         } else {
            Vector var7 = new Vector();
            String var8 = "DELETE FROM JDPSubs WHERE SubSysName = '" + var1 + "'";
            int var6 = this.jaggSQL.execSQL(var8, var7);
            if (var6 != 1) {
               return "Failed to remove subsystem " + var1 + ". See server log for details.";
            } else {
               this.subsystems.removeElement(var3);
               return null;
            }
         }
      }
   }

   public synchronized String setJobQueueSubsystem(String var1, String var2) {
      String var3 = this.checkTaskAllowed("setJobQueueSubsystem");
      if (var3 != null) {
         return var3;
      } else {
         JDPJobQueue var4 = this.getJobQueue(var1);
         var4.setSubsystem(var2);
         Vector var5 = new Vector();
         String var6 = "UPDATE JDPQueue SET SubSysName = '" + var2 + "' WHERE QueueName = '" + var1 + "'";
         this.jaggSQL.execSQL(var6, var5);
         this.queueMgr.reOrderQueue();
         return null;
      }
   }

   public String getJobsForClass(String var1) {
      String var2 = "";

      for(int var3 = 0; var3 < this.jobs.size(); ++var3) {
         JDPJob var4 = (JDPJob)this.jobs.elementAt(var3);
         if (var4.getJobName().equals(var1) && var4.getInstance() != null) {
            var2 = var2 + Integer.toString(var4.getJobNumber()) + ";";
         }
      }

      return var2;
   }

   JDPJagg setupJaggInstance(String var1) {
      return this.setupJaggInstance(var1, (String)null);
   }

   public String setJobQueuePriority(int var1, int var2, String var3) {
      String var4 = this.checkTaskAllowed("setJobQueuePriority");
      if (var4 != null) {
         return var4;
      } else {
         JDPJob var5 = this.getJob(var1);
         var5.appendJobLog("Job Queue Priority changed from " + Integer.toString(var5.getJobQueuePriority()) + " to " + Integer.toString(var2) + " by: " + var3 + " on " + (new Date()).toString());
         var5.setJobQueuePriority(var2);
         this.queueMgr.reOrderQueue();
         return null;
      }
   }

   public String removeJob(int var1, String var2) {
      String var3 = this.checkTaskAllowed("removeJob");
      if (var3 != null) {
         return var3;
      } else {
         JDPJob var4 = this.getJob(var1);
         if (var4.getJobStatus() != 2 && var4.getJobStatus() != 3) {
            return "Job no longer on Queue. Check Active Jobs or Out Queue";
         } else {
            JDPJobQueue var5 = this.getJobQueue(var4.getJobQueue());
            JDPSubsystem var6 = this.getSubsystem(var5.getSubsystem());
            var4.appendJobLog("Job removed from Queue: " + var4.getJobQueue() + " by: " + var2 + " on " + (new Date()).toString());
            var4.activeJob = new JDPActiveJob(var6, var4);
            var4.activeJob.initJob();
            var4.activeJob.completeJob(false);
            return null;
         }
      }
   }

   JDPJagg setupJaggInstance(String var1, String var2) {
      int var3 = var1.indexOf("JAGGSERVER=Server");
      if (var3 > 0) {
         var1 = var1.substring(0, var3) + var1.substring(var3 + 17);
      } else {
         var3 = var1.indexOf("JAGGSERVER=CGI");
         if (var3 > 0) {
            var1 = var1.substring(0, var3) + var1.substring(var3 + 14);
         }
      }

      String var4 = this.JDPJaggPath;
      String var5 = this.JDPJaggDSN;
      String var6 = this.JDPJaggCSTR;
      String var7 = this.JDPJaggJdbcDriver;
      String var8 = this.JDPJaggJdbcPrefix;
      String var9 = this.JDPJaggJdbcSuffix;
      String var10 = this.JDPJaggJdbcUserid;
      String var11 = this.JDPJaggJdbcPassword;
      int var12 = var1.indexOf(" ");
      var4 = var1.substring(0, var12);
      var6 = var1.substring(var12 + 1);
      int var13 = var6.indexOf("DSN=") + 4;
      int var14 = var6.indexOf(";", var13);
      var5 = var6.substring(var13, var14);
      var3 = var1.indexOf("JDBCDRIVER=");
      if (var3 >= 0) {
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var7 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("JDBCPREFIX=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var8 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("JDBCSUFFIX=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var9 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("UID=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var10 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("PWD=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var11 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("CSTR=", var3 + 1);
         var6 = "";
         if (var3 > 0) {
            var3 = var1.indexOf("=", var3 + 1);
            var6 = var1.substring(var3 + 1);
         }
      } else {
         var3 = var1.indexOf("UID=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var10 = var1.substring(var3 + 1, var12);
         var3 = var1.indexOf("PWD=", var3 + 1);
         var3 = var1.indexOf("=", var3 + 1);
         var12 = var1.indexOf(";", var3 + 1);
         var11 = var1.substring(var3 + 1, var12);
      }

      JDPJagg var15 = new JDPJagg(var4);
      var15.setDSN(var5);
      var15.setCSTR(var6);
      var15.setJdbcDriver(var7);
      var15.setJdbcPrefix(var8);
      var15.setJdbcSuffix(var9);
      var15.setUID(var10);
      var15.setPWD(var11);
      if (var2 != null && var2.equals("01001")) {
         this.JDPJaggPath = var4;
         this.JDPJaggDSN = var5;
         this.JDPJaggCSTR = var6;
         this.JDPJaggJdbcDriver = var7;
         this.JDPJaggJdbcPrefix = var8;
         this.JDPJaggJdbcSuffix = var9;
         this.JDPJaggJdbcUserid = var10;
         this.JDPJaggJdbcPassword = var11;
         var15.loadJdbcDataSources();
      }

      return var15;
   }

   public String getActiveJobs() {
      String var1 = " " + this.jaggSQL.getSEP();
      String var2 = Long.toString((new Date()).getTime());
      StringBuffer var3 = new StringBuffer();

      for(int var4 = 0; var4 < this.jobs.size(); ++var4) {
         JDPJob var5 = (JDPJob)this.jobs.elementAt(var4);
         if (var5.getJobStatus() == 4 || var5.getJobStatus() == 5) {
            var3.append(var5.getJobName());
            var3.append(var1);
            var3.append(var5.getSubsystem());
            var3.append(var1);
            var3.append(var5.getJDPSystem());
            var3.append(var1);
            var3.append(Integer.toString(var5.getJobNumber()));
            var3.append(var1);
            var3.append(var5.getJobOwner());
            var3.append(var1);
            var3.append((String)this.eventTypeDesc.elementAt(this.eventTypeCode.indexOf(var5.getEventType())));
            var3.append(var1);
            var3.append(Long.toString(var5.getStartDate().getTime() + var5.getStartTime()));
            var3.append(var1);
            var3.append(Integer.toString(var5.getJobStatus()));
            var3.append(var1);
            var3.append(Integer.toString(var5.getJobPriority()));
            var3.append(var1);
            var3.append(var2);
            var3.append(var1);
         }
      }

      return var3.toString();
   }

   public synchronized String setMaxJobs(String var1, int var2) {
      String var3 = this.checkTaskAllowed("setMaxJobs");
      if (var3 != null) {
         return var3;
      } else {
         this.getSubsystem(var1).setMaxJobs(var2);
         Vector var4 = new Vector();
         String var5 = "UPDATE JDPSubs SET maxJobs = " + Integer.toString(var2) + " WHERE SubSysName = '" + var1 + "'";
         this.jaggSQL.execSQL(var5, var4);
         this.queueMgr.reOrderQueue();
         return null;
      }
   }

   void removeOldJobs() {
      if (this.prevJobNames != null && this.prevJobNames.size() != 0) {
         long var1 = (new Date()).getTime();
         long var3 = var1 - (long)(purgeJobNameAfterDays * 24 * 60 * 60 * 1000);

         for(int var5 = 0; var5 < this.prevJobNames.size(); ++var5) {
            String var6 = (String)this.prevJobNames.elementAt(var5);
            StringTokenizer var7 = new StringTokenizer(var6, "|");
            var7.nextToken();
            var7.nextToken();
            String var8 = var7.nextToken();
            long var9 = Long.valueOf(var8);
            if (var9 < var3) {
               this.prevJobNames.removeElement(var6);
               --var5;
            }
         }

      }
   }

   void waitFor(int var1) {
      try {
         Thread.sleep((long)(var1 * 1000));
      } catch (Exception var2) {
      }
   }
}
