package javax.ejb.deployment;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.security.Identity;

public class ControlDescriptor implements Serializable {
   public static final int TX_NOT_SUPPORTED = 0;
   public static final int TX_BEAN_MANAGED = 1;
   public static final int TX_REQUIRED = 2;
   public static final int TX_SUPPORTS = 3;
   public static final int TX_REQUIRES_NEW = 4;
   public static final int TX_MANDATORY = 5;
   public static final int TRANSACTION_READ_UNCOMMITTED = 1;
   public static final int TRANSACTION_READ_COMMITTED = 2;
   public static final int TRANSACTION_REPEATABLE_READ = 4;
   public static final int TRANSACTION_SERIALIZABLE = 8;
   public static final int CLIENT_IDENTITY = 0;
   public static final int SPECIFIED_IDENTITY = 1;
   public static final int SYSTEM_IDENTITY = 2;
   private int transactionAttribute = 3;
   private int isolationLevel = 2;
   private int runAsMode = 1;
   private Identity runAsIdentity;
   private transient Method method;
   private SerializableMethod serializableMethod;

   public ControlDescriptor() {
   }

   public ControlDescriptor(Method var1) {
      this.method = var1;
   }

   public Method getMethod() {
      return this.method;
   }

   public void setMethod(Method var1) {
      this.method = var1;
   }

   boolean isMethodLevel() {
      return this.method != null;
   }

   public int getTransactionAttribute() {
      return this.transactionAttribute;
   }

   public void setTransactionAttribute(int var1) {
      switch (var1) {
         case 0:
         case 1:
         case 2:
         case 4:
         case 5:
            this.transactionAttribute = var1;
            return;
         case 3:
         default:
            throw new IllegalArgumentException("Illegal value of transactionAttribute");
      }
   }

   public int getIsolationLevel() {
      return this.isolationLevel;
   }

   public void setIsolationLevel(int var1) {
      switch (var1) {
         case 1:
         case 2:
         case 4:
         case 8:
            this.isolationLevel = var1;
            return;
         default:
            throw new IllegalArgumentException("Illegal value of isolationLevel");
      }
   }

   public int getRunAsMode() {
      return this.runAsMode;
   }

   public void setRunAsMode(int var1) {
      switch (var1) {
         case 0:
         case 1:
         case 2:
            this.runAsMode = var1;
            return;
         default:
            throw new IllegalArgumentException("Illegal value of runAsMode");
      }
   }

   public Identity getRunAsIdentity() {
      return this.runAsIdentity;
   }

   public void setRunAsIdentity(Identity var1) {
      this.runAsIdentity = var1;
   }

   private void writeObject(ObjectOutputStream var1) throws IOException {
      this.serializableMethod = new SerializableMethod(this.method);
      var1.defaultWriteObject();
   }

   private void readObject(ObjectInputStream var1) throws IOException, ClassNotFoundException {
      var1.defaultReadObject();
      this.method = this.serializableMethod.getMethod();
   }
}
