package javax.naming.directory;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.OperationNotSupportedException;

public class BasicAttribute implements Attribute {
   protected String attrID;
   protected transient Vector values;

   public Object clone() {
      BasicAttribute var1;
      try {
         var1 = (BasicAttribute)super.clone();
      } catch (CloneNotSupportedException var2) {
         var1 = new BasicAttribute(this.attrID);
      }

      var1.values = (Vector)this.values.clone();
      return var1;
   }

   public boolean equals(Object var1) {
      if (var1 != null && var1 instanceof Attribute) {
         Attribute var2 = (Attribute)var1;
         if (this.attrID.equals(var2.getID()) && this.size() == var2.size()) {
            try {
               NamingEnumeration var3 = var2.getAll();

               while(var3.hasMoreElements()) {
                  if (this.find(var3.nextElement()) < 0) {
                     return false;
                  }
               }

               return true;
            } catch (NamingException var4) {
               return false;
            }
         }
      }

      return false;
   }

   public String toString() {
      StringBuffer var1 = new StringBuffer(this.attrID + ": ");
      if (this.values.size() == 0) {
         var1.append("No values");
      } else {
         boolean var2 = true;

         for(Enumeration var3 = this.values.elements(); var3.hasMoreElements(); var2 = false) {
            if (!var2) {
               var1.append(", ");
            }

            var1.append(var3.nextElement());
         }
      }

      return var1.toString();
   }

   public BasicAttribute(String var1) {
      this.attrID = var1;
      this.values = new Vector();
   }

   public BasicAttribute(String var1, Object var2) {
      this.attrID = var1;
      this.values = new Vector();
      this.values.addElement(var2);
   }

   public NamingEnumeration getAll() throws NamingException {
      return new ValuesEnumImpl(this);
   }

   public Object get() throws NamingException {
      if (this.values.size() == 0) {
         throw new NoSuchElementException("Attribute " + this.getID() + " has no value");
      } else {
         return this.values.elementAt(0);
      }
   }

   public int size() {
      return this.values.size();
   }

   public String getID() {
      return this.attrID;
   }

   public boolean contains(Object var1) {
      return this.find(var1) >= 0;
   }

   private int find(Object var1) {
      if (var1 != null) {
         return this.values.indexOf(var1, 0);
      } else {
         int var2 = this.values.size();

         for(int var3 = 0; var3 < var2; ++var3) {
            if (this.values.elementAt(var3) == null) {
               return var3;
            }
         }

         return -1;
      }
   }

   public boolean add(Object var1) {
      if (this.find(var1) < 0) {
         this.values.addElement(var1);
         return true;
      } else {
         return false;
      }
   }

   public boolean remove(Object var1) {
      int var2 = this.find(var1);
      if (var2 >= 0) {
         this.values.removeElementAt(var2);
         return true;
      } else {
         return false;
      }
   }

   public void clear() {
      this.values.setSize(0);
   }

   public DirContext getAttributeSyntaxDefinition() throws NamingException {
      throw new OperationNotSupportedException("attribute syntax");
   }

   public DirContext getAttributeDefinition() throws NamingException {
      throw new OperationNotSupportedException("attribute definition");
   }

   private void writeObject(ObjectOutputStream var1) throws IOException {
      var1.defaultWriteObject();
      var1.writeInt(this.values.size());

      for(int var2 = 0; var2 < this.values.size(); ++var2) {
         var1.writeObject(this.values.elementAt(var2));
      }

   }

   private void readObject(ObjectInputStream var1) throws IOException, ClassNotFoundException {
      var1.defaultReadObject();
      int var2 = var1.readInt();
      this.values = new Vector(var2);

      while(true) {
         --var2;
         if (var2 < 0) {
            return;
         }

         this.values.addElement(var1.readObject());
      }
   }
}
