package javax.naming.directory;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.naming.NamingEnumeration;

public class BasicAttributes implements Attributes {
   private boolean ignoreCase;
   transient Hashtable attrs;

   public BasicAttributes() {
      this.ignoreCase = false;
      this.attrs = new Hashtable(11);
   }

   public BasicAttributes(boolean var1) {
      this.ignoreCase = false;
      this.attrs = new Hashtable(11);
      this.ignoreCase = var1;
   }

   public BasicAttributes(String var1, Object var2) {
      this();
      this.put(new BasicAttribute(var1, var2));
   }

   public BasicAttributes(String var1, Object var2, boolean var3) {
      this(var3);
      this.put(new BasicAttribute(var1, var2));
   }

   public Object clone() {
      BasicAttributes var1;
      try {
         var1 = (BasicAttributes)super.clone();
      } catch (CloneNotSupportedException var2) {
         var1 = new BasicAttributes(this.ignoreCase);
      }

      var1.attrs = (Hashtable)this.attrs.clone();
      return var1;
   }

   public boolean isCaseIgnored() {
      return this.ignoreCase;
   }

   public int size() {
      return this.attrs.size();
   }

   public Attribute get(String var1) {
      Attribute var2 = (Attribute)this.attrs.get(this.ignoreCase ? var1.toLowerCase() : var1);
      return var2;
   }

   public NamingEnumeration getAll() {
      return new AttrEnumImpl(this);
   }

   public NamingEnumeration getIDs() {
      return new IDEnumImpl(this);
   }

   public Attribute put(String var1, Object var2) {
      return this.put(new BasicAttribute(var1, var2));
   }

   public Attribute put(Attribute var1) {
      String var2 = var1.getID();
      if (this.ignoreCase) {
         var2 = var2.toLowerCase();
      }

      return (Attribute)this.attrs.put(var2, var1);
   }

   public Attribute remove(String var1) {
      String var2 = this.ignoreCase ? var1.toLowerCase() : var1;
      return (Attribute)this.attrs.remove(var2);
   }

   public String toString() {
      return this.attrs.size() == 0 ? "No attributes" : this.attrs.toString();
   }

   private void writeObject(ObjectOutputStream var1) throws IOException {
      var1.defaultWriteObject();
      var1.writeInt(this.attrs.size());
      Enumeration var2 = this.attrs.elements();

      while(var2.hasMoreElements()) {
         var1.writeObject(var2.nextElement());
      }

   }

   private void readObject(ObjectInputStream var1) throws IOException, ClassNotFoundException {
      var1.defaultReadObject();
      int var2 = var1.readInt();
      this.attrs = var2 >= 1 ? new Hashtable(var2 * 2) : new Hashtable(2);

      while(true) {
         --var2;
         if (var2 < 0) {
            return;
         }

         this.put((Attribute)var1.readObject());
      }
   }
}
