package javax.naming.spi;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.server.RMIClassLoader;
import java.util.Hashtable;
import java.util.StringTokenizer;
import javax.naming.CannotProceedException;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.NoInitialContextException;
import javax.naming.Reference;
import javax.naming.Referenceable;

public class NamingManager {
   static ObjectFactoryBuilder factory_builder = null;
   static Hashtable factories = new Hashtable(11);
   private static boolean debug = true;
   private static final String defaultPkgPrefix = "com.sun.jndi.url";
   static InitialContextFactoryBuilder initctx_factory_builder = null;

   NamingManager() {
   }

   public static synchronized void setObjectFactoryBuilder(ObjectFactoryBuilder var0) throws NamingException {
      if (factory_builder != null) {
         throw new IllegalStateException("ObjectFactoryBuilder already set");
      } else {
         SecurityManager var1 = System.getSecurityManager();
         if (var1 != null) {
            var1.checkSetFactory();
         }

         factory_builder = var0;
      }
   }

   private static ObjectFactory loadFactoryFromURL(String var0, String var1) throws Exception {
      if (System.getSecurityManager() == null) {
         throw new NamingException("Must install security manager before loading class from URL " + var1);
      } else {
         try {
            Class var2 = RMIClassLoader.loadClass(new URL(var1), var0);
            ObjectFactory var3 = (ObjectFactory)var2.newInstance();
            factories.put(var0, var3);
            return var3;
         } catch (MalformedURLException var4) {
            if (debug) {
               System.err.println("MalformedURLException when loading factory from URL: " + var1);
            }

            throw var4;
         } catch (ClassNotFoundException var5) {
            if (debug) {
               System.err.println("ClassNotFoundException when loading factory from URL: " + var0);
            }

            throw var5;
         } catch (IllegalAccessException var6) {
            if (debug) {
               System.err.println("IllegalAccessException when loading factory from URL: " + var0);
            }

            throw var6;
         } catch (InstantiationException var7) {
            if (debug) {
               System.err.println("InstantiationException when loading factory from URL: " + var0);
            }

            throw var7;
         }
      }
   }

   static ObjectFactory findObjectFactoryFromReference(Reference var0, String var1) throws Exception {
      Object var2 = null;
      ObjectFactory var5;
      if ((var5 = (ObjectFactory)factories.get(var1)) != null) {
         return var5;
      } else {
         try {
            var5 = (ObjectFactory)Class.forName(var1).newInstance();
         } catch (ClassNotFoundException var4) {
         }

         if (var5 == null) {
            String var3 = var0.getFactoryClassLocation();
            if (var3 != null) {
               var5 = loadFactoryFromURL(var1, var3);
            }
         }

         if (var5 != null) {
            factories.put(var1, var5);
         }

         return var5;
      }
   }

   static Object createObjectFromFactories(Object var0, Name var1, Context var2, Hashtable var3) throws Exception {
      String var4 = null;
      if (var3 == null || (var4 = (String)var3.get("java.naming.factory.object")) == null) {
         try {
            var4 = System.getProperty("java.naming.factory.object");
         } catch (SecurityException var9) {
         }
      }

      if (var4 == null) {
         return null;
      } else {
         Object var5 = null;
         Object var6 = null;
         StringTokenizer var8 = new StringTokenizer(var4, ":");

         while(var6 == null && var8.hasMoreTokens()) {
            String var7 = var8.nextToken();

            try {
               var11 = (ObjectFactory)Class.forName(var7).newInstance();
            } catch (ClassNotFoundException var10) {
               continue;
            }

            var6 = var11.getObjectInstance(var0, var1, var2, var3);
         }

         return var6;
      }
   }

   static String getURLScheme(String var0) {
      int var1 = var0.indexOf(58);
      int var2 = var0.indexOf(47);
      return var1 <= 0 || var2 != -1 && var1 >= var2 ? null : var0.substring(0, var1);
   }

   public static synchronized Object getObjectInstance(Object var0, Name var1, Context var2, Hashtable var3) throws Exception {
      if (factory_builder != null) {
         ObjectFactory var11 = factory_builder.createObjectFactory(var0, var3);
         return var11.getObjectInstance(var0, var1, var2, var3);
      } else {
         Reference var5 = null;
         if (var0 instanceof Reference) {
            var5 = (Reference)var0;
         } else if (var0 instanceof Referenceable) {
            var5 = ((Referenceable)var0).getReference();
         }

         String var6;
         if (var5 != null && (var6 = var5.getFactoryClassName()) != null) {
            ObjectFactory var4 = findObjectFactoryFromReference(var5, var6);
            return var4 != null ? var4.getObjectInstance(var5, var1, var2, var3) : var0;
         } else {
            Object var7 = null;
            if (var0 instanceof String) {
               String var8 = (String)var0;
               String var9 = getURLScheme(var8);
               if (var9 != null) {
                  Context var12 = getURLContext(var9, var0, var3);
                  if (var12 != null) {
                     return var12;
                  }
               }
            }

            if (var0 instanceof String[]) {
               String[] var15 = (String[])var0;

               for(int var16 = 0; var16 < var15.length; ++var16) {
                  String var10 = getURLScheme(var15[var16]);
                  if (var10 != null) {
                     Context var13 = getURLContext(var10, var0, var3);
                     if (var13 != null) {
                        return var13;
                     }
                  }
               }
            }

            var7 = createObjectFromFactories(var0, var1, var2, var3);
            return var7 == null ? var0 : var7;
         }
      }
   }

   static Context getContext(Object var0, Name var1, Context var2, Hashtable var3) throws NamingException {
      if (var0 instanceof Context) {
         return (Context)var0;
      } else {
         Object var4;
         try {
            var4 = getObjectInstance(var0, var1, var2, var3);
         } catch (NamingException var7) {
            throw var7;
         } catch (Exception var8) {
            NamingException var6 = new NamingException();
            var6.setRootCause(var8);
            throw var6;
         }

         return var4 instanceof Context ? (Context)var4 : null;
      }
   }

   static Resolver getResolver(Object var0, Name var1, Context var2, Hashtable var3) throws NamingException {
      if (var0 instanceof Resolver) {
         return (Resolver)var0;
      } else {
         Object var4;
         try {
            var4 = getObjectInstance(var0, var1, var2, var3);
         } catch (NamingException var7) {
            throw var7;
         } catch (Exception var8) {
            NamingException var6 = new NamingException();
            var6.setRootCause(var8);
            throw var6;
         }

         return var4 instanceof Resolver ? (Resolver)var4 : null;
      }
   }

   public static Context getURLContext(String var0, Hashtable var1) throws NamingException {
      return getURLContext(var0, (Object)null, var1);
   }

   private static Context getURLContext(String var0, Object var1, Hashtable var2) throws NamingException {
      ObjectFactory var3 = null;
      String var4 = var0 + "URLContextFactory";
      String var5 = getPkgPrefixes(var2);
      StringTokenizer var6 = new StringTokenizer(var5, ":");

      while(var3 == null && var6.hasMoreTokens()) {
         String var7 = var6.nextToken().trim();
         String var8 = var7 + "." + var0 + "." + var4;
         if ((var3 = (ObjectFactory)factories.get(var8)) == null) {
            try {
               var3 = (ObjectFactory)Class.forName(var8).newInstance();
               factories.put(var8, var3);
            } catch (ClassNotFoundException var11) {
            } catch (Exception var12) {
               NamingException var10 = new NamingException("Cannot get URL Context for " + var0);
               var10.setRootCause(var12);
               throw var10;
            }
         }
      }

      if (var3 == null) {
         return null;
      } else {
         Object var14;
         try {
            var14 = var3.getObjectInstance(var1, (Name)null, (Context)null, var2);
         } catch (Exception var13) {
            if (!(var13 instanceof NamingException)) {
               NamingException var9 = new NamingException();
               var9.setRootCause(var13);
               throw var9;
            }

            throw (NamingException)var13;
         }

         return var14 instanceof Context ? (Context)var14 : null;
      }
   }

   private static String getPkgPrefixes(Hashtable var0) {
      String var1 = null;
      if (var0 != null) {
         var1 = (String)var0.get("java.naming.factory.url.pkgs");
      }

      if (var1 == null) {
         try {
            var1 = System.getProperty("java.naming.factory.url.pkgs", (String)null);
         } catch (SecurityException var2) {
         }
      }

      if (var1 != null) {
         var1 = var1 + ":com.sun.jndi.url";
      } else {
         var1 = "com.sun.jndi.url";
      }

      return var1;
   }

   static InitialContextFactory getDefaultInitialContextFactory(Hashtable var0) throws NamingException {
      Object var1 = null;
      String var2 = null;
      if (var0 == null || (var2 = (String)var0.get("java.naming.factory.initial")) == null) {
         try {
            var2 = System.getProperty("java.naming.factory.initial");
         } catch (SecurityException var6) {
         }
      }

      if (var2 == null) {
         NoInitialContextException var3 = new NoInitialContextException("Need to specify class name in environment or system property: java.naming.factory.initial");
         throw var3;
      } else {
         try {
            InitialContextFactory var7 = (InitialContextFactory)Class.forName(var2).newInstance();
            return var7;
         } catch (Exception var5) {
            NoInitialContextException var4 = new NoInitialContextException("Cannot instantiate class: " + var2);
            ((NamingException)var4).setRootCause(var5);
            throw var4;
         }
      }
   }

   public static synchronized Context getInitialContext(Hashtable var0) throws NamingException {
      Object var1 = null;
      InitialContextFactory var2;
      if (initctx_factory_builder != null) {
         var2 = initctx_factory_builder.createInitialContextFactory(var0);
      } else {
         var2 = getDefaultInitialContextFactory(var0);
      }

      return var2.getInitialContext(var0);
   }

   public static synchronized void setInitialContextFactoryBuilder(InitialContextFactoryBuilder var0) throws NamingException {
      if (initctx_factory_builder != null) {
         throw new IllegalStateException("InitialContextFactoryBuilder already set");
      } else {
         SecurityManager var1 = System.getSecurityManager();
         if (var1 != null) {
            var1.checkSetFactory();
         }

         initctx_factory_builder = var0;
      }
   }

   public static synchronized boolean hasInitialContextFactoryBuilder() {
      return initctx_factory_builder != null;
   }

   public static Context getContinuationContext(CannotProceedException var0) throws NamingException {
      ContinuationContext var1 = new ContinuationContext(var0);
      return var1.getTargetContext();
   }
}
