package oracle.jdbc.dbaccess;

import java.io.PrintStream;
import java.sql.SQLException;

public class DBDataSet {
   public static final boolean DEBUG = false;
   public DBData[] data;
   public DBType[] types;
   public int array_depth;

   public DBDataSet(int depth) {
      this.array_depth = depth;
      this.data = null;
      this.types = null;
   }

   public DBDataSet() {
      this.array_depth = 0;
      this.data = null;
      this.types = null;
   }

   public DBDataSet(int col_num, int row_num) {
      this.array_depth = row_num;
      this.data = new DBData[col_num];
      this.types = new DBType[col_num];

      for(int col_index = 0; col_index < col_num; ++col_index) {
         this.data[col_index] = new DBData(this.array_depth);
      }

   }

   public void createArrayFromTypes(int array_depth) throws SQLException {
      if (this.types == null) {
         DBError.check_error(-21, "createArrayFromTypes");
      }

      if (this.types.length == 0) {
         DBError.check_error(-21, "createArrayFromTypes");
      }

      this.array_depth = array_depth;
      int col_num = this.types.length;
      this.data = new DBData[col_num];

      for(int col_index = 0; col_index < col_num; ++col_index) {
         this.data[col_index] = new DBData(array_depth);
      }

   }

   public DBDataSet(DBType[] dbtypes, DBData[] data) {
      this.types = dbtypes;
      this.data = data;
      this.array_depth = data[0].items.length;
   }

   public void needArrayAndBuffers(int index, DBType dbtype) {
      this.needArray(index, dbtype);
      this.allocItemsAndBuffers(index, dbtype);
   }

   public void needArray(int index, DBType dbtype) {
      if (this.types == null) {
         this.data = new DBData[index + 1];
         this.types = new DBType[index + 1];
      } else if (this.types != null && this.types.length < index + 1) {
         DBData[] prev_data = this.data;
         DBType[] prev_types = this.types;
         this.data = new DBData[index + 1];
         this.types = new DBType[index + 1];

         for(int col_index = 0; col_index < prev_data.length; ++col_index) {
            this.data[col_index] = prev_data[col_index];
            this.types[col_index] = prev_types[col_index];
         }
      }

      this.types[index] = dbtype;
      if (this.data[index] == null) {
         this.data[index] = new DBData(this.array_depth);
      } else if (this.data[index].items.length < this.array_depth) {
         DBItem[] prev_items = this.data[index].items;
         this.data[index].items = new DBItem[this.array_depth];
         System.arraycopy(prev_items, 0, this.data[index].items, 0, prev_items.length);
      }

   }

   public void allocItemsAndBuffers(int index, DBType dbtype) {
      if (!dbtype.is_stream) {
         for(int row_index = 0; row_index < this.data[index].items.length; ++row_index) {
            this.data[index].items[row_index] = new DBItem(dbtype.max_length);
            this.data[index].items[row_index].allocBuffer(dbtype.max_length);
         }
      } else {
         for(int row_index = 0; row_index < this.data[index].items.length; ++row_index) {
            this.data[index].items[row_index] = new DBItem(dbtype.max_length);
         }
      }

   }

   public void setItem(DBItem item, int col_index, int row_index) throws SQLException {
      if (this.data[col_index].items == null) {
         DBError.check_error(-44, "setItem");
      }

      if (row_index >= this.data[col_index].items.length) {
         DBError.check_error(-45, "setItem");
      }

      this.data[col_index].items[row_index] = item;
   }

   public void setRowItems(int column_index, DBItem item) {
      for(int row_index = 0; row_index < this.array_depth; ++row_index) {
         DBItem item_copy = new DBItem(item);
         this.data[column_index].items[row_index] = item_copy;
      }

   }

   public DBData getData(int index) {
      return this.data.length < index ? null : this.data[index];
   }

   public byte[] getBytes(int col_index, int row_index) throws SQLException {
      return this.data.length < col_index ? null : this.data[col_index].getItem(row_index).getBytes();
   }

   public boolean isNull(int index, int rank) throws SQLException {
      return this.data[index].getItem(rank).is_null;
   }

   public static DBDataSet makeEmptyDeptDataSet(int row_num) throws SQLException {
      DBDataSet dbset = new DBDataSet(3, row_num);
      DBType[] dbtypes = new DBType[]{new DBType(6, 22), new DBType(1, 14), new DBType(1, 13)};

      for(int col_index = 0; col_index < 3; ++col_index) {
         dbset.setType(col_index, dbtypes[col_index]);
      }

      for(int row_index = 0; row_index < row_num; ++row_index) {
         dbset.getData(0).getItem(row_index).defineData(22);
         dbset.getData(1).getItem(row_index).defineData(14);
         dbset.getData(2).getItem(row_index).defineData(13);
      }

      return dbset;
   }

   public static DBDataSet makeDefinesDataSet(DBColumn[] dbdescription, int row_prefetch) throws SQLException {
      int column_num = dbdescription.length;
      DBDataSet defines = new DBDataSet(column_num, row_prefetch);

      for(int column_index = 0; column_index < column_num; ++column_index) {
         DBColumn column_info = dbdescription[column_index];
         int col_type = column_info.getColumnType();
         defines.setType(column_index, new DBType(col_type));
         DBData col_data = defines.getData(column_index);

         for(int row_index = 0; row_index < row_prefetch; ++row_index) {
            DBItem item = col_data.getItem(row_index);
            item.defineData(column_info.getColumnSize());
         }
      }

      return defines;
   }

   public static DBDataSet fillInDataSet(DBDataSet dbset) throws SQLException {
      byte[] p123 = new byte[]{3, -62, 2, 24};
      String str1 = "Thin JDBC Group";
      String str2 = "JavaLand";
      String str3 = "TTC Java Group";
      String str4 = "JavaLand as well";
      byte[] bytes_str1 = new byte[str1.length()];
      byte[] bytes_str2 = new byte[str2.length()];
      str1.getBytes(0, str1.length(), bytes_str1, 0);
      str2.getBytes(0, str2.length(), bytes_str2, 0);
      byte[] bytes_str3 = new byte[str3.length()];
      byte[] bytes_str4 = new byte[str4.length()];
      str3.getBytes(0, str3.length(), bytes_str3, 0);
      str4.getBytes(0, str4.length(), bytes_str4, 0);
      dbset.getData(0).getItem(0).copyFixedSizeData(false, p123);
      dbset.getData(1).getItem(0).copyVariableSizeData(false, bytes_str1);
      dbset.getData(2).getItem(0).copyVariableSizeData(false, bytes_str2);
      dbset.getData(0).getItem(1).copyFixedSizeData(false, p123);
      dbset.getData(1).getItem(1).copyVariableSizeData(false, bytes_str3);
      dbset.getData(2).getItem(1).copyVariableSizeData(false, bytes_str4);
      return dbset;
   }

   public DBType getType(int index) {
      return this.types != null && index >= 0 && index < this.types.length ? this.types[index] : null;
   }

   public void setType(int index, DBType dbtype) {
      if (this.types != null && index >= 0 && index < this.types.length) {
         this.types[index] = dbtype;
      }

   }

   public void print(PrintStream p) throws SQLException {
      p.println("depth = " + this.array_depth);
      if (this.data == null) {
         System.out.println("data is null");
      } else {
         for(int col_index = 0; col_index < this.data.length; ++col_index) {
            p.print("  column " + col_index + ": ");
            if (this.data[col_index] == null) {
               System.out.println(" null data");
            } else {
               p.print("type " + this.getType(col_index).type + ", ");
               p.println("maxlength " + this.getType(col_index).max_length);

               for(int row_index = 0; row_index < this.array_depth; ++row_index) {
                  DBItem item = this.data[col_index].getItem(row_index);
                  p.print("    row " + row_index + ": ");
                  if (item == null) {
                     System.out.println("null item");
                  } else {
                     p.print("data_size = " + item.data_size + ", ");
                     p.print("original_size = " + item.original_size + ", ");
                     p.print("is_null = " + item.is_null + ", ");
                     p.println("is_stream = " + this.types[col_index].is_stream);
                     if (item.byte_value != null) {
                        p.print("      byte_value: length " + item.byte_value.length + ", ");
                        p.println("bytes " + item.byte_value.length);
                        p.print("      ");
                        item.printBytes();
                     }

                     if (item.stream_value != null) {
                        p.println("      stream " + item.stream_value);
                     }
                  }
               }
            }
         }

      }
   }
}
