package oracle.jdbc.dbaccess;

import java.io.InputStream;

public class DBItem {
   public byte[] byte_value;
   public InputStream stream_value;
   public int data_size;
   public int original_size;
   public int max_out_size;
   public boolean is_null;

   public DBItem() {
      this.is_null = false;
      this.byte_value = null;
      this.stream_value = null;
      this.data_size = 0;
      this.original_size = 0;
      this.max_out_size = 0;
   }

   public DBItem(int max_length) {
      this.is_null = false;
      this.byte_value = null;
      this.stream_value = null;
      this.data_size = 0;
      this.original_size = 0;
      this.max_out_size = max_length;
   }

   public DBItem(DBItem dbitem) {
      this.is_null = dbitem.is_null;
      this.byte_value = new byte[dbitem.byte_value.length];
      this.stream_value = null;
      this.data_size = dbitem.data_size;
      this.original_size = dbitem.original_size;
      this.max_out_size = dbitem.max_out_size;
   }

   public void allocBuffer(int nbytes) {
      if (this.byte_value == null || this.byte_value.length < nbytes) {
         this.byte_value = new byte[nbytes];
      }

   }

   public void setArrayData(boolean null_flag, byte[] array_value) {
      this.is_null = null_flag;
      if (array_value != null) {
         this.data_size = array_value.length;
         this.byte_value = array_value;
      } else {
         this.byte_value = new byte[0];
         this.data_size = 0;
      }

   }

   public void copyFixedSizeData(boolean null_flag, byte[] array_value) {
      this.is_null = null_flag;
      System.arraycopy(array_value, 0, this.byte_value, 0, array_value.length);
      this.data_size = array_value.length;
   }

   public void copyVariableSizeData(boolean null_flag, byte[] array) {
      this.is_null = null_flag;
      System.arraycopy(array, 0, this.byte_value, 0, array.length);
      this.data_size = array.length;
   }

   public void setStreamData(boolean null_flag, int data_size, InputStream stream) {
      this.is_null = null_flag;
      this.stream_value = stream;
      this.data_size = data_size;
      this.max_out_size = data_size;
   }

   public void setNull() {
      this.is_null = true;
      this.data_size = 0;
      this.allocBuffer(0);
   }

   public void defineStreamData(InputStream stream) {
      this.data_size = 0;
      this.stream_value = stream;
   }

   public void defineData(int max_size) {
      this.allocBuffer(max_size);
      this.max_out_size = max_size;
      this.data_size = 0;
      this.stream_value = null;
   }

   public byte[] getBytes() {
      return this.byte_value;
   }

   public void printBytes() {
      int index = 0;
      if (this.byte_value != null) {
         while(index < this.byte_value.length) {
            System.out.print(this.byte_value[index] + " ");
            ++index;
         }

         System.out.println();
      }

   }
}
