package oracle.jdbc.driver;

import java.io.IOException;
import java.io.InputStream;

public class OracleBufferedStream extends InputStream {
   byte[] buf;
   int pos = 0;
   int count = 0;
   boolean closed = false;
   int chunkSize;

   public OracleBufferedStream(int chunkSize) {
      this.chunkSize = chunkSize;
      this.buf = new byte[chunkSize];
   }

   public void close() throws IOException {
      this.closed = true;
   }

   public boolean needBytes() throws IOException {
      throw new IOException("You should not call this method");
   }

   public int flushBytes(int n) {
      int availableLength = n > this.count - this.pos ? this.count - this.pos : n;
      this.pos += availableLength;
      return availableLength;
   }

   public int writeBytes(byte[] destbuf, int offset, int length) {
      int availableLength = length > this.count - this.pos ? this.count - this.pos : length;
      System.arraycopy(this.buf, this.pos, destbuf, offset, availableLength);
      this.pos += availableLength;
      return availableLength;
   }

   public synchronized int read() throws IOException {
      if (!this.closed && !this.isNull()) {
         return this.needBytes() ? this.buf[this.pos++] & 255 : -1;
      } else {
         return -1;
      }
   }

   public int read(byte[] b) throws IOException {
      return this.read(b, 0, b.length);
   }

   public synchronized int read(byte[] destbuf, int offset, int length) throws IOException {
      if (!this.closed && !this.isNull()) {
         int end;
         if (length > destbuf.length) {
            end = offset + destbuf.length;
         } else {
            end = offset + length;
         }

         if (!this.needBytes()) {
            return -1;
         } else {
            int start;
            for(start = offset + this.writeBytes(destbuf, offset, end - offset); start < end && this.needBytes(); start += this.writeBytes(destbuf, start, end - start)) {
            }

            return start - offset;
         }
      } else {
         return -1;
      }
   }

   public int available() throws IOException {
      return !this.closed && !this.isNull() ? this.count - this.pos : 0;
   }

   public boolean isNull() throws IOException {
      return false;
   }

   public synchronized void mark(int readlimit) {
   }

   public synchronized void reset() throws IOException {
      throw new IOException("mark/reset not supported");
   }

   public boolean markSupported() {
      return false;
   }

   public long skip(int n) throws IOException {
      int start = 0;
      int end = n;
      if (!this.closed && !this.isNull()) {
         if (!this.needBytes()) {
            return -1L;
         } else {
            while(start < end && this.needBytes()) {
               start += this.flushBytes(end - start);
            }

            return (long)start;
         }
      } else {
         return -1L;
      }
   }
}
