package oracle.jdbc.driver;

import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import oracle.jdbc.dbaccess.DBDataSet;
import oracle.jdbc.dbaccess.DBItem;
import oracle.jdbc.dbaccess.DBType;

public class OracleCallableStatement extends OraclePreparedStatement implements CallableStatement {
   public OracleCallableStatement(OracleConnection c, String s, int batch_value, int row_prefetch_value) throws SQLException {
      super(c, s, batch_value, row_prefetch_value);
      super.binds_out = new DBDataSet(super.batch);
   }

   public void registerOutParameter(int parameterIndex, int sqlType) throws SQLException {
      this.registerOutParameter(parameterIndex, sqlType, 0);
   }

   public synchronized void registerOutParameter(int parameterIndex, int sqlType, int scale) throws SQLException {
      OracleConnection var4 = super.connection;
      synchronized(var4){}

      try {
         int index = parameterIndex - 1;
         int internal_type = ((OracleStatement)this).get_internal_type(sqlType);
         ((OraclePreparedStatement)this).checkBindTypes(false, index, internal_type);
         ((OraclePreparedStatement)this).setExecuteBatch(1);
         DBType dbtype;
         if (internal_type == 8) {
            dbtype = new DBType(1);
            dbtype.max_length = 32767;
         } else if (internal_type == 24) {
            dbtype = new DBType(23);
            dbtype.max_length = 32767;
         } else if (internal_type == 102) {
            ((OraclePreparedStatement)this).setNull(parameterIndex, -10);
            int cursor_size = super.connection.db_access.getRefCursorBytesSize();
            dbtype = new DBType(102, cursor_size);
         } else {
            dbtype = new DBType(internal_type);
         }

         super.binds_out.needArrayAndBuffers(index, dbtype);
      } catch (Throwable var11) {
         throw var11;
      }

   }

   public boolean wasNull() throws SQLException {
      return ((OracleStatement)this).wasNullValue();
   }

   public String getString(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getStringValue(false, parameterIndex);
   }

   public OracleRowid getRowid(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getRowidValue(false, parameterIndex);
   }

   public OracleBlob getBlob(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getBlobValue(false, parameterIndex);
   }

   public OracleClob getClob(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getClobValue(false, parameterIndex);
   }

   public OracleBfile getBfile(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getBfileValue(false, parameterIndex);
   }

   public OracleCfile getCfile(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getCfileValue(false, parameterIndex);
   }

   public boolean getBoolean(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getBooleanValue(false, parameterIndex);
   }

   public byte getByte(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getByteValue(false, parameterIndex);
   }

   public short getShort(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getShortValue(false, parameterIndex);
   }

   public int getInt(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getIntValue(false, parameterIndex);
   }

   public long getLong(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getLongValue(false, parameterIndex);
   }

   public float getFloat(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getFloatValue(false, parameterIndex);
   }

   public double getDouble(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getDoubleValue(false, parameterIndex);
   }

   public BigDecimal getBigDecimal(int parameterIndex, int scale) throws SQLException {
      return ((OracleStatement)this).getBigDecimalValue(false, parameterIndex, scale);
   }

   public byte[] getBytes(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getBytesValue(false, parameterIndex);
   }

   public Date getDate(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getDateValue(false, parameterIndex);
   }

   public Time getTime(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getTimeValue(false, parameterIndex);
   }

   public Timestamp getTimestamp(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getTimestampValue(false, parameterIndex);
   }

   public InputStream getAsciiStream(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getAsciiStreamValue(false, parameterIndex);
   }

   public InputStream getUnicodeStream(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getUnicodeStreamValue(false, parameterIndex);
   }

   public InputStream getBinaryStream(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getBinaryStreamValue(false, parameterIndex);
   }

   public Object getObject(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getObjectValue(false, parameterIndex);
   }

   public ResultSet getCursor(int parameterIndex) throws SQLException {
      return ((OracleStatement)this).getCursorValue(false, parameterIndex);
   }

   public synchronized void clearParameters() throws SQLException {
      if (super.binds_out.data != null && super.binds_out.data.length != 0) {
         for(int col_index = 0; col_index < super.binds_out.types.length; ++col_index) {
            super.binds_out.setItem((DBItem)null, col_index, super.current_rank);
         }
      }

      super.clearParameters();
   }
}
