package oracle.jdbc.driver;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBError;

public class OracleConversionInputStream extends OracleBufferedStream {
   static final boolean DEBUG = false;
   int conversion;
   InputStream istream;
   DBConversion converter;
   byte[] convbuf;
   char[] java_chars;

   public OracleConversionInputStream(DBConversion conv_object, InputStream istream, int conversion) {
      super(4096);
      this.istream = istream;
      this.conversion = conversion;
      this.converter = conv_object;
      int requestLength = this.converter.requestLength(super.chunkSize, conversion);
      this.convbuf = new byte[requestLength];
      this.java_chars = new char[requestLength];
   }

   public boolean needBytes() throws IOException {
      if (super.closed) {
         return false;
      } else if (super.pos < super.count) {
         return true;
      } else {
         int bytes_read = 0;
         bytes_read = this.istream.read(this.convbuf);
         if (bytes_read == -1) {
            ((OracleBufferedStream)this).close();
            return false;
         } else {
            try {
               switch (this.conversion) {
                  case 0:
                     int chars_read = this.converter.CHARBytesToJavaChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToAsciiBytes(this.java_chars, chars_read, super.buf);
                     break;
                  case 1:
                     int chars_read = this.converter.CHARBytesToJavaChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToUnicodeBytes(this.java_chars, chars_read, super.buf);
                     break;
                  case 2:
                     int chars_read = this.converter.RAWBytesToHexChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToAsciiBytes(this.java_chars, chars_read, super.buf);
                     break;
                  case 3:
                     int chars_read = this.converter.RAWBytesToHexChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToUnicodeBytes(this.java_chars, chars_read, super.buf);
                     break;
                  case 4:
                     int chars_read = this.converter.UnicodeBytesToJavaChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToCHARBytes(this.java_chars, chars_read, super.buf);
                     break;
                  case 5:
                     int chars_read = this.converter.AsciiBytesToJavaChars(this.convbuf, bytes_read, this.java_chars);
                     super.count = this.converter.JavaCharsToCHARBytes(this.java_chars, chars_read, super.buf);
                     break;
                  default:
                     System.arraycopy(this.convbuf, 0, super.buf, 0, bytes_read);
                     super.count = bytes_read;
               }
            } catch (SQLException e) {
               DBError.SQLToIOException(e);
            }

            super.pos = 0;
            return true;
         }
      }
   }
}
