package oracle.jdbc.driver;

import java.io.IOException;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class OracleDatabaseMetaData implements DatabaseMetaData {
   OracleConnection connection;
   int procedureResultUnknown;
   int procedureNoResult = 1;
   int procedureReturnsResult = 2;
   int procedureColumnUnknown;
   int procedureColumnIn = 1;
   int procedureColumnInOut = 2;
   int procedureColumnOut = 4;
   int procedureColumnReturn = 5;
   int procedureColumnResult = 3;
   int procedureNoNulls;
   int procedureNullable = 1;
   int procedureNullableUnknown = 2;
   int columnNoNulls;
   int columnNullable = 1;
   int columnNullableUnknown = 2;
   static final int bestRowTemporary = 0;
   static final int bestRowTransaction = 1;
   static final int bestRowSession = 2;
   static final int bestRowUnknown = 0;
   static final int bestRowNotPseudo = 1;
   static final int bestRowPseudo = 2;
   int versionColumnUnknown;
   int versionColumnNotPseudo = 1;
   int versionColumnPseudo = 2;
   int importedKeyCascade;
   int importedKeyRestrict = 1;
   int importedKeySetNull = 2;
   int typeNoNulls;
   int typeNullable = 1;
   int typeNullableUnknown = 2;
   int typePredNone;
   int typePredChar = 1;
   int typePredBasic = 2;
   int typeSearchable = 3;
   short tableIndexStatistic;
   short tableIndexClustered = 1;
   short tableIndexHashed = 2;
   short tableIndexOther = 3;

   public OracleDatabaseMetaData(OracleConnection conn) {
      this.connection = conn;
   }

   public boolean allProceduresAreCallable() throws SQLException {
      this.connection.trace("allProceduresAreCallable");
      return false;
   }

   public boolean allTablesAreSelectable() throws SQLException {
      this.connection.trace("allTablesAreSelectable");
      return false;
   }

   public String getURL() throws SQLException {
      return this.connection.url;
   }

   public String getUserName() throws SQLException {
      this.connection.trace("getUserName");
      return this.connection.user;
   }

   public boolean isReadOnly() throws SQLException {
      this.connection.trace("isReadOnly");
      return false;
   }

   public boolean nullsAreSortedHigh() throws SQLException {
      this.connection.trace("nullsAreSortedHigh");
      return false;
   }

   public boolean nullsAreSortedLow() throws SQLException {
      this.connection.trace("nullsAreSortedLow");
      return true;
   }

   public boolean nullsAreSortedAtStart() throws SQLException {
      this.connection.trace("nullsAreSortedAtStart");
      return false;
   }

   public boolean nullsAreSortedAtEnd() throws SQLException {
      this.connection.trace("nullsAreSortedAtEnd");
      return false;
   }

   public String getDatabaseProductName() throws SQLException {
      return "Oracle";
   }

   public String getDatabaseProductVersion() throws SQLException {
      this.connection.trace("getDatabaseProductVersion");
      int[] error = new int[1];

      try {
         this.connection.needLine();
         byte[] version = this.connection.db_access.getVersion();
         return this.connection.conversion.CharBytesToString(version, version.length);
      } catch (IOException var3) {
         return null;
      }
   }

   public String getDriverName() throws SQLException {
      this.connection.trace("getDriverName");
      return "Oracle JDBC driver";
   }

   public String getDriverVersion() throws SQLException {
      this.connection.trace("getDriverVersion");
      return "7.3.3.1.3";
   }

   public int getDriverMajorVersion() {
      this.connection.trace("getDriverMajorVersion");
      return 7;
   }

   public int getDriverMinorVersion() {
      this.connection.trace("getDriverMinorVersion");
      return 3;
   }

   public boolean usesLocalFiles() throws SQLException {
      this.connection.trace("usesLocalFiles");
      return false;
   }

   public boolean usesLocalFilePerTable() throws SQLException {
      this.connection.trace("usesLocalFilePerTable");
      return false;
   }

   public boolean supportsMixedCaseIdentifiers() throws SQLException {
      this.connection.trace("supportsMixedCaseIdentifiers");
      return false;
   }

   public boolean storesUpperCaseIdentifiers() throws SQLException {
      this.connection.trace("storesUpperCaseIdentifiers");
      return true;
   }

   public boolean storesLowerCaseIdentifiers() throws SQLException {
      this.connection.trace("storesLowerCaseIdentifiers");
      return false;
   }

   public boolean storesMixedCaseIdentifiers() throws SQLException {
      this.connection.trace("storesMixedCaseIdentifiers");
      return false;
   }

   public boolean supportsMixedCaseQuotedIdentifiers() throws SQLException {
      this.connection.trace("supportsMixedCaseQuotedIdentifiers");
      return true;
   }

   public boolean storesUpperCaseQuotedIdentifiers() throws SQLException {
      this.connection.trace("storesUpperCaseQuotedIdentifiers");
      return false;
   }

   public boolean storesLowerCaseQuotedIdentifiers() throws SQLException {
      this.connection.trace("storesLowerCaseQuotedIdentifiers");
      return false;
   }

   public boolean storesMixedCaseQuotedIdentifiers() throws SQLException {
      this.connection.trace("storesMixedCaseQuotedIdentifiers");
      return true;
   }

   public String getIdentifierQuoteString() throws SQLException {
      this.connection.trace("getIdentifierQuoteString");
      return "\"";
   }

   public String getSQLKeywords() throws SQLException {
      this.connection.trace("getSQLKeywords");
      return "ACCESS, ADD, ALTER, AUDIT, CLUSTER, COLUMN, COMMENT, COMPRESS, CONNECT, DATE, DROP, EXCLUSIVE, FILE, IDENTIFIED, IMMEDIATE, INCREMENT, INDEX, INITIAL, INTERSECT, LEVEL, LOCK, LONG, MAXEXTENTS, MINUS, MODE, NOAUDIT, NOCOMPRESS, NOWAIT, NUMBER, OFFLINE, ONLINE, PCTFREE, PRIOR, all_PL_SQL_reserved_ words";
   }

   public String getNumericFunctions() throws SQLException {
      this.connection.trace("getNumericFunctions");
      return "ABS, CEIL, COS, COSH, EXP, FLOOR, LN, LOG, MOD, POWER, ROUND, SIGN, SIN, SINH, SQRT, TAN, TANH, TRUNC, AVG, COUNT, GLB, LUB, MAX, MIN, STDDEV, SUM, VARIANCE";
   }

   public String getStringFunctions() throws SQLException {
      this.connection.trace("getStringFunctions");
      return "CHR, INITCAP, LOWER, LPAD, LTRIM, NLS,_INITCAP, NLS,_LOWER, NLS,_UPPER, REPLACE, RPAD, RTRIM, SOUNDEX, SUBSTR, SUBSTRB, TRANSLATE, UPPER, ASCII, INSTR, INSTRB, LENGTH, LENGTHB, NLSSORT, CHARTOROWID, CONVERT, HEXTORAW, RAWTOHEX, ROWIDTOCHAR, TO_CHAR, TO_DATE, TO_LABEL, TO_MULTI_BYTE, TO_NUMBER, TO_SINGLE_BYTE";
   }

   public String getSystemFunctions() throws SQLException {
      this.connection.trace("getSystemFunctions");
      return "DUMP, GREATEST, GREATEST_LB, LEAST, LEAST_UB, NVL, UID, USER, USERENV, VSIZE";
   }

   public String getTimeDateFunctions() throws SQLException {
      this.connection.trace("getTimeDateFunctions");
      return "ADD_MONTHS, LAST_DAY, MONTHS_BETWEEN, NEW_TIME, NEXT_DAY, ROUND, SYSDATE, TRUNC";
   }

   public String getSearchStringEscape() throws SQLException {
      this.connection.trace("getSearchStringEscape");
      return "//";
   }

   public String getExtraNameCharacters() throws SQLException {
      this.connection.trace("getExtraNameCharacters");
      return "$#";
   }

   public boolean supportsAlterTableWithAddColumn() throws SQLException {
      this.connection.trace("supportsAlterTableWithAddColumn");
      return true;
   }

   public boolean supportsAlterTableWithDropColumn() throws SQLException {
      this.connection.trace("supportsAlterTableWithDropColumn");
      return false;
   }

   public boolean supportsColumnAliasing() throws SQLException {
      this.connection.trace("supportsColumnAliasing");
      return true;
   }

   public boolean nullPlusNonNullIsNull() throws SQLException {
      this.connection.trace("nullPlusNonNullIsNull");
      return true;
   }

   public boolean supportsConvert() throws SQLException {
      this.connection.trace("supportsConvert");
      return true;
   }

   public boolean supportsConvert(int fromType, int toType) throws SQLException {
      this.connection.trace("supportsConvert");
      if (fromType != toType && fromType != 1 && fromType != 12 && fromType != -1 && toType != 1 && toType != 12 && toType != -1) {
         switch (fromType) {
            case 2:
            case 3:
               return toType == 92 || toType == 93 || toType == 91;
            case 91:
            case 92:
            case 93:
               return toType == 2 || toType == 3;
            default:
               return false;
         }
      } else {
         return true;
      }
   }

   public boolean supportsTableCorrelationNames() throws SQLException {
      this.connection.trace("supportsTableCorrelationNames");
      return true;
   }

   public boolean supportsDifferentTableCorrelationNames() throws SQLException {
      this.connection.trace("supportsDifferentTableCorrelationNames");
      return true;
   }

   public boolean supportsExpressionsInOrderBy() throws SQLException {
      this.connection.trace("supportsExpressionsInOrderBy");
      return true;
   }

   public boolean supportsOrderByUnrelated() throws SQLException {
      this.connection.trace("supportsOrderByUnrelated");
      return true;
   }

   public boolean supportsGroupBy() throws SQLException {
      this.connection.trace("supportsGroupBy");
      return true;
   }

   public boolean supportsGroupByUnrelated() throws SQLException {
      this.connection.trace("supportsGroupByUnrelated");
      return true;
   }

   public boolean supportsGroupByBeyondSelect() throws SQLException {
      this.connection.trace("supportsGroupByBeyondSelect");
      return true;
   }

   public boolean supportsLikeEscapeClause() throws SQLException {
      this.connection.trace("supportsLikeEscapeClause");
      return true;
   }

   public boolean supportsMultipleResultSets() throws SQLException {
      this.connection.trace("supportsMultipleResultSets");
      return false;
   }

   public boolean supportsMultipleTransactions() throws SQLException {
      this.connection.trace("supportsMultipleTransactions");
      return true;
   }

   public boolean supportsNonNullableColumns() throws SQLException {
      this.connection.trace("supportsNonNullableColumns");
      return true;
   }

   public boolean supportsMinimumSQLGrammar() throws SQLException {
      this.connection.trace("supportsMinimumSQLGrammar");
      return true;
   }

   public boolean supportsCoreSQLGrammar() throws SQLException {
      this.connection.trace("supportsCoreSQLGrammar");
      return true;
   }

   public boolean supportsExtendedSQLGrammar() throws SQLException {
      this.connection.trace("supportsExtendedSQLGrammar");
      return true;
   }

   public boolean supportsANSI92EntryLevelSQL() throws SQLException {
      this.connection.trace("supportsANSI92EntryLevelSQL");
      return false;
   }

   public boolean supportsANSI92IntermediateSQL() throws SQLException {
      this.connection.trace("supportsANSI92IntermediateSQL");
      return false;
   }

   public boolean supportsANSI92FullSQL() throws SQLException {
      this.connection.trace("supportsANSI92FullSQL");
      return false;
   }

   public boolean supportsIntegrityEnhancementFacility() throws SQLException {
      this.connection.trace("supportsIntegrityEnhancementFacility");
      return true;
   }

   public boolean supportsOuterJoins() throws SQLException {
      this.connection.trace("supportsOuterJoins");
      return true;
   }

   public boolean supportsFullOuterJoins() throws SQLException {
      this.connection.trace("supportsFullOuterJoins");
      return true;
   }

   public boolean supportsLimitedOuterJoins() throws SQLException {
      this.connection.trace("supportsLimitedOuterJoins");
      return true;
   }

   public String getSchemaTerm() throws SQLException {
      this.connection.trace("getSchemaTerm");
      return "schema";
   }

   public String getProcedureTerm() throws SQLException {
      this.connection.trace("getProcedureTerm");
      return "procedure";
   }

   public String getCatalogTerm() throws SQLException {
      this.connection.trace("getCatalogTerm");
      return "";
   }

   public boolean isCatalogAtStart() throws SQLException {
      this.connection.trace("isCatalogAtStart");
      return false;
   }

   public String getCatalogSeparator() throws SQLException {
      this.connection.trace("getCatalogSeparator");
      return "";
   }

   public boolean supportsSchemasInDataManipulation() throws SQLException {
      this.connection.trace("supportsSchemasInDataManipulation");
      return true;
   }

   public boolean supportsSchemasInProcedureCalls() throws SQLException {
      this.connection.trace("supportsSchemasInProcedureCalls");
      return true;
   }

   public boolean supportsSchemasInTableDefinitions() throws SQLException {
      this.connection.trace("supportsSchemasInTableDefinitions");
      return true;
   }

   public boolean supportsSchemasInIndexDefinitions() throws SQLException {
      this.connection.trace("supportsSchemasInIndexDefinitions");
      return true;
   }

   public boolean supportsSchemasInPrivilegeDefinitions() throws SQLException {
      this.connection.trace("supportsSchemasInPrivilegeDefinitions");
      return true;
   }

   public boolean supportsCatalogsInDataManipulation() throws SQLException {
      this.connection.trace("supportsCatalogsInDataManipulation");
      return false;
   }

   public boolean supportsCatalogsInProcedureCalls() throws SQLException {
      this.connection.trace("supportsCatalogsInProcedureCalls");
      return false;
   }

   public boolean supportsCatalogsInTableDefinitions() throws SQLException {
      this.connection.trace("supportsCatalogsInTableDefinitions");
      return false;
   }

   public boolean supportsCatalogsInIndexDefinitions() throws SQLException {
      this.connection.trace("supportsCatalogsInIndexDefinitions");
      return false;
   }

   public boolean supportsCatalogsInPrivilegeDefinitions() throws SQLException {
      this.connection.trace("supportsCatalogsInPrivilegeDefinitions");
      return false;
   }

   public boolean supportsPositionedDelete() throws SQLException {
      this.connection.trace("supportsPositionedDelete");
      return true;
   }

   public boolean supportsPositionedUpdate() throws SQLException {
      this.connection.trace("supportsPositionedUpdate");
      return true;
   }

   public boolean supportsSelectForUpdate() throws SQLException {
      this.connection.trace("supportsSelectForUpdate");
      return true;
   }

   public boolean supportsStoredProcedures() throws SQLException {
      this.connection.trace("supportsStoredProcedures");
      return true;
   }

   public boolean supportsSubqueriesInComparisons() throws SQLException {
      this.connection.trace("supportsSubqueriesInComparisons");
      return true;
   }

   public boolean supportsSubqueriesInExists() throws SQLException {
      this.connection.trace("supportsSubqueriesInExists");
      return true;
   }

   public boolean supportsSubqueriesInIns() throws SQLException {
      this.connection.trace("supportsSubqueriesInIns");
      return true;
   }

   public boolean supportsSubqueriesInQuantifieds() throws SQLException {
      this.connection.trace("supportsSubqueriesInQuantifieds");
      return true;
   }

   public boolean supportsCorrelatedSubqueries() throws SQLException {
      this.connection.trace("supportsCorrelatedSubqueries");
      return true;
   }

   public boolean supportsUnion() throws SQLException {
      this.connection.trace("supportsUnion");
      return true;
   }

   public boolean supportsUnionAll() throws SQLException {
      this.connection.trace("supportsUnionAll");
      return true;
   }

   public boolean supportsOpenCursorsAcrossCommit() throws SQLException {
      this.connection.trace("supportsOpenCursorsAcrossCommit");
      return false;
   }

   public boolean supportsOpenCursorsAcrossRollback() throws SQLException {
      this.connection.trace("supportsOpenCursorsAcrossRollback");
      return false;
   }

   public boolean supportsOpenStatementsAcrossCommit() throws SQLException {
      this.connection.trace("supportsOpenStatementsAcrossCommit");
      return false;
   }

   public boolean supportsOpenStatementsAcrossRollback() throws SQLException {
      this.connection.trace("supportsOpenStatementsAcrossRollback");
      return false;
   }

   public int getMaxBinaryLiteralLength() throws SQLException {
      this.connection.trace("getMaxBinaryLiteralLength");
      return 1000;
   }

   public int getMaxCharLiteralLength() throws SQLException {
      this.connection.trace("getMaxCharLiteralLength");
      return 2000;
   }

   public int getMaxColumnNameLength() throws SQLException {
      this.connection.trace("getMaxColumnNameLength");
      return 30;
   }

   public int getMaxColumnsInGroupBy() throws SQLException {
      this.connection.trace("getMaxColumnsInGroupBy");
      return 0;
   }

   public int getMaxColumnsInIndex() throws SQLException {
      this.connection.trace("getMaxColumnsInIndex");
      return 16;
   }

   public int getMaxColumnsInOrderBy() throws SQLException {
      this.connection.trace("getMaxColumnsInOrderBy");
      return 0;
   }

   public int getMaxColumnsInSelect() throws SQLException {
      this.connection.trace("getMaxColumnsInSelect");
      return 0;
   }

   public int getMaxColumnsInTable() throws SQLException {
      this.connection.trace("getMaxColumnsInTable");
      return 254;
   }

   public int getMaxConnections() throws SQLException {
      this.connection.trace("getMaxConnections");
      return 0;
   }

   public int getMaxCursorNameLength() throws SQLException {
      this.connection.trace("getMaxCursorNameLength");
      return 0;
   }

   public int getMaxIndexLength() throws SQLException {
      this.connection.trace("getMaxIndexLength");
      return 0;
   }

   public int getMaxSchemaNameLength() throws SQLException {
      this.connection.trace("getMaxSchemaNameLength");
      return 30;
   }

   public int getMaxProcedureNameLength() throws SQLException {
      this.connection.trace("getMaxProcedureNameLength");
      return 30;
   }

   public int getMaxCatalogNameLength() throws SQLException {
      this.connection.trace("getMaxCatalogNameLength");
      return 0;
   }

   public int getMaxRowSize() throws SQLException {
      this.connection.trace("getMaxRowSize");
      return 0;
   }

   public boolean doesMaxRowSizeIncludeBlobs() throws SQLException {
      this.connection.trace("doesMaxRowSizeIncludeBlobs");
      return true;
   }

   public int getMaxStatementLength() throws SQLException {
      this.connection.trace("getMaxStatementLength");
      return 65535;
   }

   public int getMaxStatements() throws SQLException {
      this.connection.trace("getMaxStatements");
      return 0;
   }

   public int getMaxTableNameLength() throws SQLException {
      this.connection.trace("getMaxTableNameLength");
      return 30;
   }

   public int getMaxTablesInSelect() throws SQLException {
      this.connection.trace("getMaxTablesInSelect");
      return 0;
   }

   public int getMaxUserNameLength() throws SQLException {
      this.connection.trace("getMaxUserNameLength");
      return 30;
   }

   public int getDefaultTransactionIsolation() throws SQLException {
      this.connection.trace("getDefaultTransactionIsolation");
      return 1;
   }

   public boolean supportsTransactions() throws SQLException {
      this.connection.trace("supportsTransactions");
      return true;
   }

   public boolean supportsTransactionIsolationLevel(int level) throws SQLException {
      this.connection.trace("supportsTransactionIsolationLevel");
      return level == 1;
   }

   public boolean supportsDataDefinitionAndDataManipulationTransactions() throws SQLException {
      this.connection.trace("supportsDataDefinitionAndDataManipulationTransactions");
      return false;
   }

   public boolean supportsDataManipulationTransactionsOnly() throws SQLException {
      this.connection.trace("supportsDataManipulationTransactionsOnly");
      return true;
   }

   public boolean dataDefinitionCausesTransactionCommit() throws SQLException {
      this.connection.trace("dataDefinitionCausesTransactionCommit");
      return true;
   }

   public boolean dataDefinitionIgnoredInTransactions() throws SQLException {
      this.connection.trace("dataDefinitionIgnoredInTransactions");
      return false;
   }

   public synchronized ResultSet getProcedures(String catalog, String schemaPattern, String procedureNamePattern) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT NULL AS procedure_cat,\n       owner AS procedure_schem,\n       object_name AS procedure_name,\n       NULL,\n       NULL,\n       NULL,\n       NULL AS remarks,\n       DECODE (object_type, 'PROCEDURE', 1, 'FUNCTION', 2, 0) as procedure_type\nFROM all_objects\nWHERE (object_type = 'PROCEDURE' OR object_type = 'FUNCTION')\n  AND owner LIKE ? ESCAPE '/'\n  AND object_name LIKE ? ESCAPE '/'\nORDER BY procedure_schem, procedure_name\n");
      s.setString(1, schemaPattern == null ? "%" : schemaPattern);
      s.setString(2, procedureNamePattern == null ? "%" : procedureNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getProcedureColumns(String catalog, String schemaPattern, String procedureNamePattern, String columnNamePattern) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT NULL AS procedure_cat,\n       owner AS procedure_schem,\n       object_name AS procedure_name,\n       argument_name AS column_name,\n       DECODE (position, NULL, 3,\n               DECODE (in_out, 'IN', 1, 'OUT', 4, 'IN/OUT', 5, null))\n              AS column_type,\n       DECODE (data_type, 'CHAR', 1, 'VARCHAR2', 12, 'NUMBER', 3,\n               'LONG', -1, 'DATE', 93, 'RAW', -3, 'LONG RAW', -4, 1111)\n              AS data_type,\n       data_type AS type_name,\n       DECODE (data_precision, NULL, data_length, data_precision)\n              AS precision,\n       data_length AS length,\n       data_scale AS scale,\n       10 AS radix,\n       1 AS nullable,\n       null AS remarks,\n       package_name,\n       sequence,\n       overload\n FROM all_arguments\nWHERE (1 = ? or package_name LIKE ? ESCAPE '/')\n  AND owner LIKE ? ESCAPE '/'\n  AND object_name LIKE ? escape '/'\n  AND argument_name LIKE ? escape '/'\nORDER BY procedure_schem, procedure_name, overload, sequence\n");
      s.setInt(1, catalog == null ? 1 : 0);
      s.setString(2, catalog == null ? "%" : catalog);
      s.setString(3, schemaPattern == null ? "%" : schemaPattern);
      s.setString(4, procedureNamePattern == null ? "%" : procedureNamePattern);
      s.setString(5, columnNamePattern == null ? "%" : columnNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getTables(String catalog, String schemaPattern, String tableNamePattern, String[] types) throws SQLException {
      String type_select = "";
      if (types != null) {
         type_select = "  AND c.table_type IN (";

         int i;
         for(i = 0; i < types.length - 1; ++i) {
            type_select = type_select + "'" + types[i] + "', ";
         }

         type_select = type_select + "'" + types[i] + "')\n";
      }

      String query;
      if (this.connection.report_remarks) {
         query = "SELECT NULL AS table_cat,\n       c.owner AS table_schem,\n       c.table_name AS table_name,\n       c.table_type AS table_type,\n       k.comments AS table_remarks\nFROM all_catalog c, all_tab_comments k\nWHERE c.owner LIKE ? escape '/'\n  AND c.table_name LIKE ? escape '/'\n" + type_select + "  AND c.owner = k.owner\n" + "  AND c.table_name = k.table_name (+)\n" + "ORDER BY table_type, table_schem, table_name\n";
      } else {
         query = "SELECT NULL AS table_cat,\n       c.owner AS table_schem,\n       c.table_name AS table_name,\n       c.table_type AS table_type,\n       null AS table_remarks\nFROM all_catalog c\nWHERE c.owner LIKE ? escape '/'\n  AND c.table_name LIKE ? escape '/'\n" + type_select + "ORDER BY table_type, table_schem, table_name\n";
      }

      PreparedStatement s = this.connection.prepareStatement(query);
      s.setString(1, schemaPattern == null ? "%" : schemaPattern);
      s.setString(2, tableNamePattern == null ? "%" : tableNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getSchemas() throws SQLException {
      Statement s = this.connection.createStatement();
      String basic_query = "SELECT username AS table_schem FROM all_users ORDER BY table_schem";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(basic_query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getCatalogs() throws SQLException {
      this.connection.trace("getCatalogs");
      Statement s = this.connection.createStatement();
      String query = "select 'nothing' as table_cat from dual where 1 = 2";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getTableTypes() throws SQLException {
      Statement s = this.connection.createStatement();
      String query = "select 'TABLE' as table_type from dual\nunion select 'VIEW' as table_type from dual\nunion select 'SYNONYM' as table_type from dual\n";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getColumns(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern) throws SQLException {
      String query;
      if (this.connection.report_remarks) {
         query = "SELECT NULL AS table_cat,\n       t.owner AS table_schem,\n       t.table_name,\n       t.column_name,\n       DECODE (t.data_type, 'CHAR', 1, 'VARCHAR2', 12, 'NUMBER', 3,\n               'LONG', -1, 'DATE', 93, 'RAW', -3, 'LONG RAW', -4, 1111)\n              AS data_type,\n       t.data_type AS type_name,\n       DECODE (t.data_precision, null, t.data_length, t.data_precision)\n              AS column_size,\n       0 AS buffer_length,\n       t.data_scale AS decimal_digits,\n       10 AS num_prec_radix,\n       DECODE (t.nullable, 'N', 0, 1) AS nullable,\n       null AS remarks,\n       t.data_default AS column_def,\n       0 AS sql_data_type,\n       0 AS sql_datetime_sub,\n       t.data_length AS char_octet_length,\n       t.column_id AS ordinal_position,\n       DECODE (t.nullable, 'N', 'NO', 'YES') AS is_nullable\nFROM all_tab_columns t\nWHERE t.owner LIKE ? ESCAPE '/'\n  AND t.table_name LIKE ? ESCAPE '/'\n  AND t.column_name LIKE ? ESCAPE '/'\nORDER BY table_schem, table_name, ordinal_position\n";
      } else {
         query = "SELECT NULL AS table_cat,\n       t.owner AS table_schem,\n       t.table_name,\n       t.column_name,\n       DECODE (t.data_type, 'CHAR', 1, 'VARCHAR2', 12, 'NUMBER', 3,\n               'LONG', -1, 'DATE', 93, 'RAW', -3, 'LONG RAW', -4, 1111)\n              AS data_type,\n       t.data_type AS type_name,\n       DECODE (t.data_precision, null, t.data_length, t.data_precision)\n              AS column_size,\n       0 AS buffer_length,\n       t.data_scale AS decimal_digits,\n       10 AS num_prec_radix,\n       DECODE (t.nullable, 'N', 0, 1) AS nullable,\n       c.comments AS remarks,\n       t.data_default AS column_def,\n       0 AS sql_data_type,\n       0 AS sql_datetime_sub,\n       t.data_length AS char_octet_length,\n       t.column_id AS ordinal_position,\n       DECODE (t.nullable, 'N', 'NO', 'YES') AS is_nullable\nFROM all_tab_columns t, all_col_comments c\nWHERE t.owner LIKE ? ESCAPE '/'\n  AND t.table_name LIKE ? ESCAPE '/'\n  AND t.column_name LIKE ? ESCAPE '/'\n  AND t.owner = c.owner (+)\n  AND t.table_name = c.table_name (+)\n  AND t.column_name = c.column_name (+)\nORDER BY table_schem, table_name, ordinal_position\n";
      }

      PreparedStatement s = this.connection.prepareStatement(query);
      s.setString(1, schemaPattern == null ? "%" : schemaPattern);
      s.setString(2, tableNamePattern == null ? "%" : tableNamePattern);
      s.setString(3, columnNamePattern == null ? "%" : columnNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getColumnPrivileges(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT NULL AS table_cat,\n       table_schema AS table_schem,\n       table_name,\n       column_name,\n       grantor,\n       grantee,\n       privilege,\n       grantable AS is_grantable\nFROM all_col_privs\nWHERE table_schema LIKE ? ESCAPE '/'\n  AND table_name LIKE ? ESCAPE '/'\n  AND column_name LIKE ? ESCAPE '/'\nORDER BY column_name, privilege\n");
      s.setString(1, schemaPattern == null ? "%" : schemaPattern);
      s.setString(2, tableNamePattern == null ? "%" : tableNamePattern);
      s.setString(3, columnNamePattern == null ? "%" : columnNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getTablePrivileges(String catalog, String schemaPattern, String tableNamePattern) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT NULL AS table_cat,\n       table_schema AS table_schem,\n       table_name,\n       null as column_name,\n       grantor,\n       grantee,\n       privilege,\n       grantable AS is_grantable\nFROM all_tab_privs\nWHERE table_schema LIKE ? ESCAPE '/'\n  AND table_name LIKE ? ESCAPE '/'\nORDER BY table_schem, table_name, privilege\n");
      s.setString(1, schemaPattern == null ? "%" : schemaPattern);
      s.setString(2, tableNamePattern == null ? "%" : tableNamePattern);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getBestRowIdentifier(String catalog, String schema, String table, int scope, boolean nullable) throws SQLException {
      this.connection.trace("getBestRowIdentifier");
      PreparedStatement s = this.connection.prepareStatement("SELECT 1 AS scope, 'ROWID' AS column_name, -8 AS data_type,\n       'ROWID' AS type_name, 0 AS column_size, 0 AS buffer_length,\n       0 AS decimal_digits, 2 AS pseudo_column\nFROM DUAL\nWHERE ? = 1\nUNION\nSELECT 2 AS scope,\n       t.column_name,\n       DECODE (t.data_type, 'CHAR', 1, 'VARCHAR2', 12, 'NUMBER', 3,\n               'LONG', -1, 'DATE', 93, 'RAW', -3, 'LONG RAW', -4, 1111)\n              AS data_type,\n       t.data_type AS type_name,\n       DECODE (t.data_precision, null, t.data_length, t.data_precision)\n              AS column_size,\n       0 AS buffer_length,\n       t.data_scale AS decimal_digits,\n       1 AS pseudo_column\nFROM all_tab_columns t, all_ind_columns i\nWHERE ? = 1\n  AND t.table_name = ?\n  AND t.owner like ? escape '/',\n  AND t.nullable != ?\n  AND t.owner = i.table_owner\n  AND t.table_name = i.table_name\n  AND t.column_name = i.column_name\n");
      switch (scope) {
         case 0:
            s.setInt(1, 0);
            s.setInt(2, 0);
            break;
         case 1:
            s.setInt(1, 1);
            s.setInt(2, 1);
            break;
         case 2:
            s.setInt(1, 0);
            s.setInt(2, 1);
      }

      s.setString(3, table);
      s.setString(4, schema == null ? "%" : schema);
      s.setString(5, nullable ? "X" : "Y");
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getVersionColumns(String catalog, String schema, String table) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT 0 AS scope,\n       t.column_name,\n       DECODE (c.data_type, 'CHAR', 1, 'VARCHAR2', 12, 'NUMBER', 3,\n               'LONG', -1, 'DATE', 93, 'RAW', -3, 'LONG RAW', -4, 1111)\n              AS data_type,\n       c.data_type AS type_name,\n       DECODE (c.data_precision, null, c.data_length, c.data_precision)\n              AS column_size,\n       0 as buffer_length,\n       c.data_scale as decimal_digits,\n       0 as pseudo_column\nFROM all_trigger_cols t, all_tab_columns c\nWHERE t.table_name = ?\n  AND c.owner like ? escape '/',\n  AND t.table_owner = c.owner\n  AND t.table_name = c.table_name\n  AND t.column_name = c.column_name\n");
      s.setString(1, table);
      s.setString(2, schema == null ? "%" : schema);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getPrimaryKeys(String catalog, String schema, String table) throws SQLException {
      PreparedStatement s = this.connection.prepareStatement("SELECT NULL AS table_cat,\n       c.owner AS table_schem,\n       c.table_name,\n       c.column_name,\n       c.position AS key_seq,\n       c.constraint_name AS pk_name\nFROM all_cons_columns c, all_constraints k\nWHERE k.constraint_type = 'P'\n  AND k.table_name = ?\n  AND k.owner like ? escape '/',\n  AND k.constraint_name = c.constraint_name \n  AND k.table_name = c.table_name \n  AND k.owner = c.owner \nORDER BY column_name\n");
      s.setString(1, table);
      s.setString(2, schema == null ? "%" : schema);
      OracleResultSet rs = (OracleResultSet)s.executeQuery();
      rs.close_statement_on_close = true;
      return rs;
   }

   String keys_query(String pSchema, String pTable, String fSchema, String fTable) {
      return "select null as pktable_cat,\n       p.owner as pktable_schem,\n       p.table_name as pktable_name,\n       pc.column_name as pkcolumn_name,\n       null as fktable_cat,\n       f.owner as fktable_schem,\n       f.table_name as fktable_name,\n       fc.column_name as fkcolumn_name,\n       fc.position as key_seq,\n       null as update_rule,\n       decode (f.delete_rule, 'CASCADE', 0, 1) as delete_rule,\n       f.constraint_name as fk_name,\n       p.constraint_name as pk_name,\n       null as deferrability\nfrom all_constraints p, all_cons_columns pc,\n     all_constraints f, all_cons_columns fc\nwhere 1 = 1\n" + (pTable != null ? "  and p.table_name = '" + pTable + "'\n" : "") + (fTable != null ? "  and f.table_name = '" + fTable + "'\n" : "") + (pSchema != null && pSchema.length() > 0 ? "  and p.owner = '" + pSchema + "'\n" : "") + (fSchema != null && fSchema.length() > 0 ? "  and f.owner = '" + fSchema + "'\n" : "") + "  and p.constraint_type = 'P'\n" + "  and f.constraint_type = 'R'\n" + "  and p.constraint_name = f.r_constraint_name\n" + "  and p.owner = pc.owner\n" + "  and p.constraint_name = pc.constraint_name\n" + "  and p.table_name = pc.table_name\n" + "  and f.owner = fc.owner\n" + "  and f.constraint_name = fc.constraint_name\n" + "  and f.table_name = fc.table_name\n" + "  and fc.position = pc.position\n";
   }

   public synchronized ResultSet getImportedKeys(String catalog, String schema, String table) throws SQLException {
      this.connection.trace("getImportedKeys");
      Statement s = this.connection.createStatement();
      String query = this.keys_query((String)null, (String)null, schema, table) + "order by pktable_schem, pktable_name, key_seq\n";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getExportedKeys(String catalog, String schema, String table) throws SQLException {
      this.connection.trace("getExportedKeys");
      Statement s = this.connection.createStatement();
      String query = this.keys_query(schema, table, (String)null, (String)null) + "order by fktable_schem, fktable_name, key_seq\n";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getCrossReference(String primaryCatalog, String primarySchema, String primaryTable, String foreignCatalog, String foreignSchema, String foreignTable) throws SQLException {
      this.connection.trace("getCrossReference");
      Statement s = this.connection.createStatement();
      String query = this.keys_query(primarySchema, primaryTable, foreignSchema, foreignTable) + "order by fktable_schem, fktable_name, key_seq\n";
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public ResultSet getTypeInfo() throws SQLException {
      this.connection.trace("getTypeInfo");
      Statement s = this.connection.createStatement();
      String number_query = "select\n 'NUMBER' as type_name, 2 as data_type, 38 as precision,\n NULL as literal_prefix, NULL as literal_suffix, NULL as create_params,\n 1 as nullable, 0 as case_sensitive, 1 as searchable,\n 0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n 'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String bit_query = "union select 'NUMBER' as type_name, -7 as data_type, 1 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \n'(1)' as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String tinyint_query = "union select 'NUMBER' as type_name, -6 as data_type, 3 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \n'(3)' as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String smallint_query = "union select 'NUMBER' as type_name, 5 as data_type, 5 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \n'(5)' as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String integer_query = "union select 'NUMBER' as type_name, 4 as data_type, 10 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \n'(10)' as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String bigint_query = "union select 'NUMBER' as type_name, -5 as data_type, 38 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \nNULL as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'NUMBER' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String float_query = "union select 'FLOAT' as type_name, 6 as data_type, 63 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \nNULL as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'FLOAT' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String real_query = "union select 'REAL' as type_name, 7 as data_type, 63 as precision,\nNULL as literal_prefix, NULL as literal_suffix, \nNULL as create_params, 1 as nullable, 0 as case_sensitive, 1 as searchable,\n0 as unsigned_attribute, 1 as fixed_prec_scale, 0 as auto_increment,\n'REAL' as local_type_name, -84 as minimum_scale, 127 as maximum_scale,\nNULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String char_query = "union select\n 'CHAR' as type_name, 1 as data_type, 255 as precision,\n '''' as literal_prefix, '''' as literal_suffix, NULL as create_params,\n 1 as nullable, 1 as case_sensitive, 1 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'CHAR' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String varchar2_query = "union select\n 'VARCHAR2' as type_name, 12 as data_type, 2000 as precision,\n '''' as literal_prefix, '''' as literal_suffix, NULL as create_params,\n 1 as nullable, 1 as case_sensitive, 1 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'VARCHAR2' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String long_query = "union select\n 'LONG' as type_name, -1 as data_type, 134217727 as precision,\n '''' as literal_prefix, '''' as literal_suffix, NULL as create_params,\n 1 as nullable, 1 as case_sensitive, 0 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'LONG' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String timestamp_query = "union select\n 'DATE' as type_name, 93 as data_type, 7 as precision,\n NULL as literal_prefix, NULL as literal_suffix, NULL as create_params,\n 1 as nullable, 0 as case_sensitive, 1 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'DATE' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String raw_query = "union select\n 'RAW' as type_name, -3 as data_type, 255 as precision,\n '''' as literal_prefix, '''' as literal_suffix, NULL as create_params,\n 1 as nullable, 0 as case_sensitive, 0 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'RAW' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String long_raw_query = "union select\n 'LONG RAW' as type_name, -4 as data_type, 134217727 as precision,\n '''' as literal_prefix, '''' as literal_suffix, NULL as create_params,\n 1 as nullable, 0 as case_sensitive, 0 as searchable,\n 0 as unsigned_attribute, 0 as fixed_prec_scale, 0 as auto_increment,\n 'LONG RAW' as local_type_name, 0 as minimum_scale, 0 as maximum_scale,\n NULL as sql_data_type, NULL as sql_datetime_sub, 10 as num_prec_radix\nfrom dual\n";
      String order_by = "order by data_type\n";
      String query = number_query + char_query + varchar2_query + timestamp_query + raw_query + long_query + long_raw_query + bit_query + tinyint_query + smallint_query + integer_query + bigint_query + float_query + real_query + order_by;
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   public synchronized ResultSet getIndexInfo(String catalog, String schema, String table, boolean unique, boolean approximate) throws SQLException {
      this.connection.trace("getIndexInfo");
      Statement s = this.connection.createStatement();
      String analyze = "analyze table " + (schema == null ? "" : schema + ".") + table + (approximate ? " estimate statistics" : " compute statistics");
      this.connection.trace(analyze);
      s.executeUpdate(analyze);
      String table_query = "select null as table_cat,\n       owner as table_schem,\n       table_name,\n       0 as NON_UNIQUE,\n       null as index_qualifier,\n       null as index_name, 0 as type,\n       0 as ordinal_position, null as column_name,\n       null as asc_or_desc,\n       num_rows as cardinality,\n       blocks as pages,\n       null as filter_condition\nfrom all_tables\nwhere table_name = '" + table + "'\n";
      String table_schema_select = "";
      if (schema != null && schema.length() > 0) {
         table_schema_select = "  and owner = '" + schema + "'\n";
      }

      String index_query = "select null as table_cat,\n       i.owner as table_schem,\n       i.table_name,\n       decode (i.uniqueness, 'UNIQUE', 0, 1),\n       null as index_qualifier,\n       i.index_name,\n       1 as type,\n       c.column_position as ordinal_position,\n       c.column_name,\n       null as asc_or_desc,\n       i.distinct_keys as cardinality,\n       i.leaf_blocks as pages,\n       null as filter_condition\nfrom all_indexes i, all_ind_columns c\nwhere i.table_name = '" + table + "'\n";
      String index_schema_select = "";
      if (schema != null && schema.length() > 0) {
         index_schema_select = "  and i.owner = '" + schema + "'\n";
      }

      String index_unique_select = "";
      if (unique) {
         index_unique_select = "  and i.uniqueness = 'UNIQUE'\n";
      }

      String index_join_select = "  and i.index_name = c.index_name\n  and i.table_owner = c.table_owner\n  and i.table_name = c.table_name\n  and i.owner = c.index_owner\n";
      String index_order_by = "order by non_unique, type, index_name, ordinal_position\n";
      String query = table_query + table_schema_select + "union\n" + index_query + index_schema_select + index_unique_select + index_join_select + index_order_by;
      OracleResultSet rs = (OracleResultSet)s.executeQuery(query);
      rs.close_statement_on_close = true;
      return rs;
   }

   SQLException fail() {
      SQLException ex = new SQLException("Not implemented yet");
      return ex;
   }
}
