package oracle.jdbc.driver;

import java.io.IOException;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;

public class OracleInputStream extends OracleBufferedStream {
   static final boolean DEBUG = false;
   boolean isNull;
   OracleStatement statement;
   int columnIndex;
   OracleInputStream previous_stream;

   public OracleInputStream(OracleStatement stmt, int index, OracleInputStream previous) {
      super(stmt.connection.db_access.getDefaultStreamChunkSize());
      super.closed = true;
      this.previous_stream = previous;
      this.statement = stmt;
      this.columnIndex = index;
   }

   public boolean needBytes() throws IOException {
      if (super.closed) {
         return false;
      } else {
         if (super.pos >= super.count) {
            try {
               int readLength = this.statement.connection.db_access.read(this.statement.dbstmt, super.buf, super.chunkSize);
               super.pos = 0;
               super.count = readLength;
               if (super.count == -1) {
                  if (this.statement.streams == this) {
                     this.statement.connection.releaseLine();
                  }

                  super.closed = true;
                  return false;
               }
            } catch (SQLException e) {
               DBError.SQLToIOException(e);
            }
         }

         return true;
      }
   }

   public boolean isNull() throws IOException {
      boolean result = false;

      try {
         if (this.statement.defines.types != null) {
            result = this.statement.defines.isNull(this.columnIndex - 1, 0);
         } else {
            result = this.statement.binds_out.isNull(this.columnIndex - 1, 0);
         }
      } catch (SQLException e) {
         DBError.SQLToIOException(e);
      }

      return result;
   }

   public boolean isClosed() {
      return super.closed;
   }

   public void close() throws IOException {
      if (!super.closed) {
         if (this.previous_stream != null) {
            this.previous_stream.close();
         }

         if (!this.isNull()) {
            while(this.needBytes()) {
               super.pos = super.count;
            }
         }

         super.closed = true;
      }

   }

   public void resetLong() {
      if (this.previous_stream != null) {
         this.previous_stream.resetLong();
      }

      super.pos = super.count = 0;
      super.closed = false;
   }
}
