package oracle.jdbc.driver;

import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;

public class OracleResultSetMetaData implements ResultSetMetaData {
   OracleConnection connection;
   OracleStatement statement;

   OracleResultSetMetaData(OracleConnection conn, OracleStatement stmt) throws SQLException {
      this.connection = conn;
      this.statement = stmt;
      stmt.describe();
   }

   public int getColumnCount() throws SQLException {
      return this.statement.dbdescription.length;
   }

   public boolean isAutoIncrement(int column) throws SQLException {
      return false;
   }

   private int getValidColumnIndex(int column) throws SQLException {
      int index = column - 1;
      if (index < 0 || index >= this.statement.dbdescription.length) {
         DBError.check_error(-3, "getValidColumnIndex");
      }

      return index;
   }

   public boolean isCaseSensitive(int column) throws SQLException {
      int type = this.getColumnType(column);
      return type == 1 || type == 12 || type == -1;
   }

   public boolean isSearchable(int column) throws SQLException {
      int type = this.getColumnType(column);
      return type != -2 && type != -3 && type != -4 && type != -1;
   }

   public boolean isCurrency(int column) throws SQLException {
      return this.getColumnType(column) == 2;
   }

   public int isNullable(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      return this.statement.dbdescription[index].nullable ? 1 : 0;
   }

   public boolean isSigned(int column) throws SQLException {
      return true;
   }

   public int getColumnDisplaySize(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      return this.statement.dbdescription[index].max_length;
   }

   public String getColumnLabel(int column) throws SQLException {
      return this.getColumnName(column);
   }

   public String getColumnName(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      return this.statement.dbdescription[index].name;
   }

   public String getSchemaName(int column) throws SQLException {
      return "";
   }

   public int getPrecision(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      return this.statement.dbdescription[index].precision;
   }

   public int getScale(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      return this.statement.dbdescription[index].scale;
   }

   public String getTableName(int column) throws SQLException {
      return "";
   }

   public String getCatalogName(int column) throws SQLException {
      return "";
   }

   public int getColumnType(int column) throws SQLException {
      int index = this.getValidColumnIndex(column);
      int type = this.statement.dbdescription[index].type;
      switch (type) {
         case 1:
            return 12;
         case 2:
         case 6:
            return 2;
         case 8:
            return -1;
         case 11:
            return -8;
         case 12:
            return 93;
         case 23:
            return -3;
         case 24:
            return -4;
         case 96:
            return 1;
         case 102:
            return -10;
         case 112:
            return -12;
         case 113:
            return -11;
         case 114:
            return -13;
         case 115:
            return -14;
         default:
            return 1111;
      }
   }

   public String getColumnTypeName(int column) throws SQLException {
      int type = this.getColumnType(column);
      switch (type) {
         case -14:
            return "CFILE";
         case -13:
            return "BFILE";
         case -12:
            return "CLOB";
         case -11:
            return "BLOB";
         case -10:
            return "REFCURSOR";
         case -8:
            return "ROWID";
         case -4:
            return "LONG RAW";
         case -3:
            return "RAW";
         case -1:
            return "LONG";
         case 1:
            return "CHAR";
         case 2:
            return "NUMBER";
         case 12:
            return "VARCHAR2";
         case 93:
            return "DATE";
         case 1111:
         default:
            return null;
      }
   }

   public boolean isReadOnly(int column) throws SQLException {
      return false;
   }

   public boolean isWritable(int column) throws SQLException {
      return true;
   }

   public boolean isDefinitelyWritable(int column) throws SQLException {
      return false;
   }
}
