package oracle.jdbc.driver;

import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;

public class OracleSql {
   int current_argument;
   // $FF: renamed from: i int
   int field_0;
   int length;
   // $FF: renamed from: c char
   char field_1;
   boolean first;
   boolean in_string;
   String odbc_sql;
   StringBuffer oracle_sql = new StringBuffer(128);
   StringBuffer token_buffer = new StringBuffer(32);

   public String parse(String os) throws SQLException {
      this.current_argument = 1;
      this.field_0 = 0;
      this.first = true;
      this.in_string = false;
      this.oracle_sql.ensureCapacity(this.length);
      this.oracle_sql.setLength(0);
      this.odbc_sql = os;
      this.length = this.odbc_sql.length();
      this.handleODBC();
      if (this.field_0 < this.length) {
         Integer index = new Integer(this.field_0);
         DBError.check_error(-33, index);
      }

      return this.oracle_sql.toString();
   }

   void handleODBC() throws SQLException {
      while(this.field_0 < this.length) {
         this.field_1 = this.odbc_sql.charAt(this.field_0);
         if (this.in_string) {
            this.oracle_sql.append(this.field_1);
            if (this.field_1 == '\'') {
               this.in_string = false;
            }

            ++this.field_0;
         } else {
            switch (this.field_1) {
               case '\'':
                  this.oracle_sql.append(this.field_1);
                  this.in_string = true;
                  ++this.field_0;
                  this.first = false;
                  break;
               case '{':
                  this.token_buffer.setLength(0);
                  ++this.field_0;
                  this.skipSpace();

                  while(this.field_0 < this.length && (Character.isJavaLetterOrDigit(this.field_1 = this.odbc_sql.charAt(this.field_0)) || this.field_1 == '?')) {
                     this.token_buffer.append(this.field_1);
                     ++this.field_0;
                  }

                  this.handleToken(this.token_buffer.toString());
                  this.field_1 = this.odbc_sql.charAt(this.field_0);
                  if (this.field_1 != '}') {
                     String s = new String(this.field_0 + ": Expecting \"}\" got \"" + this.field_1 + "\"");
                     DBError.check_error(-33, s);
                  }

                  ++this.field_0;
                  break;
               case '}':
                  return;
               default:
                  this.appendChar(this.oracle_sql, this.field_1);
                  ++this.field_0;
                  this.first = false;
            }
         }
      }

   }

   void handleToken(String token) throws SQLException {
      if (token.equalsIgnoreCase("?")) {
         this.handleFunction();
      } else if (token.equalsIgnoreCase("call")) {
         this.handleCall();
      } else if (token.equalsIgnoreCase("ts")) {
         this.handleTimestamp();
      } else if (token.equalsIgnoreCase("t")) {
         this.handleTime();
      } else if (token.equalsIgnoreCase("d")) {
         this.handleDate();
      } else if (token.equalsIgnoreCase("escape")) {
         this.handleEscape();
      } else {
         String s = new String(this.field_0 + ": " + token);
         DBError.check_error(-34, s);
      }

   }

   void handleFunction() throws SQLException {
      boolean need_block = this.first;
      if (need_block) {
         this.oracle_sql.append("BEGIN ");
      }

      this.appendChar(this.oracle_sql, '?');
      this.skipSpace();
      if (this.field_1 != '=') {
         String s = new String(this.field_0 + ". Expecting \"=\" got \"" + this.field_1 + "\"");
         DBError.check_error(-33, s);
      }

      ++this.field_0;
      this.skipSpace();
      if (!this.odbc_sql.startsWith("call", this.field_0)) {
         String s = new String(this.field_0 + ". Expecting \"call\"");
         DBError.check_error(-33, s);
      }

      this.field_0 += 4;
      this.oracle_sql.append(" := ");
      this.skipSpace();
      this.handleODBC();
      if (need_block) {
         this.oracle_sql.append("; END;");
      }

   }

   void handleCall() throws SQLException {
      boolean need_block = this.first;
      if (need_block) {
         this.oracle_sql.append("BEGIN ");
      }

      this.skipSpace();
      this.handleODBC();
      this.skipSpace();
      if (need_block) {
         this.oracle_sql.append("; END;");
      }

   }

   void handleTimestamp() throws SQLException {
      this.oracle_sql.append("TO_DATE (");
      this.skipSpace();

      boolean in_nanos;
      for(in_nanos = false; this.field_0 < this.length && (this.field_1 = this.odbc_sql.charAt(this.field_0)) != '}'; ++this.field_0) {
         if (!in_nanos) {
            if (this.field_1 == '.') {
               in_nanos = true;
            } else {
               this.oracle_sql.append(this.field_1);
            }
         }
      }

      if (in_nanos) {
         this.oracle_sql.append('\'');
      }

      this.oracle_sql.append(", 'YYYY-MM-DD HH24:MI:SS')");
   }

   void handleTime() throws SQLException {
      this.oracle_sql.append("TO_DATE (");
      this.skipSpace();
      this.handleODBC();
      this.oracle_sql.append(", 'HH24:MI:SS')");
   }

   void handleDate() throws SQLException {
      this.oracle_sql.append("TO_DATE (");
      this.skipSpace();
      this.handleODBC();
      this.oracle_sql.append(", 'YYYY-MM-DD')");
   }

   void handleEscape() throws SQLException {
      this.oracle_sql.append("ESCAPE ");
      this.skipSpace();
      this.handleODBC();
   }

   String nextArgument() {
      String result = ":" + this.current_argument;
      ++this.current_argument;
      return result;
   }

   void appendChar(StringBuffer oracle_sql, char c) {
      if (c == '?') {
         oracle_sql.append(this.nextArgument());
      } else {
         oracle_sql.append(c);
      }

   }

   void skipSpace() {
      while(this.field_0 < this.length && (this.field_1 = this.odbc_sql.charAt(this.field_0)) == ' ') {
         ++this.field_0;
      }

   }
}
