package oracle.jdbc.oci7;

import java.io.IOException;
import java.io.PrintStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBColumn;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBData;
import oracle.jdbc.dbaccess.DBItem;
import oracle.jdbc.dbaccess.DBStatement;
import oracle.jdbc.dbaccess.DBType;
import oracle.jdbc.oracore.JavaConversion;

public class OCIDBAccess implements DBAccess {
   DBConversion conversion;
   DBData[] curr_out_values;
   DBType[] curr_out_types;
   int start_index;
   int end_index;
   static String oci_semaphore = "oci_semaphore";
   short char_set;
   int c_state = 0;
   private static boolean loaded_library;
   static final int EOJ_SUCCESS = 0;
   static final int EOJ_ERROR = -1;
   static final int EOJ_IOEXCEPTION = -2;
   static final int EOJ_INVALID_COLUMN_INDEX = -3;
   static final int EOJ_INVALID_COLUMN_TYPE = -4;
   static final int EOJ_UNSUPPORTED_COLUMN_TYPE = -5;
   static final int EOJ_INVALID_COLUMN_NAME = -6;
   static final int EOJ_INVALID_DYNAMIC_COLUMN = -7;
   static final int EOJ_CLOSED_CONNECTION = -8;
   static final int EOJ_CLOSED_STATEMENT = -9;
   static final int EOJ_CLOSED_RESULTSET = -10;
   static final int EOJ_EXHAUSTED_RESULTSET = -11;
   static final int EOJ_TYPE_CONFLICT = -12;
   static final int EOJ_WAS_NULL = -13;
   static final int EOJ_RESULTSET_BEFORE_FIRST_ROW = -14;
   static final int EOJ_STATEMENT_WAS_CANCELLED = -15;
   static final int EOJ_STATEMENT_TIMED_OUT = -16;
   static final int EOJ_CURSOR_ALREADY_INITIALIZED = -17;
   static final int EOJ_INVALID_CURSOR = -18;
   static final int EOJ_CAN_ONLY_DESCRIBE_A_QUERY = -19;

   native int make_c_state();

   native int free_c_state(boolean var1);

   public synchronized DBConversion logon(String user, String password, String database, Properties info) throws SQLException, IOException, OCIDBException {
      String var6 = oci_semaphore;
      synchronized(var6){}

      DBConversion var5;
      try {
         this.trace("DBAccess.logon");
         if (!loaded_library) {
            String dll_str = info.getProperty("dll");
            System.loadLibrary(dll_str);
            loaded_library = true;
         }

         if (this.c_state == 0) {
            this.check_error(this.make_c_state(), (OCIDBStatement)null);
         }

         this.char_set = (short)this.get_char_set();
         this.conversion = new JavaConversion(this.char_set);
         DBConversion default_conv = new JavaConversion();
         byte[] user_bytearray = this.conversion.StringToCharBytes(user);
         byte[] passwd_bytearray = default_conv.StringToCharBytes(password);
         byte[] db_bytearray = default_conv.StringToCharBytes(database);
         this.check_error(this.do_open(user_bytearray, passwd_bytearray, db_bytearray), (OCIDBStatement)null);
         var5 = this.conversion;
      } catch (Throwable var13) {
         throw var13;
      }

      return var5;
   }

   native int do_open(byte[] var1, byte[] var2, byte[] var3);

   native int get_char_set();

   public synchronized void logoff() throws SQLException, IOException, OCIDBException {
      String var1 = oci_semaphore;
      synchronized(var1){}

      try {
         this.trace("DBAccess.logoff");
         this.check_error(this.free_c_state(true), (OCIDBStatement)null);
      } catch (Throwable var3) {
         throw var3;
      }

   }

   public synchronized void commit() throws SQLException, IOException, OCIDBException {
      String var1 = oci_semaphore;
      synchronized(var1){}

      try {
         this.trace("DBAccess.commit");
         this.check_error(this.do_commit(), (OCIDBStatement)null);
      } catch (Throwable var3) {
         throw var3;
      }

   }

   native int do_commit();

   public synchronized void rollback() throws SQLException, IOException, OCIDBException {
      String var1 = oci_semaphore;
      synchronized(var1){}

      try {
         this.trace("DBAccess.rollback");
         this.check_error(this.do_rollback(), (OCIDBStatement)null);
      } catch (Throwable var3) {
         throw var3;
      }

   }

   native int do_rollback();

   public synchronized void setAutoCommit(boolean auto_commit) throws SQLException, IOException, OCIDBException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      try {
         this.trace("DBAccess.setAutoCommit");
         this.check_error(this.set_auto_commit(auto_commit), (OCIDBStatement)null);
      } catch (Throwable var4) {
         throw var4;
      }

   }

   native int set_auto_commit(boolean var1);

   public synchronized DBStatement open() throws SQLException, IOException, OCIDBException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      OCIDBStatement var1;
      try {
         OCIDBStatement stmt = new OCIDBStatement(this);
         this.trace("DBAccess.open");
         this.check_error(this.make_statement_c_state(stmt), stmt);
         this.check_error(this.open_statement(stmt), stmt);
         var1 = stmt;
      } catch (Throwable var6) {
         throw var6;
      }

      return var1;
   }

   native int make_statement_c_state(OCIDBStatement var1);

   native int open_statement(OCIDBStatement var1);

   public synchronized void closeQuery(DBStatement dbstmt) throws SQLException, IOException, OCIDBException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      try {
         this.trace("DBAccess.closeQuery");
         this.check_error(this.close_query((OCIDBStatement)dbstmt), (OCIDBStatement)dbstmt);
      } catch (Throwable var4) {
         throw var4;
      }

   }

   native int close_query(OCIDBStatement var1);

   public synchronized void close(DBStatement stmt) throws SQLException, OCIDBException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      try {
         this.trace("DBAccess.close");
         this.check_error(this.close_stmt((OCIDBStatement)stmt), (OCIDBStatement)stmt);
      } catch (Throwable var4) {
         throw var4;
      }

   }

   native int close_stmt(OCIDBStatement var1);

   public synchronized void parse(DBStatement dbstmt, byte[] sqlStmt, DBType[] bind_types) throws SQLException, IOException, OCIDBException {
      String var4 = oci_semaphore;
      synchronized(var4){}

      try {
         this.trace("DBAccess.parse");

         try {
            this.check_error(this.do_parse((OCIDBStatement)dbstmt, sqlStmt), (OCIDBStatement)dbstmt);
         } finally {
            this.free_alloc();
         }
      } catch (Throwable var13) {
         throw var13;
      }

   }

   native int do_parse(OCIDBStatement var1, byte[] var2);

   int getByteArrayLength(byte[] barray) {
      return barray.length;
   }

   public synchronized DBColumn[] describe(DBStatement dbstmt) throws SQLException, IOException, OCIDBException {
      String var3 = oci_semaphore;
      synchronized(var3){}

      DBColumn[] var2;
      try {
         int num_columns = this.get_column_count((OCIDBStatement)dbstmt);
         DBColumn[] Columns = new DBColumn[num_columns];

         for(int i = 0; i < Columns.length; ++i) {
            Columns[i] = new DBColumn();
            byte[] bytearray = this.get_describe((OCIDBStatement)dbstmt, Columns[i], i + 1);
            if (bytearray != null) {
               this.enterColumnName(bytearray, Columns[i]);
            } else {
               this.check_error(-1, (OCIDBStatement)dbstmt);
            }
         }

         var2 = Columns;
      } catch (Throwable var10) {
         throw var10;
      }

      return var2;
   }

   native int get_column_count(OCIDBStatement var1);

   native byte[] get_describe(OCIDBStatement var1, DBColumn var2, int var3);

   void enterColumnName(byte[] bytearray, DBColumn column) throws SQLException {
      column.name = ((JavaConversion)this.conversion).CharBytesToString(bytearray, bytearray.length);
   }

   public synchronized DBColumn[] parseExecuteDescribe(DBStatement dbstmt, byte[] sqlstmt, DBType[] in_types, DBData[] in_values) throws SQLException, IOException, OCIDBException {
      String var6 = oci_semaphore;
      synchronized(var6){}

      DBColumn[] var5;
      try {
         try {
            int batch = 1;
            this.start_index = 0;
            this.end_index = 0;
            this.check_error(this.do_parse((OCIDBStatement)dbstmt, sqlstmt), (OCIDBStatement)dbstmt);
            int len;
            if (in_types != null) {
               len = in_types.length;
            } else {
               len = 0;
            }

            if (len != 0) {
               this.check_error(this.initialize_bind_columns((OCIDBStatement)dbstmt, len, batch), (OCIDBStatement)null);

               for(int i = 0; i < in_types.length; ++i) {
                  this.set_up_bind_column((OCIDBStatement)dbstmt, i, in_types[i], batch);

                  for(int j = 0; j < batch; ++j) {
                     this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, in_types[i], in_values[i].getItem(j), batch, true);
                  }
               }
            }

            this.check_error(this.do_execute((OCIDBStatement)dbstmt), (OCIDBStatement)dbstmt);
            int num_columns = this.check_error(this.get_column_count((OCIDBStatement)dbstmt), (OCIDBStatement)dbstmt);
            DBColumn[] Columns = new DBColumn[num_columns];

            for(int i = 0; i < Columns.length; ++i) {
               Columns[i] = new DBColumn();
               byte[] bytearray = this.get_describe((OCIDBStatement)dbstmt, Columns[i], i + 1);
               if (bytearray != null) {
                  this.enterColumnName(bytearray, Columns[i]);
               } else {
                  this.check_error(-1, (OCIDBStatement)dbstmt);
               }
            }

            var5 = Columns;
         } finally {
            this.free_alloc();
         }
      } catch (Throwable var22) {
         throw var22;
      }

      return var5;
   }

   native int initialize_bind_columns(OCIDBStatement var1, int var2, int var3);

   native int set_up_bind_column(OCIDBStatement var1, int var2, DBType var3, int var4);

   native int set_up_bind_rows(OCIDBStatement var1, int var2, int var3, DBType var4, DBItem var5, int var6, boolean var7);

   native int do_execute(OCIDBStatement var1);

   void copy_back_to_next_stream(OCIDBStatement dbstmt, int status) throws SQLException {
      String var3 = oci_semaphore;
      synchronized(var3){}

      try {
         if (this.curr_out_types != null) {
            for(this.end_index = this.start_index; this.end_index < this.curr_out_types.length; ++this.end_index) {
               if (this.curr_out_types[this.end_index] != null) {
                  if (status > 0) {
                     for(int row = 0; row < status; ++row) {
                        this.copy_value_from_defines(dbstmt, this.end_index, row, this.curr_out_values[this.end_index].getItem(row));
                     }
                  } else {
                     this.copy_value_from_defines(dbstmt, this.end_index, 0, this.curr_out_values[this.end_index].getItem(0));
                     if (this.curr_out_types[this.end_index].is_stream) {
                        return;
                     }
                  }
               }
            }

         }
      } catch (Throwable var7) {
         throw var7;
      }
   }

   void copy_back_to_next_stream_from_binds(OCIDBStatement dbstmt) throws SQLException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      try {
         if (this.curr_out_types != null) {
            for(this.end_index = this.start_index; this.end_index < this.curr_out_types.length; ++this.end_index) {
               if (this.curr_out_types[this.end_index] != null) {
                  this.copy_value_from_binds(dbstmt, this.end_index, 0, this.curr_out_values[this.end_index].getItem(0));
                  if (this.curr_out_types[this.end_index].is_stream) {
                     return;
                  }
               }
            }

         }
      } catch (Throwable var5) {
         throw var5;
      }
   }

   public synchronized int executeFetch(DBStatement dbstmt, byte type, DBType[] in_types, DBData[] in_values, int batch, DBType[] out_types, DBData[] out_values, int rows) throws SQLException, IOException, OCIDBException {
      String var10 = oci_semaphore;
      synchronized(var10){}

      int var9;
      try {
         int error_status = 0;
         int maxlen = 0;
         int status = 0;
         int len;
         if (in_types != null) {
            len = in_types.length;
         } else {
            len = 0;
         }

         int olen;
         if (out_types != null) {
            this.curr_out_values = out_values;
            this.curr_out_types = out_types;
            this.start_index = 0;
            this.end_index = 0;
            olen = out_types.length;
         } else {
            olen = 0;
         }

         if (len < olen) {
            maxlen = olen;
         } else {
            maxlen = len;
         }

         if (type == 1) {
            this.check_error(this.initialize_bind_columns((OCIDBStatement)dbstmt, maxlen, batch), (OCIDBStatement)null);

            for(int i = 0; i < maxlen; ++i) {
               if (i < len && i < olen) {
                  if (in_values[i] != null) {
                     if (out_values[i] == null) {
                        for(int j = 0; j < batch; ++j) {
                           this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, in_types[i], in_values[i].getItem(j), batch, true);
                        }
                     } else {
                        DBType my_type = out_types[i];
                        if (in_types[i].type == out_types[i].type && in_types[i].max_length < out_types[i].max_length) {
                           my_type = out_types[i];
                        }

                        for(int j = 0; j < batch; ++j) {
                           this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, my_type, in_values[i].getItem(j), batch, true);
                        }
                     }
                  } else {
                     if (out_values[i] == null) {
                        throw new OCIDBException("Both in_values and out_values arrays are null.");
                     }

                     for(int j = 0; j < batch; ++j) {
                        this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, out_types[i], out_values[i].getItem(j), batch, false);
                     }
                  }
               } else if (i >= len && i < olen) {
                  for(int j = 0; j < batch; ++j) {
                     this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, out_types[i], out_values[i].getItem(j), batch, false);
                  }
               } else if (i >= olen && i < len) {
                  for(int j = 0; j < batch; ++j) {
                     this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, in_types[i], in_values[i].getItem(j), batch, true);
                  }
               }
            }
         } else if (len != 0) {
            this.check_error(this.initialize_bind_columns((OCIDBStatement)dbstmt, in_types.length, batch), (OCIDBStatement)null);

            for(int i = 0; i < in_types.length; ++i) {
               if (in_types[i] != null) {
                  for(int j = 0; j < batch; ++j) {
                     this.set_up_bind_rows((OCIDBStatement)dbstmt, i, j, in_types[i], in_values[i].getItem(j), batch, true);
                  }
               }
            }
         }

         if (type != 1 && out_values != null) {
            if (out_values != null) {
               len = out_values.length;
            } else {
               len = 0;
            }

            if (len != 0) {
               this.check_error(this.initialize_define_columns((OCIDBStatement)dbstmt, out_types.length, batch), (OCIDBStatement)null);

               for(int i = 0; i < out_types.length; ++i) {
                  if (out_types[i] != null) {
                     this.check_error(this.set_up_define_column((OCIDBStatement)dbstmt, i, out_types[i], rows), (OCIDBStatement)dbstmt);
                  }
               }
            }
         }

         status = this.do_execute((OCIDBStatement)dbstmt);
         this.check_error(status, (OCIDBStatement)dbstmt);
         if (type != 1 && out_values != null) {
            status = this.do_fetch((OCIDBStatement)dbstmt, rows, error_status);
            this.check_error(error_status, (OCIDBStatement)dbstmt);
         }

         if (out_values != null) {
            if (type == 0) {
               this.copy_back_to_next_stream((OCIDBStatement)dbstmt, status);
            } else if (type == 1) {
               this.copy_back_to_next_stream_from_binds((OCIDBStatement)dbstmt);
            }
         }

         if (status != -2 && type != 1) {
            this.free_alloc();
         }

         var9 = status;
      } catch (Throwable var24) {
         throw var24;
      }

      return var9;
   }

   native void patch_bind_rows(OCIDBStatement var1, int var2, int var3, int var4);

   native void free_alloc();

   native int initialize_define_columns(OCIDBStatement var1, int var2, int var3);

   native int get_number_of_rows_processed(OCIDBStatement var1);

   native int set_up_define_column(OCIDBStatement var1, int var2, DBType var3, int var4);

   native void copy_value_from_binds(OCIDBStatement var1, int var2, int var3, DBItem var4);

   native void copy_value_from_defines(OCIDBStatement var1, int var2, int var3, DBItem var4);

   public synchronized int parseExecuteFetch(DBStatement dbstmt, byte type, byte[] sqlstmt, DBType[] in_types, DBData[] in_values, int batch, DBType[] out_types, DBData[] out_values, int rows) throws SQLException, IOException {
      String var11 = oci_semaphore;
      synchronized(var11){}

      int var10;
      try {
         this.check_error(this.do_parse((OCIDBStatement)dbstmt, sqlstmt), (OCIDBStatement)dbstmt);
         if (in_types != null) {
            int len = in_types.length;
         } else {
            int len = 0;
         }

         int status = this.executeFetch(dbstmt, type, in_types, in_values, batch, out_types, out_values, rows);
         var10 = status;
      } catch (Throwable var20) {
         throw var20;
      }

      return var10;
   }

   native int do_fetch(OCIDBStatement var1, int var2, int var3);

   public synchronized int fetch(DBStatement dbstmt, DBType[] out_types, DBData[] out_values, int rows) throws SQLException, IOException {
      String var6 = oci_semaphore;
      synchronized(var6){}

      int var5;
      try {
         int error_status = 0;
         int status = 0;
         int len;
         if (out_types != null) {
            len = out_types.length;
         } else {
            len = 0;
         }

         if (len != 0) {
            this.curr_out_values = out_values;
            this.curr_out_types = out_types;
            this.start_index = 0;
            this.end_index = 0;
            this.check_error(this.initialize_define_columns((OCIDBStatement)dbstmt, out_types.length, rows), (OCIDBStatement)null);

            for(int i = 0; i < out_types.length; ++i) {
               if (out_types[i] != null) {
                  this.check_error(this.set_up_define_column((OCIDBStatement)dbstmt, i, out_types[i], rows), (OCIDBStatement)dbstmt);
               }
            }

            status = this.do_fetch((OCIDBStatement)dbstmt, rows, error_status);
            this.check_error(error_status, (OCIDBStatement)dbstmt);
            this.copy_back_to_next_stream((OCIDBStatement)dbstmt, status);
         }

         if (status != -2 || status < rows) {
            this.free_alloc();
         }

         var5 = status;
      } catch (Throwable var15) {
         throw var15;
      }

      return var5;
   }

   native void copy_indicators(OCIDBStatement var1, int var2, DBItem var3);

   public synchronized int read(DBStatement dbstmt, byte[] buffer, int length) throws SQLException, IOException {
      String var5 = oci_semaphore;
      synchronized(var5){}

      int var4;
      try {
         int status = 0;
         int min = length;
         int max_length = 0;
         if (this.curr_out_values != null) {
            max_length = this.curr_out_values[this.end_index].getItem(0).max_out_size;
            if (max_length < length) {
               min = max_length;
            } else {
               min = length;
            }
         }

         int stat = this.do_read((OCIDBStatement)dbstmt, buffer, min, status);
         this.check_error(status, (OCIDBStatement)dbstmt);
         if (stat == -1) {
            this.start_index = this.end_index + 1;
            this.copy_back_to_next_stream((OCIDBStatement)dbstmt, status);
            if (this.curr_out_values != null && this.end_index >= this.curr_out_values.length) {
               this.free_alloc();
               this.start_index = 0;
               this.end_index = 0;
               this.curr_out_values = null;
               this.curr_out_types = null;
            }
         }

         var4 = stat;
      } catch (Throwable var14) {
         throw var14;
      }

      return var4;
   }

   native int do_read(OCIDBStatement var1, byte[] var2, int var3, int var4);

   public synchronized byte[] getVersion() throws SQLException, IOException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      byte[] var1;
      try {
         var1 = this.get_database_version();
      } catch (Throwable var5) {
         throw var5;
      }

      return var1;
   }

   native byte[] get_database_version();

   byte[] makeByteArray(int size) {
      return new byte[size];
   }

   public int getDefaultPrefetch() {
      return 10;
   }

   public int getDefaultStreamChunkSize() {
      return 2000;
   }

   public short getCharSet() {
      return 0;
   }

   public void finalize() {
      String var1 = oci_semaphore;
      synchronized(var1){}

      try {
         this.trace("Connection.finalize");
         this.free_c_state(false);
      } catch (Throwable var3) {
         throw var3;
      }

   }

   public static Connection defaultConnection() throws SQLException {
      return DriverManager.getConnection("jdbc:oracle:oci7:scott/tiger");
   }

   public static native void init_driver();

   public short getCharacterSet() {
      return 0;
   }

   public void trace(String s) {
      PrintStream outStream = DriverManager.getLogStream();
      if (outStream != null) {
         outStream.println(s);
      }

   }

   public void cancel() throws SQLException {
      this.check_error(this.conn_cancel(), (OCIDBStatement)null);
   }

   native int conn_cancel();

   public synchronized int check_error(IOException e) throws SQLException {
      ((Throwable)e).printStackTrace();
      throw new SQLException("IOException from DBAccess driver");
   }

   public synchronized int check_error(int status, OCIDBStatement stmt) throws SQLException, OCIDBException {
      String var4 = oci_semaphore;
      synchronized(var4){}

      int var3;
      try {
         if (status < 0) {
            this.free_alloc();
            switch (status) {
               case -18:
                  throw new OCIDBException("Invalid cursor");
               case -17:
                  throw new OCIDBException("Cursor already initialized");
               case -16:
                  throw new OCIDBException("Statement timed out");
               case -15:
                  throw new OCIDBException("Statement was cancelled");
               case -14:
                  throw new OCIDBException("ResultSet.next was not called");
               case -13:
                  var3 = status;
                  return var3;
               case -12:
                  throw new OCIDBException("Parameter Type Conflict");
               case -11:
                  throw new OCIDBException("Exhausted Resultset");
               case -10:
                  throw new OCIDBException("Closed Resultset");
               case -9:
                  throw new OCIDBException("Closed Statement");
               case -8:
                  throw new OCIDBException("Closed Connection");
               case -7:
                  throw new OCIDBException("Invalid dynamic column");
               case -6:
                  throw new OCIDBException("Invalid column name");
               case -5:
                  throw new OCIDBException("Unsupported column type");
               case -4:
                  throw new OCIDBException("Invalid column type");
               case -3:
                  throw new OCIDBException("Invalid column index");
               case -2:
               default:
                  throw new OCIDBException();
               case -1:
                  OCIDBError description = new OCIDBError();
                  OCIDBException current = null;
                  this.describe_error(stmt, description);
                  byte[] reason_bytearr = new byte[description.reason.length()];
                  description.reason.getBytes(0, description.reason.length(), reason_bytearr, 0);
                  String reason_string = this.conversion.CharBytesToString(reason_bytearr, reason_bytearr.length);
                  byte[] sqlstate_bytearr = new byte[description.SQLState.length()];
                  description.SQLState.getBytes(0, description.SQLState.length(), sqlstate_bytearr, 0);
                  String sqlstate_string = this.conversion.CharBytesToString(sqlstate_bytearr, sqlstate_bytearr.length);
                  current = new OCIDBException(reason_string, sqlstate_string, description.vendorCode);
                  throw current;
            }
         }

         var3 = status;
      } catch (Throwable var13) {
         throw var13;
      }

      return var3;
   }

   native void describe_error(OCIDBStatement var1, OCIDBError var2);

   public DBStatement RefCursorBytesToDBStatement(byte[] array) throws SQLException {
      String var3 = oci_semaphore;
      synchronized(var3){}

      OCIDBStatement var2;
      try {
         OCIDBStatement stmt = new OCIDBStatement(this);
         this.make_statement(stmt, array);
         var2 = stmt;
      } catch (Throwable var7) {
         throw var7;
      }

      return var2;
   }

   native int make_statement(OCIDBStatement var1, byte[] var2);

   public byte[] DBStatementToRefCursorBytes(DBStatement dbstmt) throws SQLException {
      String var3 = oci_semaphore;
      synchronized(var3){}

      byte[] var2;
      try {
         var2 = this.get_cursor_bytes((OCIDBStatement)dbstmt);
      } catch (Throwable var6) {
         throw var6;
      }

      return var2;
   }

   native byte[] get_cursor_bytes(OCIDBStatement var1);

   public int getRefCursorBytesSize() throws SQLException {
      String var2 = oci_semaphore;
      synchronized(var2){}

      int var1;
      try {
         var1 = this.get_cursor_size();
      } catch (Throwable var5) {
         throw var5;
      }

      return var1;
   }

   native int get_cursor_size();

   public void setWaitandAutoRollback(DBStatement dbstmt, int autoRollback, int waitOption) throws SQLException {
      String var4 = oci_semaphore;
      synchronized(var4){}

      try {
         this.check_error(this.set_wait_and_auto_rollback((OCIDBStatement)dbstmt, waitOption, autoRollback), (OCIDBStatement)dbstmt);
      } catch (Throwable var6) {
         throw var6;
      }

   }

   native int set_wait_and_auto_rollback(OCIDBStatement var1, int var2, int var3);
}
