package oracle.jdbc.oracore;

import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.driver.OracleConversionInputStream;
import oracle.jdbc.util.RepConversion;

public class JavaConversion implements DBConversion {
   static final boolean DEBUG = false;
   private short char_set;
   public static short[] supp_char_sets = new short[]{1, 31, 870, 871};

   public JavaConversion(short char_set) throws SQLException {
      if (isCharSetValid(char_set)) {
         this.char_set = char_set;
      } else {
         DBError.check_error(-35, "JavaConversion");
      }

   }

   public JavaConversion() throws SQLException {
      this.char_set = 31;
   }

   public static short[] getSupportedCharSets() {
      return supp_char_sets;
   }

   public static boolean isCharSetValid(short charSet) {
      for(int i = 0; i < supp_char_sets.length; ++i) {
         if (supp_char_sets[i] == charSet) {
            return true;
         }
      }

      return false;
   }

   public static short whichCharSetToUse(short charSet, byte oracleVersion) {
      short cs;
      if (isCharSetValid(charSet)) {
         cs = charSet;
      } else {
         cs = 1;
      }

      return cs;
   }

   public byte[] StringToCharBytes(String str) throws SQLException {
      if (str != null) {
         if (this.char_set != 31 && this.char_set != 1) {
            int len = str.length();
            byte[] bytearr = new byte[this.StringUTFLength(str)];
            char[] chararr = new char[len];
            str.getChars(0, len, chararr, 0);
            this.JavaCharsToCHARBytes(chararr, len, bytearr);
            return bytearr;
         } else {
            byte[] bytearr = new byte[str.length()];
            str.getBytes(0, str.length(), bytearr, 0);
            return bytearr;
         }
      } else {
         return null;
      }
   }

   public int StringToWorkbench(String value, byte[] out, int offset) {
      int strLen = value.length();

      int i;
      for(i = 0; i < strLen; ++i) {
         byte character = (byte)value.charAt(i);
         if (character >= 97 && character <= 122) {
            character += -32;
         }

         out[offset + i * 2] = 0;
         out[offset + i * 2 + 1] = character;
      }

      return offset + i * 2;
   }

   public byte[] StringToWorkbench(String value) {
      int strLen = value.length();
      byte[] array = new byte[strLen * 2];
      this.StringToWorkbench(value, array, 0);
      return array;
   }

   public byte[] StringToNetworkRep(String str) throws SQLException {
      return this.StringToCharBytes(str);
   }

   public String CharBytesToString(byte[] array, int data_size) throws SQLException {
      if (this.char_set != 31 && this.char_set != 1) {
         int length = this.UTFStringLength(array, data_size);
         char[] chars = new char[length];
         this.CHARBytesToJavaChars(array, data_size, chars);
         return new String(chars);
      } else {
         return new String(array, 0, 0, data_size);
      }
   }

   public byte[] BigDecimalToNumberBytes(BigDecimal java_num) throws SQLException {
      if (java_num == null) {
         return null;
      } else {
         try {
            OracleNumber n = new OracleNumber(java_num);
            return n.toBytes();
         } catch (CoreException var3) {
            DBError.check_error(-36, "BigDecimalToNumberBytes");
            return null;
         }
      }
   }

   public BigDecimal NumberBytesToBigDecimal(byte[] ora_bytes, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(ora_bytes, data_size);

      try {
         return n.toBigDecimal();
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToBigDecimal");
         return null;
      }
   }

   public byte NumberBytesToByte(byte[] array, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(array, data_size);

      try {
         return n.byteValue();
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToByte");
         return 0;
      }
   }

   public byte[] ByteToNumberBytes(byte x) {
      OracleNumber n = new OracleNumber(x);
      return n.toBytes();
   }

   public short NumberBytesToShort(byte[] array, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(array, data_size);

      try {
         return n.shortValue();
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToShort");
         return 0;
      }
   }

   public byte[] ShortToNumberBytes(short x) {
      OracleNumber n = new OracleNumber(x);
      return n.toBytes();
   }

   public int NumberBytesToInt(byte[] array, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(array, data_size);

      try {
         return n.intValue();
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToInt");
         return 0;
      }
   }

   public byte[] IntToNumberBytes(int x) {
      OracleNumber n = new OracleNumber(x);
      return n.toBytes();
   }

   public long NumberBytesToLong(byte[] array, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(array, data_size);

      try {
         return n.longValue();
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToLong");
         return 0L;
      }
   }

   public byte[] LongToNumberBytes(long x) {
      OracleNumber n = new OracleNumber(x);
      return n.toBytes();
   }

   public float NumberBytesToFloat(byte[] array, int data_size) {
      OracleNumber n = new OracleNumber(array, data_size);
      return n.floatValue();
   }

   public byte[] FloatToNumberBytes(float x) {
      OracleNumber n = new OracleNumber(x);
      return n.toBytes();
   }

   public double NumberBytesToDouble(byte[] array, int data_size) {
      OracleNumber n = new OracleNumber(array, data_size);
      return n.doubleValue();
   }

   public byte[] DoubleToNumberBytes(double x) throws SQLException {
      try {
         OracleNumber n = new OracleNumber(x);
         return n.toBytes();
      } catch (CoreException var4) {
         DBError.check_error(-36, "DoubleToNumberBytes");
         return null;
      }
   }

   public byte[] BooleanToNumberBytes(boolean b) {
      OracleNumber n;
      if (b) {
         n = new OracleNumber(1);
      } else {
         n = new OracleNumber(0);
      }

      return n.toBytes();
   }

   public boolean NumberBytesToBoolean(byte[] array, int data_size) throws SQLException {
      OracleNumber n = new OracleNumber(array, data_size);

      try {
         return n.intValue() != 0;
      } catch (CoreException var4) {
         DBError.check_error(-36, "NumberBytesToBoolean");
         return false;
      }
   }

   public Date DateBytesToDate(byte[] array, int data_size) {
      int[] result = new int[7];

      for(int i = 0; i < 7; ++i) {
         if (array[i] < 0) {
            result[i] = array[i] + 256;
         } else {
            result[i] = array[i];
         }
      }

      int year = (result[0] - 100) * 100 + (result[1] - 100) - 1900;
      return new Date(year, result[2] - 1, result[3]);
   }

   public byte[] DateToDateBytes(Date x) {
      if (x == null) {
         return null;
      } else {
         byte[] result = new byte[7];
         result[0] = 119;
         result[1] = (byte)(((java.util.Date)x).getYear() + 100);
         result[2] = (byte)(((java.util.Date)x).getMonth() + 1);
         result[3] = (byte)((java.util.Date)x).getDate();
         result[4] = 1;
         result[5] = 1;
         result[6] = 1;
         return result;
      }
   }

   public Time DateBytesToTime(byte[] array, int data_size) {
      int[] result = new int[7];

      for(int i = 0; i < 7; ++i) {
         if (array[i] < 0) {
            result[i] = array[i] + 256;
         } else {
            result[i] = array[i];
         }
      }

      return new Time(result[4] - 1, result[5] - 1, result[6] - 1);
   }

   public byte[] TimeToDateBytes(Time x) {
      if (x == null) {
         return null;
      } else {
         byte[] result = new byte[7];
         result[0] = 119;
         result[1] = 100;
         result[2] = 1;
         result[3] = 1;
         result[4] = (byte)(((java.util.Date)x).getHours() + 1);
         result[5] = (byte)(((java.util.Date)x).getMinutes() + 1);
         result[6] = (byte)(((java.util.Date)x).getSeconds() + 1);
         return result;
      }
   }

   public Timestamp DateBytesToTimestamp(byte[] array, int data_size) {
      int[] result = new int[7];

      for(int i = 0; i < 7; ++i) {
         if (array[i] < 0) {
            result[i] = array[i] + 256;
         } else {
            result[i] = array[i];
         }
      }

      int year = (result[0] - 100) * 100 + (result[1] - 100) - 1900;
      return new Timestamp(year, result[2] - 1, result[3], result[4] - 1, result[5] - 1, result[6] - 1, 0);
   }

   public byte[] TimestampToDateBytes(Timestamp x) {
      if (x == null) {
         return null;
      } else {
         byte[] result = new byte[7];
         result[0] = 119;
         result[1] = (byte)(((java.util.Date)x).getYear() + 100);
         result[2] = (byte)(((java.util.Date)x).getMonth() + 1);
         result[3] = (byte)((java.util.Date)x).getDate();
         result[4] = (byte)(((java.util.Date)x).getHours() + 1);
         result[5] = (byte)(((java.util.Date)x).getMinutes() + 1);
         result[6] = (byte)(((java.util.Date)x).getSeconds() + 1);
         return result;
      }
   }

   public int requestLength(int final_length, int conversion) {
      switch (conversion) {
         case 0:
            return final_length;
         case 1:
            return final_length / 2;
         case 2:
            return final_length / 2;
         case 3:
            return final_length / 4;
         case 4:
            return final_length * 2;
         case 5:
            return final_length;
         default:
            return final_length;
      }
   }

   public int JavaCharsToAsciiBytes(char[] chars, int chars_length, byte[] bytes) {
      for(int index = 0; index < chars_length; ++index) {
         bytes[index] = (byte)chars[index];
      }

      return chars_length;
   }

   public int JavaCharsToUnicodeBytes(char[] chars, int chars_length, byte[] bytes) {
      for(int index = 0; index < chars_length; ++index) {
         bytes[2 * index + 1] = (byte)(chars[index] & 255);
         bytes[2 * index] = (byte)(chars[index] >> 8 & 255);
      }

      return chars_length * 2;
   }

   public int RAWBytesToHexChars(byte[] bytes, int bytes_length, char[] chars) {
      for(int i = 0; i < bytes_length; ++i) {
         chars[i * 2 + 1] = (char)RepConversion.nibbleToHex((byte)(bytes[i] & 15));
         chars[i * 2] = (char)RepConversion.nibbleToHex((byte)(bytes[i] >> 4 & 15));
      }

      return bytes_length * 2;
   }

   public int UnicodeBytesToJavaChars(byte[] bytes, int bytes_length, char[] chars) {
      for(int index = 0; index < bytes_length / 2; ++index) {
         int hibyte = bytes[2 * index];
         hibyte <<= 8;
         chars[index] = (char)(hibyte | bytes[2 * index + 1] & 255);
      }

      return bytes_length / 2;
   }

   void failUTFConversion() throws SQLException {
      DBError.check_error(-37, "failUTFConversion");
   }

   public int CHARBytesToJavaChars(byte[] bytes, int bytes_length, char[] chars) throws SQLException {
      if (this.char_set != 31 && this.char_set != 1) {
         int chars_index = 0;
         int bytes_index = 0;

         while(bytes_index < bytes_length) {
            if (chars_index >= chars.length) {
               DBError.check_error(-39, "JavaCharsToCHARBytes");
            }

            byte c = bytes[bytes_index++];
            int b = c & 240;
            switch (b / 16) {
               case 0:
               case 1:
               case 2:
               case 3:
               case 4:
               case 5:
               case 6:
               case 7:
                  chars[chars_index++] = (char)c;
                  break;
               case 12:
               case 13:
                  if (bytes_index >= bytes.length) {
                     this.failUTFConversion();
                  }

                  int c2 = bytes[bytes_index++];
                  if ((c2 & 192) != 128) {
                     this.failUTFConversion();
                  }

                  chars[chars_index++] = (char)((c & 31) << 6 | c2 & 63);
                  break;
               case 14:
                  if (bytes_index + 1 >= bytes_length) {
                     this.failUTFConversion();
                  }

                  int c2 = bytes[bytes_index++];
                  int c3 = bytes[bytes_index++];
                  if ((c2 & 192) != 128 || (c3 & 192) != 128) {
                     this.failUTFConversion();
                  }

                  chars[chars_index++] = (char)((c & 15) << 12 | (c2 & 63) << 6 | c3 & 63);
                  break;
               default:
                  this.failUTFConversion();
            }
         }

         return chars_index;
      } else {
         for(int index = 0; index < bytes_length; ++index) {
            chars[index] = (char)bytes[index];
         }

         return bytes_length;
      }
   }

   int StringUTFLength(String s) {
      int utflen = 0;
      int slen = s.length();

      for(int i = 0; i < slen; ++i) {
         int c = s.charAt(i);
         if (c >= 1 && c <= 127) {
            ++utflen;
         } else if (c > 2047) {
            utflen += 3;
         } else {
            utflen += 2;
         }
      }

      return utflen;
   }

   int UTFStringLength(byte[] bytes, int length) {
      int strlen = 0;

      for(int bytes_index = 0; bytes_index < length; ++strlen) {
         switch ((bytes[bytes_index] & 240) >> 4) {
            case 12:
            case 13:
               bytes_index += 2;
               break;
            case 14:
               bytes_index += 3;
               break;
            default:
               ++bytes_index;
         }
      }

      return strlen;
   }

   public int JavaCharsToCHARBytes(char[] chars, int chars_length, byte[] bytes) throws SQLException {
      if (this.char_set != 31 && this.char_set != 1) {
         int utflen = 0;

         for(int i = 0; i < chars_length; ++i) {
            int c = chars[i];
            if (c >= 1 && c <= 127) {
               ++utflen;
            } else if (c > 2047) {
               utflen += 3;
            } else {
               utflen += 2;
            }
         }

         if (utflen > bytes.length) {
            DBError.check_error(-38, "JavaCharsToCHARBytes");
         }

         int byte_index = 0;

         for(int var9 = 0; var9 < chars_length; ++var9) {
            int c = chars[var9];
            if (c >= 1 && c <= 127) {
               bytes[byte_index++] = (byte)c;
            } else if (c > 2047) {
               bytes[byte_index++] = (byte)(224 | c >> 12 & 15);
               bytes[byte_index++] = (byte)(128 | c >> 6 & 63);
               bytes[byte_index++] = (byte)(128 | c & 63);
            } else {
               bytes[byte_index++] = (byte)(192 | c >> 6 & 31);
               bytes[byte_index++] = (byte)(128 | c & 63);
            }
         }

         return byte_index;
      } else {
         for(int i = 0; i < chars_length; ++i) {
            bytes[i] = (byte)chars[i];
         }

         return chars_length;
      }
   }

   public int AsciiBytesToJavaChars(byte[] bytes, int bytes_read, char[] chars) {
      for(int index = 0; index < bytes_read; ++index) {
         chars[index] = (char)bytes[index];
      }

      return bytes_read;
   }

   public InputStream ConvertStream(InputStream stream, int conversion) {
      return new OracleConversionInputStream(this, stream, conversion);
   }

   public short getCharacterSet() {
      return this.char_set;
   }
}
