package oracle.jdbc.oracore;

import java.math.BigDecimal;
import java.math.BigInteger;

public final class OracleNumber {
   private static final byte MAXBYTES = 22;
   private static final int SIGNBIT = 128;
   private static final byte ORANUM_DIG = 20;
   private static final byte XPONOFFSET = 65;
   private static final double ORANUM_FBASE = (double)100.0F;
   private static final int ORANUM_BASE = 100;
   private static final byte IEEE_DBL_DIG = 15;
   private static final byte IEEE_FLT_DIG = 6;
   private static final int ORANUM_MAX_EXP = 62;
   private static final int ORANUM_MIN_EXP = -65;
   private static final byte BYTE_MAX_VALUE = 127;
   private static final byte BYTE_MIN_VALUE = -128;
   private static final short SHORT_MAX_VALUE = Short.MAX_VALUE;
   private static final short SHORT_MIN_VALUE = Short.MIN_VALUE;
   private byte flag;
   private int exponent;
   private byte[] mantissa;
   private byte length;

   public OracleNumber() {
      this.mantissa = new byte[20];
      this.setToZero();
   }

   public OracleNumber(byte byteNum) {
      this((long)byteNum);
   }

   public OracleNumber(short shortNum) {
      this((long)shortNum);
   }

   public OracleNumber(int intNum) {
      this((long)intNum);
   }

   public OracleNumber(long longNum) {
      this.mantissa = new byte[20];
      if (longNum == 0L) {
         this.setToZero();
      } else {
         this.flag = (byte)(longNum < 0L ? -1 : 1);
         byte[] temp = new byte[20];

         int i;
         for(i = 0; longNum != 0L; ++i) {
            temp[i] = (byte)((int)Math.abs(longNum % 100L));
            longNum /= 100L;
         }

         --i;
         this.exponent = i;
         this.length = 0;

         for(int j = this.exponent; this.length <= this.exponent; --j) {
            this.mantissa[this.length] = temp[j];
            ++this.length;
         }

         while(i > 0 && this.mantissa[i--] == 0) {
            --this.length;
         }

      }
   }

   public OracleNumber(float floatNum) {
      this.mantissa = new byte[20];
      if (floatNum == 0.0F) {
         this.setToZero();
      } else if (floatNum == Float.POSITIVE_INFINITY) {
         this.setToPosInf();
      } else if (floatNum == Float.NEGATIVE_INFINITY) {
         this.setToNegInf();
      } else {
         float[][] powerTable = new float[][]{{16.0F, 1.0E32F, 1.0E-32F}, {8.0F, 1.0E16F, 1.0E-16F}, {4.0F, 1.0E8F, 1.0E-8F}, {2.0F, 10000.0F, 1.0E-4F}, {1.0F, 100.0F, 0.01F}};
         this.flag = (byte)(floatNum < 0.0F ? -1 : 1);
         floatNum = Math.abs(floatNum);
         this.exponent = 0;
         if (floatNum < 1.0F) {
            for(int i = 0; i < 5; ++i) {
               if (powerTable[i][2] >= floatNum) {
                  this.exponent -= (int)powerTable[i][0];
                  floatNum *= powerTable[i][1];
               }
            }

            if (floatNum < 1.0F) {
               --this.exponent;
               floatNum = (float)((double)floatNum * (double)100.0F);
            }
         } else {
            for(int i = 0; i < 5; ++i) {
               if (powerTable[i][1] <= floatNum) {
                  this.exponent += (int)powerTable[i][0];
                  floatNum *= powerTable[i][2];
               }
            }
         }

         byte digits;
         boolean fullDig;
         if (floatNum >= 10.0F) {
            fullDig = true;
            digits = 3;
         } else {
            fullDig = false;
            digits = 4;
         }

         int oIndex = 0;

         byte digit;
         for(digit = (byte)((int)floatNum); oIndex < digits; ++oIndex) {
            this.mantissa[oIndex] = digit;
            floatNum = (floatNum - (float)digit) * 100.0F;
            digit = (byte)((int)floatNum);
         }

         if (fullDig) {
            if (digit >= 50) {
               ++this.mantissa[oIndex];
            }
         } else {
            this.mantissa[oIndex] = (byte)((this.mantissa[oIndex] + 5) / 10 * 10);
         }

         while(this.mantissa[oIndex] == 100) {
            if (digits == 1) {
               ++this.exponent;
               this.mantissa[oIndex] = 1;
               break;
            }

            --oIndex;
            --digits;
            ++this.mantissa[oIndex];
         }

         while(oIndex != 0 && this.mantissa[oIndex] == 0) {
            --digits;
            --oIndex;
         }

         this.length = (byte)(digits + 1);
      }
   }

   public OracleNumber(double doubleNum) throws CoreException {
      this.mantissa = new byte[20];
      if (doubleNum != (double)0.0F && doubleNum != (double)-0.0F) {
         if (doubleNum == Double.POSITIVE_INFINITY) {
            this.setToPosInf();
         } else if (doubleNum == Double.NEGATIVE_INFINITY) {
            this.setToNegInf();
         } else {
            double[][] powerTable = new double[][]{{(double)128.0F, 1.0E256, 1.0E-256}, {(double)64.0F, 1.0E128, 1.0E-128}, {(double)32.0F, 1.0E64, 1.0E-64}, {(double)16.0F, 1.0E32, 1.0E-32}, {(double)8.0F, 1.0E16, 1.0E-16}, {(double)4.0F, (double)1.0E8F, 1.0E-8}, {(double)2.0F, (double)10000.0F, 1.0E-4}, {(double)1.0F, (double)100.0F, 0.01}};
            this.flag = (byte)(doubleNum < (double)0.0F ? -1 : 1);
            doubleNum = Math.abs(doubleNum);
            this.exponent = 0;
            if (doubleNum < (double)1.0F) {
               for(int i = 0; i < 8; ++i) {
                  if (powerTable[i][2] >= doubleNum) {
                     this.exponent -= (int)powerTable[i][0];
                     doubleNum *= powerTable[i][1];
                  }
               }

               if (doubleNum < (double)1.0F) {
                  --this.exponent;
                  doubleNum *= (double)100.0F;
               }
            } else {
               for(int i = 0; i < 8; ++i) {
                  if (powerTable[i][1] <= doubleNum) {
                     this.exponent += (int)powerTable[i][0];
                     doubleNum *= powerTable[i][2];
                  }
               }
            }

            if (this.exponent > 62) {
               this.setToPosInf();
               throw new CoreException((byte)1);
            } else if (this.exponent < -65) {
               this.setToNegInf();
               throw new CoreException((byte)0);
            } else {
               boolean fullDig = !(doubleNum >= (double)10.0F);
               byte digits = 8;
               int oIndex = 0;

               byte digit;
               for(digit = (byte)((int)doubleNum); oIndex < digits; ++oIndex) {
                  this.mantissa[oIndex] = digit;
                  doubleNum = (doubleNum - (double)digit) * (double)100.0F;
                  digit = (byte)((int)doubleNum);
               }

               if (fullDig) {
                  if (digit >= 50) {
                     ++this.mantissa[oIndex];
                  }
               } else {
                  this.mantissa[oIndex] = (byte)((this.mantissa[oIndex] + 5) / 10 * 10);
               }

               while(this.mantissa[oIndex] == 100) {
                  if (digits == 1) {
                     ++this.exponent;
                     this.mantissa[oIndex] = 1;
                     break;
                  }

                  --oIndex;
                  --digits;
                  ++this.mantissa[oIndex];
               }

               while(oIndex != 0 && this.mantissa[oIndex] == 0) {
                  --digits;
                  --oIndex;
               }

               this.length = (byte)(digits + 1);
            }
         }
      } else {
         this.setToZero();
      }
   }

   public OracleNumber(BigDecimal n) throws CoreException {
      this.mantissa = new byte[20];
      this.exponent = 0;
      BigDecimal NEGZERO = new BigDecimal("-0");
      BigDecimal ZERO = new BigDecimal("0");
      BigDecimal ONE = new BigDecimal("1");
      if (!n.equals(ZERO) && !n.equals(NEGZERO)) {
         boolean lessThanOne = false;
         int scale = n.scale();
         String s;
         if (n.compareTo(ZERO) == -1) {
            this.flag = -1;
            BigDecimal NEGATIVEONE = new BigDecimal((double)-1.0F);
            String tmp = n.toString();
            BigDecimal temp = new BigDecimal(tmp);
            temp = temp.multiply(NEGATIVEONE);
            s = temp.toString();
            if (n.compareTo(NEGATIVEONE) == 1) {
               lessThanOne = true;
            }
         } else {
            this.flag = 1;
            s = n.toString();
            if (n.compareTo(ONE) == -1) {
               lessThanOne = true;
            }
         }

         int start = 0;
         int end = s.length();
         int mantIndex = 0;
         if (lessThanOne) {
            while(s.substring(end - 1, end).equals("0")) {
               --end;
               --scale;
            }

            start = 2;
            --this.exponent;

            while(start < end - 1 && s.substring(start, start + 2).equals("00")) {
               start += 2;
               --this.exponent;
            }

            if (this.exponent <= -65) {
               this.setToNegInf();
               throw new CoreException((byte)0);
            } else {
               mantIndex = (end - start) / 2;
               if (mantIndex >= 20) {
                  this.setToZero();
                  throw new CoreException((byte)0);
               } else {
                  if ((end - start) % 2 != 0) {
                     Integer oneDig = new Integer(s.substring(end - 1, end));
                     this.mantissa[mantIndex] = (byte)(oneDig * 10);
                     ++this.length;
                     --end;
                  }

                  while(end > start) {
                     --mantIndex;
                     Integer oneDig = new Integer(s.substring(end - 2, end));
                     this.mantissa[mantIndex] = (byte)oneDig;
                     end -= 2;
                     ++this.length;
                  }

               }
            }
         } else {
            while(s.substring(end - 1, end).equals("0") && scale > 0) {
               --end;
               --scale;
            }

            if (scale == 0 && end > 1) {
               if (s.substring(end - 1, end).equals(".")) {
                  --end;
               }

               while(end > 1 && s.substring(end - 2, end).equals("00")) {
                  end -= 2;
                  ++this.exponent;
               }
            }

            if (this.exponent > 62 || end - start > 40) {
               this.setToPosInf();
               throw new CoreException((byte)1);
            } else {
               int wholeDigs = scale == 0 ? end : end - scale - 1;
               Integer oneDig;
               if (wholeDigs % 2 != 0) {
                  oneDig = new Integer(s.substring(start, start + 1));
                  ++start;
                  --wholeDigs;
               } else {
                  oneDig = new Integer(s.substring(start, start + 2));
                  start += 2;
                  wholeDigs -= 2;
               }

               this.mantissa[mantIndex] = (byte)oneDig;
               ++mantIndex;
               ++this.length;

               while(wholeDigs > 0) {
                  oneDig = new Integer(s.substring(start, start + 2));
                  this.mantissa[mantIndex] = (byte)oneDig;
                  ++mantIndex;
                  start += 2;
                  ++this.exponent;
                  wholeDigs -= 2;
                  ++this.length;
               }

               if (start < end) {
                  if (scale % 2 != 0) {
                     mantIndex += scale / 2;
                     oneDig = new Integer(s.substring(end - 1, end));
                     this.mantissa[mantIndex] = (byte)(oneDig * 10);
                     --end;
                     --scale;
                  } else {
                     mantIndex += scale / 2 - 1;
                     oneDig = new Integer(s.substring(end - 2, end));
                     this.mantissa[mantIndex] = (byte)oneDig;
                     end -= 2;
                     scale -= 2;
                  }

                  ++this.length;
                  --mantIndex;
               }

               while(scale > 0) {
                  oneDig = new Integer(s.substring(end - 2, end));
                  this.mantissa[mantIndex] = (byte)oneDig;
                  --mantIndex;
                  end -= 2;
                  scale -= 2;
                  ++this.length;
               }

            }
         }
      } else {
         this.setToZero();
      }
   }

   public OracleNumber(BigInteger intNum) {
      this.mantissa = new byte[20];
      new BigInteger("0");
      switch (intNum.signum()) {
         case 0:
            this.setToZero();
         case -1:
            this.flag = -1;
            intNum = intNum.negate();
         case 1:
            this.flag = 1;
         default:
            byte[] temp = new byte[20];
            String baseStr = Integer.toString(100);
            BigInteger base = new BigInteger(baseStr);

            int nIndex;
            for(nIndex = 0; intNum.signum() != 0; ++nIndex) {
               temp[nIndex] = (byte)((int)(intNum.longValue() % 100L));
               intNum = intNum.divide(base);
            }

            --nIndex;
            this.exponent = nIndex;
            this.length = 0;

            for(int j = this.exponent; this.length <= this.exponent; --j) {
               this.mantissa[this.length] = temp[j];
               ++this.length;
            }

            while(nIndex > 0 && this.mantissa[nIndex--] == 0) {
               --this.length;
            }

      }
   }

   public OracleNumber(byte[] oraNum) {
      this(oraNum, 0);
   }

   public OracleNumber(byte[] oraNum, int size) {
      byte lengthByte;
      byte exponentByte;
      int arrayIndex;
      if (size == 0) {
         lengthByte = oraNum[0];
         exponentByte = oraNum[1];
         arrayIndex = 2;
      } else {
         lengthByte = (byte)size;
         exponentByte = oraNum[0];
         arrayIndex = 1;
      }

      this.mantissa = new byte[20];
      if (lengthByte == 1 && exponentByte == -128) {
         this.setToZero();
      } else if (lengthByte == 2 && exponentByte == -1 && oraNum[arrayIndex] == 101) {
         this.setToPosInf();
      } else if (lengthByte == 1 && exponentByte == 0) {
         this.setToNegInf();
      } else {
         this.flag = (byte)((exponentByte & 128) == 128 ? 1 : -1);
         boolean positive = this.isPositive();
         int mantIndex = 0;
         if (positive) {
            this.exponent = (byte)((exponentByte & -129) - 65);

            for(this.length = (byte)(lengthByte - 1); mantIndex < this.length; ++mantIndex) {
               this.mantissa[mantIndex] = (byte)(oraNum[arrayIndex] - 1);
               ++arrayIndex;
            }
         } else {
            this.exponent = (byte)((~exponentByte & -129) - 65);

            for(this.length = (byte)(lengthByte - 2); mantIndex < this.length; ++mantIndex) {
               this.mantissa[mantIndex] = (byte)(101 - oraNum[arrayIndex]);
               ++arrayIndex;
            }

            if (oraNum[arrayIndex] != 102) {
               this.mantissa[mantIndex] = oraNum[arrayIndex];
               ++this.length;
            }
         }

      }
   }

   public BigDecimal toBigDecimal() throws CoreException {
      if (this.isZero()) {
         BigDecimal b = new BigDecimal("0.0");
         return b;
      } else if (!this.isPosInf() && !this.isNegInf()) {
         String temp = this.toStringInternal();
         BigDecimal b = new BigDecimal(temp);
         return b;
      } else {
         throw new CoreException((byte)1);
      }
   }

   public BigInteger toBigInteger() throws CoreException {
      long temp = this.longValue();
      String str = Long.toString(temp);
      BigInteger intNum = new BigInteger(str);
      return intNum;
   }

   public byte[] toBytes() {
      if (this.isZero()) {
         byte[] bytes = new byte[1];
         bytes[0] = -128;
         return bytes;
      } else if (this.isPosInf()) {
         byte[] bytes = new byte[2];
         bytes[0] = -1;
         bytes[1] = 101;
         return bytes;
      } else if (this.isNegInf()) {
         byte[] bytes = new byte[1];
         bytes[0] = 0;
         return bytes;
      } else {
         boolean positive = this.isPositive();
         byte[] bytes;
         if (positive) {
            bytes = new byte[this.length + 1];
            bytes[0] = (byte)(this.exponent + 128 + 65);

            for(int i = 0; i < this.length; ++i) {
               bytes[i + 1] = (byte)(this.mantissa[i] + 1);
            }
         } else {
            if (this.length < 20) {
               bytes = new byte[this.length + 2];
            } else {
               bytes = new byte[this.length + 1];
            }

            bytes[0] = (byte)(~(this.exponent + 128 + 65));

            int i;
            for(i = 0; i < this.length; ++i) {
               bytes[i + 1] = (byte)(101 - this.mantissa[i]);
            }

            if (i < 20) {
               bytes[i + 1] = 102;
            }
         }

         return bytes;
      }
   }

   public byte[] toBytes22() {
      byte[] bytes = new byte[22];
      if (this.isZero()) {
         bytes[0] = 1;
         bytes[1] = -128;
         return bytes;
      } else if (this.isPosInf()) {
         bytes[0] = 2;
         bytes[1] = -1;
         bytes[2] = 101;
         return bytes;
      } else if (this.isNegInf()) {
         bytes[0] = 1;
         bytes[1] = 0;
         return bytes;
      } else {
         boolean positive = this.isPositive();
         bytes[1] = (byte)(this.exponent + 128 + 65);
         if (positive) {
            bytes[0] = (byte)(this.length + 1);

            for(int i = 0; i < this.length; ++i) {
               bytes[i + 2] = (byte)(this.mantissa[i] + 1);
            }
         } else {
            bytes[0] = (byte)(this.length + 2);
            bytes[1] = (byte)(~bytes[1]);

            int i;
            for(i = 0; i < this.length; ++i) {
               bytes[i + 2] = (byte)(101 - this.mantissa[i]);
            }

            if (i < 20) {
               bytes[i + 2] = 102;
            } else {
               --bytes[0];
            }
         }

         return bytes;
      }
   }

   public byte byteValue() throws CoreException {
      long temp = 0L;
      temp = this.longValue();
      if (temp > 127L) {
         throw new CoreException((byte)1);
      } else if (temp < -128L) {
         throw new CoreException((byte)0);
      } else {
         return (byte)((int)temp);
      }
   }

   public short shortValue() throws CoreException {
      long temp = 0L;
      temp = this.longValue();
      if (temp > 32767L) {
         throw new CoreException((byte)1);
      } else if (temp < -32768L) {
         throw new CoreException((byte)0);
      } else {
         return (short)((int)temp);
      }
   }

   public int intValue() throws CoreException {
      long temp = 0L;
      temp = this.longValue();
      if (temp > 2147483647L) {
         throw new CoreException((byte)1);
      } else if (temp < -2147483648L) {
         throw new CoreException((byte)0);
      } else {
         return (int)temp;
      }
   }

   public long longValue() throws CoreException {
      if (this.isZero()) {
         return 0L;
      } else if (!this.isPosInf() && !this.isNegInf()) {
         long longNum = 0L;
         long temp = 0L;
         int convDigs = this.length > this.exponent + 1 ? this.exponent + 1 : this.length;

         for(int i = 0; i < convDigs; ++i) {
            longNum = longNum * 100L + (long)this.mantissa[i];
         }

         for(int i = this.exponent - this.length; i >= 0; --i) {
            longNum *= 100L;
            if (longNum > 92233720368547758L) {
               throw new CoreException((byte)1);
            }
         }

         if (!this.isPositive()) {
            longNum = -longNum;
         }

         return longNum;
      } else {
         throw new CoreException((byte)1);
      }
   }

   public float floatValue() {
      return (float)this.doubleValue();
   }

   public double doubleValue() {
      if (this.isZero()) {
         return (double)0.0F;
      } else if (this.isPosInf()) {
         Double Number = new Double(Double.POSITIVE_INFINITY);
         return Number;
      } else if (this.isNegInf()) {
         Double Number = new Double(Double.NEGATIVE_INFINITY);
         return Number;
      } else {
         double number = (double)0.0F;
         int i = 0;
         int j = 0;

         int xpon;
         for(xpon = this.exponent + 1; i < this.length && j < 15; --xpon) {
            number = number * (double)100.0F + (double)this.mantissa[i];
            ++i;
            ++j;
         }

         if (i < this.length && this.mantissa[i] >= 50) {
            ++number;
         }

         if (xpon > 0) {
            while(xpon > 0) {
               number *= (double)100.0F;
               --xpon;
            }
         } else {
            while(xpon < 0) {
               number /= (double)100.0F;
               ++xpon;
            }
         }

         return this.isPositive() ? number : -number;
      }
   }

   public boolean isPositive() {
      return this.flag == 1 || this.flag == 0 || this.flag == 2;
   }

   public void negate() {
      if (this.flag == 1) {
         this.flag = -1;
      } else if (this.flag == -1) {
         this.flag = 1;
      }

   }

   private String toStringInternal() {
      if (this.isZero()) {
         String s = new String("0");
         return s;
      } else if (this.isPosInf()) {
         Double temp = new Double(Double.POSITIVE_INFINITY);
         return temp.toString();
      } else if (this.isNegInf()) {
         Double temp = new Double(Double.NEGATIVE_INFINITY);
         return temp.toString();
      } else {
         StringBuffer sb = new StringBuffer();
         sb.append(this.mantissa[0]);
         int i = 1;

         int xpon;
         for(xpon = this.exponent; i < this.length; --xpon) {
            if (this.mantissa[i] / 10 == 0) {
               sb.append(0);
            }

            sb.append(this.mantissa[i]);
            ++i;
         }

         if (xpon > 0) {
            while(xpon > 0) {
               sb.append("00");
               --xpon;
            }
         } else {
            int fixedPoint = sb.length() + xpon * 2;
            if (fixedPoint > 0) {
               sb.insert(fixedPoint, ".");
            } else {
               while(fixedPoint < 0) {
                  sb.insert(0, 0);
                  ++fixedPoint;
               }

               sb.insert(0, ".");
            }
         }

         if (!this.isPositive()) {
            sb.insert(0, "-");
         }

         return sb.toString();
      }
   }

   private void setToZero() {
      this.flag = 0;
   }

   private void setToPosInf() {
      this.flag = 2;
   }

   private void setToNegInf() {
      this.flag = -2;
   }

   private boolean isZero() {
      return this.flag == 0;
   }

   private boolean isPosInf() {
      return this.flag == 2;
   }

   private boolean isNegInf() {
      return this.flag == -2;
   }
}
