package oracle.jdbc.ttc7;

import java.io.IOException;
import java.net.InetAddress;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.oracore.JavaConversion;
import oracle.jdbc.util.RepConversion;
import oracle.jdbc.util.WorkBench;
import oracle.sqlnet.SQLnet;

public class O3log extends TTIfun {
   private final boolean DEBUG = false;
   private byte[] user;
   private byte[] password;
   private int auditFlag;
   private long connectFlag;
   private int revisionLev;
   private int padO2UPtr;
   private boolean padO2U = false;
   private byte[] terminal = new byte[0];
   private byte[] machine = new byte[0];
   private byte[] sysUserName = new byte[0];
   private int sizeofUCAUAC = 4096;
   private byte[] processID = new byte[0];
   private byte[] programName = new byte[0];
   private short svrAttrPtr;
   private int lenSvrAttrs;
   private boolean svrDataO2U = false;
   private long svrDataO2UPtr;
   private int lenSvrData;
   private int svrInfoLen = 16;
   private boolean svrInfoO2U = false;
   private byte[] svrInfoO2UPtr;
   private boolean retLenO2U;
   private int retLenO2UPtr;
   private final int metaDataSize;
   private int ptrDataSize;
   public byte[] encryptedSK;

   public O3log(TTCTypeRep types, SQLnet sqlnet, JavaConversion conv, String user) throws IOException, SQLException {
      super(types, sqlnet, conv);
      this.svrInfoO2UPtr = new byte[this.svrInfoLen];
      this.retLenO2U = false;
      this.metaDataSize = 118;
      this.user = conv.StringToCharBytes(user);
      this.password = new byte[0];
      this.terminal = conv.StringToCharBytes("unknown");
      this.machine = conv.StringToCharBytes(InetAddress.getLocalHost().getHostName());
      this.sysUserName = conv.StringToCharBytes("jdbcuser");
      this.programName = conv.StringToCharBytes("JDBC-1.0-Client");
      super.ttcCode = 3;
      super.funCode = 82;
      super.seqNumber = 0;
      this.ptrDataSize = this.user.length + this.password.length + this.terminal.length + this.machine.length + this.sysUserName.length + this.processID.length + this.programName.length + 20;
      ((TTIMsg)this).initBuffers(120, this.ptrDataSize);
      this.svrInfoO2U = true;
      this.retLenO2U = true;
      this.marshal();
   }

   public O3log(TTCTypeRep types, SQLnet sqlnet, JavaConversion conv, String user, String password, byte[] encryptedSK) throws IOException, SQLException {
      super(types, sqlnet, conv);
      this.svrInfoO2UPtr = new byte[this.svrInfoLen];
      this.retLenO2U = false;
      this.metaDataSize = 118;
      if (encryptedSK.length > 16) {
         DBError.check_error((short)106);
      }

      WorkBench wb = new WorkBench();
      byte[] ePwdOneWay = wb.normO(user, password);
      byte[] sessionKey = wb.decrypt(ePwdOneWay, encryptedSK);
      byte[] passwordNet = conv.StringToNetworkRep(password);
      byte pwdPadLen;
      if (passwordNet.length % 8 > 0) {
         pwdPadLen = (byte)(8 - passwordNet.length % 8);
      } else {
         pwdPadLen = 0;
      }

      byte[] paddedPwd = new byte[passwordNet.length + pwdPadLen];
      System.arraycopy(passwordNet, 0, paddedPwd, 0, passwordNet.length);
      byte[] ePwdOnSessKey = wb.encrypt(sessionKey, paddedPwd);
      this.password = new byte[2 * paddedPwd.length + 1];
      if (this.password.length < 2 * ePwdOnSessKey.length) {
         DBError.check_error((short)106);
      }

      RepConversion.bArray2nibbles(ePwdOnSessKey, this.password);
      this.password[this.password.length - 1] = RepConversion.nibbleToHex(pwdPadLen);
      this.user = conv.StringToCharBytes(user);
      super.ttcCode = 3;
      super.funCode = 81;
      super.seqNumber = 0;
      this.ptrDataSize = this.user.length + this.password.length + this.terminal.length + this.machine.length + this.sysUserName.length + this.processID.length + this.programName.length + 20;
      ((TTIMsg)this).initBuffers(120, this.ptrDataSize);
      this.marshal();
   }

   private void marshal() throws IOException {
      ((TTIMsg)this).marshalUB1((short)super.ttcCode, false);
      ((TTIMsg)this).marshalUB1((short)super.funCode, false);
      ((TTIMsg)this).marshalUB1((short)super.seqNumber, false);
      ((TTIMsg)this).marshalB1Array(this.user, true);
      ((TTIMsg)this).marshalSB4(this.user.length, false);
      ((TTIMsg)this).marshalB1Array(this.password, true);
      ((TTIMsg)this).marshalSB4(this.password.length, false);
      ((TTIMsg)this).marshalSB4(this.auditFlag, false);
      ((TTIMsg)this).marshalUB4(this.connectFlag, false);
      ((TTIMsg)this).marshalSB4(this.revisionLev, false);
      ((TTIMsg)this).marshalO2U(this.padO2U);
      ((TTIMsg)this).marshalB1Array(this.terminal, true);
      ((TTIMsg)this).marshalSB4(this.terminal.length, false);
      ((TTIMsg)this).marshalB1Array(this.machine, true);
      ((TTIMsg)this).marshalSB4(this.machine.length, false);
      ((TTIMsg)this).marshalB1Array(this.sysUserName, true);
      ((TTIMsg)this).marshalSB4(this.sysUserName.length, false);
      ((TTIMsg)this).marshalSB4(this.sizeofUCAUAC, false);
      ((TTIMsg)this).marshalB1Array(this.processID, true);
      ((TTIMsg)this).marshalSB4(this.processID.length, false);
      ((TTIMsg)this).marshalB1Array(this.programName, true);
      ((TTIMsg)this).marshalSB4(this.programName.length, false);
      ((TTIMsg)this).marshalSB2(this.svrAttrPtr, true);
      ((TTIMsg)this).marshalSB4(this.lenSvrAttrs, false);
      ((TTIMsg)this).marshalO2U(this.svrDataO2U);
      ((TTIMsg)this).marshalSB4(this.lenSvrData, false);
      ((TTIMsg)this).marshalO2U(this.svrInfoO2U);
      ((TTIMsg)this).marshalSB4(this.svrInfoLen, false);
      ((TTIMsg)this).marshalO2U(this.retLenO2U);
   }

   public void receive1st() throws SQLException, IOException {
      boolean rpaProcessed = false;

      while(true) {
         byte code = ((TTIMsg)this).unmarshalSB1();
         switch (code) {
            case 4:
               TTIoer oer = new TTIoer(super.types, super.sqlnet, super.conv);
               oer.unmarshal();
               oer.processError();
               return;
            case 8:
               if (rpaProcessed) {
                  DBError.check_error((short)1);
               }

               int eskLength = ((TTIMsg)this).unmarshalUB2();
               this.encryptedSK = ((TTIMsg)this).unmarshalNBytes(eskLength);
               rpaProcessed = true;
               break;
            default:
               DBError.check_error((short)1);
         }
      }
   }

   public void receive2nd() throws SQLException, IOException {
      while(true) {
         byte code = ((TTIMsg)this).unmarshalSB1();
         switch (code) {
            case 8:
               DBError.check_error((short)1);
            case 4:
               TTIoer oer = new TTIoer(super.types, super.sqlnet, super.conv);
               oer.unmarshal();
               oer.processError();
               return;
            default:
               DBError.check_error((short)1);
         }
      }
   }

   public byte[] getEncryptedSK() {
      return this.encryptedSK;
   }
}
