package oracle.jdbc.ttc7;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Properties;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBColumn;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBData;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.dbaccess.DBStatement;
import oracle.jdbc.dbaccess.DBType;
import oracle.jdbc.oracore.JavaConversion;
import oracle.sqlnet.SQLnet;
import oracle.sqlnet.SQLnetBufferedOutputStream;

public class TTC7Protocol implements DBAccess {
   private final boolean DEBUG = false;
   private final boolean DEBUGOPTIONS = false;
   private final byte TTC_NOTLOGGEDON;
   private final byte TTC_LOGGEDON = 1;
   private static final byte PARSE_CALL = 1;
   private static final byte PARSE_EXECUTE_CALL = 2;
   private static final byte EXECUTE_FETCH_CALL = 3;
   private static final byte PARSE_EXECUTE_FETCH_CALL = 4;
   private static final byte FETCH_CALL = 5;
   private static final int STREAM_CHUNK_SIZE = 255;
   private static final int REFCURSOR_SIZE = 5;
   private static final String DEFAULT_CONNECT_STRING = "localhost:1521:demo";
   private JavaConversion conv;
   private byte state = 0;
   private SQLnet sqlnet;
   private TTCTypeRep types;
   private TTIpro pro;
   private TTIdty dty;
   private TTIrxd rxd;
   private O3log log1;
   private O3log log2;
   private Oversion ver;
   private Oopen opencall;
   private Odscrarr describe;
   private Oall7 all7;
   private Oclose close;
   private Ocommoncall commoncall;

   public synchronized DBConversion logon(String user, String password, String connection, Properties info) throws SQLException, IOException {
      if (this.state > 0) {
         DBError.check_error((short)200);
      }

      if (user == null || password == null) {
         DBError.check_error((short)205);
      }

      if (connection == null) {
         connection = "localhost:1521:demo";
      }

      this.conv = this.connect(connection);
      this.log1 = new O3log(this.types, this.sqlnet, this.conv, user);
      this.log1.send();
      this.log1.receive1st();
      this.log2 = new O3log(this.types, this.sqlnet, this.conv, user, password, this.log1.encryptedSK);
      this.log2.send();
      this.log2.receive2nd();
      this.ver = new Oversion(this.types, this.sqlnet, this.conv);
      this.ver.send();
      this.ver.receive();
      this.state = 1;
      return this.conv;
   }

   public synchronized void logoff() throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)201);
      }

      this.commoncall = new Ocommoncall(this.types, this.sqlnet, this.conv, (byte)9);
      this.commoncall.send();
      this.commoncall.receive();
      this.sqlnet.Disconnect();
      this.state = 0;
   }

   public synchronized void commit() throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.commoncall = new Ocommoncall(this.types, this.sqlnet, this.conv, (byte)14);
      this.commoncall.send();
      this.commoncall.receive();
   }

   public synchronized void rollback() throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.commoncall = new Ocommoncall(this.types, this.sqlnet, this.conv, (byte)15);
      this.commoncall.send();
      this.commoncall.receive();
   }

   public synchronized void setAutoCommit(boolean on) throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      if (on) {
         this.commoncall = new Ocommoncall(this.types, this.sqlnet, this.conv, (byte)12);
      } else {
         this.commoncall = new Ocommoncall(this.types, this.sqlnet, this.conv, (byte)13);
      }

      this.commoncall.send();
      this.commoncall.receive();
   }

   public synchronized DBStatement open() throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.opencall = new Oopen(this.types, this.sqlnet, this.conv);
      this.opencall.send();
      int cursorid = this.opencall.receive();
      return new TTCStatement(cursorid);
   }

   public synchronized void closeQuery(DBStatement stmt) throws SQLException, IOException {
      TTCStatement ttcStmt = (TTCStatement)stmt;
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.close = new Oclose(this.types, this.sqlnet, this.conv, ttcStmt, (byte)20);
      this.close.send();
      this.close.receive();
   }

   public synchronized void close(DBStatement stmt) throws SQLException, IOException {
      TTCStatement ttcStmt = (TTCStatement)stmt;
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.close = new Oclose(this.types, this.sqlnet, this.conv, ttcStmt, (byte)8);
      this.close.send();
      this.close.receive();
   }

   public synchronized DBColumn[] describe(DBStatement stmt) throws SQLException, IOException {
      if (stmt == null) {
         DBError.check_error((short)205);
      }

      TTCStatement ttcStmt = (TTCStatement)stmt;
      int colOffset = 0;
      DBColumn[] dbcols = null;
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      do {
         this.describe = new Odscrarr(this.types, this.sqlnet, this.conv, ttcStmt, colOffset);
         this.describe.send();
         dbcols = this.describe.receive(dbcols);
         colOffset += this.describe.numuds;
      } while(colOffset < dbcols.length);

      return dbcols;
   }

   public synchronized DBColumn[] parseExecuteDescribe(DBStatement dbstmt, byte[] sqlStmt, DBType[] inTypes, DBData[] inValues) throws SQLException, IOException {
      if (dbstmt == null || sqlStmt == null) {
         DBError.check_error((short)205);
      }

      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      long options = this.setOptions((byte)2, (byte)0, inTypes, (DBType[])null);
      this.all7 = this.doOall7(options, ((TTCStatement)dbstmt).getCursor(), sqlStmt, inTypes, inValues, 1, (DBType[])null, (DBData[])null, 0);
      DBColumn[] columns = this.describe(dbstmt);
      return columns;
   }

   public synchronized int executeFetch(DBStatement dbstmt, byte type, DBType[] inTypes, DBData[] inValues, int inDepth, DBType[] outTypes, DBData[] outValues, int outDepth) throws SQLException, IOException {
      if (dbstmt == null || type < 0 || type > 2) {
         DBError.check_error((short)205);
      }

      long options = this.setOptions((byte)3, type, inTypes, outTypes);
      this.all7 = this.doOall7(options, ((TTCStatement)dbstmt).getCursor(), new byte[0], inTypes, inValues, inDepth, outTypes, outValues, outDepth);
      if (this.all7.streamingOn) {
         return -2;
      } else {
         return type == 2 ? this.all7.rowsProcessed : this.all7.rxd.unmarshaledRXDs;
      }
   }

   public synchronized int parseExecuteFetch(DBStatement dbstmt, byte type, byte[] sqlStmt, DBType[] inTypes, DBData[] inValues, int inDepth, DBType[] outTypes, DBData[] outValues, int outDepth) throws SQLException, IOException {
      if (dbstmt == null || sqlStmt == null || type < 0 || type > 2) {
         DBError.check_error((short)205);
      }

      long options = this.setOptions((byte)4, type, inTypes, outTypes);
      this.all7 = this.doOall7(options, ((TTCStatement)dbstmt).getCursor(), sqlStmt, inTypes, inValues, inDepth, outTypes, outValues, outDepth);
      if (this.all7.streamingOn) {
         return -2;
      } else {
         return type == 2 ? this.all7.rowsProcessed : this.all7.rxd.unmarshaledRXDs;
      }
   }

   public synchronized int fetch(DBStatement dbstmt, DBType[] outTypes, DBData[] outValues, int outDepth) throws SQLException, IOException {
      if (dbstmt == null) {
         DBError.check_error((short)205);
      }

      long options = this.setOptions((byte)5, (byte)-1, (DBType[])null, outTypes);
      this.all7 = this.doOall7(options, ((TTCStatement)dbstmt).getCursor(), new byte[0], (DBType[])null, (DBData[])null, 0, outTypes, outValues, outDepth);
      return this.all7.streamingOn ? -2 : this.all7.rxd.unmarshaledRXDs;
   }

   public synchronized int read(DBStatement dbstmt, byte[] buffer, int length) throws SQLException, IOException {
      int bytesToRead = -1;
      TTCStatement stmt = (TTCStatement)dbstmt;
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      if (this.all7 == null || this.all7.rxd == null || length > 255 || length < 0) {
         DBError.check_error((short)205);
      }

      if (!this.all7.streamingOn) {
         DBError.check_error((short)206);
      }

      bytesToRead = this.all7.unmarshalUB1();
      if (stmt.isFirstStreamPiece()) {
         if (!this.all7.rxd.escapeSequenceNull(bytesToRead)) {
            if (bytesToRead == 254) {
               bytesToRead = this.all7.unmarshalUB1();
            } else {
               DBError.check_error((short)1);
            }
         } else {
            bytesToRead = -1;
         }

         stmt.firstPieceRead();
      }

      if (bytesToRead > 0) {
         this.all7.unmarshalNBytes(buffer, 0, bytesToRead);
      } else {
         bytesToRead = -1;
         stmt.firstPieceReset();
         this.all7.getColumnsUptoNextSream();
      }

      return bytesToRead;
   }

   public synchronized byte[] getVersion() throws SQLException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      return this.ver.getVersion();
   }

   public synchronized int getDefaultPrefetch() {
      return 10;
   }

   public synchronized int getDefaultStreamChunkSize() {
      return 255;
   }

   public void cancel() throws SQLException, IOException {
      this.sqlnet.Break();
   }

   public DBStatement RefCursorBytesToDBStatement(byte[] refCursor) throws SQLException, IOException {
      return new TTCStatement(this.log1.unmarshalSB4(refCursor));
   }

   public byte[] DBStatementToRefCursorBytes(DBStatement dbstmt) throws SQLException, IOException {
      TTCOutBuffer outBuf = new TTCOutBuffer(5, (SQLnetBufferedOutputStream)null, this.types);
      outBuf.add(((TTCStatement)dbstmt).getCursor());
      return outBuf.getData();
   }

   public int getRefCursorBytesSize() throws SQLException {
      return 5;
   }

   public void setWaitandAutoRollback(DBStatement dbstmt, int autoRollback, int waitOption) throws SQLException {
   }

   private JavaConversion connect(String connection) throws SQLException, IOException {
      if (connection == null) {
         DBError.check_error((short)205);
      }

      this.sqlnet = new SQLnet();
      this.sqlnet.Connect(connection);
      this.types = new TTCTypeRep();
      this.types.setRep((byte)1, (byte)2);
      this.pro = new TTIpro(this.types, this.sqlnet);
      this.pro.send();
      this.pro.receive();
      short cs = JavaConversion.whichCharSetToUse(this.pro.getCharacterSet(), this.pro.getOracleVersion());
      JavaConversion conv = new JavaConversion(cs);
      this.dty = new TTIdty(this.types, this.sqlnet, conv, conv.getCharacterSet(), this.pro.getFlags());
      this.dty.send();
      this.dty.receive();
      return conv;
   }

   private long setOptions(byte call, byte type, DBType[] inTypes, DBType[] outTypes) throws SQLException {
      long options;
      options = 0L;
      label33:
      switch (call) {
         case 2:
            options = 32800L;
         case 1:
            options |= 1L;
            break;
         case 4:
            options = 1L;
         case 3:
            switch (type) {
               case 0:
                  options |= 32864L;
                  break label33;
               case 1:
                  options |= 1056L;
                  if (inTypes != null || outTypes != null) {
                     options |= 8L;
                  }
                  break label33;
               case 2:
                  options |= 32800L;
                  break label33;
               default:
                  DBError.check_error((short)204);
                  break label33;
            }
         case 5:
            options = 32832L;
            break;
         default:
            DBError.check_error((short)204);
      }

      if (type != 1) {
         if (inTypes != null) {
            options |= 8L;
         }

         if (outTypes != null) {
            options |= 16L;
         }
      }

      options &= -1L;
      return options;
   }

   private Oall7 doOall7(long options, int cursor, byte[] sqlStmt, DBType[] inTypes, DBData[] inValues, int inDepth, DBType[] outTypes, DBData[] outValues, int outDepth) throws SQLException, IOException {
      if (this.state < 1) {
         DBError.check_error((short)202);
      }

      this.all7 = new Oall7(this.types, this.sqlnet, this.conv, options, cursor, sqlStmt, inTypes, inValues, inDepth, outTypes, outValues, outDepth);
      this.all7.send();
      this.all7.receive();
      return this.all7;
   }
}
