package oracle.jdbc.ttc7;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;
import oracle.sqlnet.SQLnet;
import oracle.sqlnet.SQLnetBreakException;
import oracle.sqlnet.SQLnetBufferedInputStream;

class TTCInBuffer {
   private static final boolean DEBUG = false;
   private static final boolean DEBUG1 = false;
   private static final boolean DEBUG2 = false;
   private TTCTypeRep types;
   private SQLnetBufferedInputStream netIn;
   private SQLnet sqlnet;

   TTCInBuffer(TTCTypeRep types, SQLnet sqlnet) {
      this.types = types;
      this.sqlnet = sqlnet;
      this.netIn = sqlnet.GetBufferedInputStream();
   }

   public short getByte() throws SQLException, IOException {
      short value = 0;

      while(true) {
         try {
            value = (short)this.netIn.read();
            break;
         } catch (SQLnetBreakException var2) {
            this.sqlnet.reset();
         }
      }

      if (value < 0) {
         DBError.check_error((short)103);
      }

      return value;
   }

   public int get2Bytes() throws SQLException, IOException {
      byte[] tmpBuffer = new byte[2];
      int value = (int)this.buffer2Value(tmpBuffer, (byte)1);
      return value & '\uffff';
   }

   public long get4Bytes() throws SQLException, IOException {
      byte[] tmpBuffer = new byte[4];
      long value = this.buffer2Value(tmpBuffer, (byte)2);
      return value;
   }

   public byte[] getNBytes(int n) throws SQLException, IOException {
      byte[] tmpBuffer = new byte[n];

      while(true) {
         try {
            if (this.netIn.read(tmpBuffer) < 0) {
               DBError.check_error((short)103);
            }

            return tmpBuffer;
         } catch (SQLnetBreakException var3) {
            this.sqlnet.reset();
         }
      }
   }

   public int getNBytes(byte[] buf, int off, int len) throws SQLException, IOException {
      int cnt = 0;

      while(true) {
         try {
            if ((cnt = this.netIn.read(buf, off, len)) < 0) {
               DBError.check_error((short)103);
            }

            return cnt;
         } catch (SQLnetBreakException var5) {
            this.sqlnet.reset();
         }
      }
   }

   public byte[] getBytesBefore00(int bytes) throws SQLException, IOException {
      int offset = 0;
      byte[] tmpBuffer = new byte[bytes];

      while(offset < bytes) {
         while(true) {
            try {
               if (this.netIn.read(tmpBuffer, offset, 1) < 0) {
                  DBError.check_error((short)103);
               }
               break;
            } catch (SQLnetBreakException var5) {
               this.sqlnet.reset();
            }
         }

         if (tmpBuffer[offset++] == 0) {
            break;
         }
      }

      --offset;
      byte[] buffer = new byte[offset];
      System.arraycopy(tmpBuffer, 0, buffer, 0, offset);
      return buffer;
   }

   public byte[] getBytesBefore00() throws SQLException, IOException {
      int offset = 0;
      int incTimes = 1;
      int bufLimit = 50;
      byte[] tmpBuffer = new byte[bufLimit];

      while(true) {
         if (offset >= bufLimit) {
            if (incTimes++ >= 10) {
               break;
            }

            bufLimit *= 2;
            byte[] cpBuffer = tmpBuffer;
            tmpBuffer = new byte[bufLimit];
            System.arraycopy(cpBuffer, 0, tmpBuffer, 0, cpBuffer.length);
         } else {
            while(true) {
               try {
                  if (this.netIn.read(tmpBuffer, offset, 1) < 0) {
                     DBError.check_error((short)103);
                  }
                  break;
               } catch (SQLnetBreakException var7) {
                  this.sqlnet.reset();
               }
            }

            if (tmpBuffer[offset++] == 0) {
               break;
            }
         }
      }

      --offset;
      byte[] buffer = new byte[offset];
      System.arraycopy(tmpBuffer, 0, buffer, 0, offset);
      return buffer;
   }

   private long buffer2Value(byte[] tmpBuffer, byte repOffset) throws SQLException, IOException {
      int bufLength = 0;
      long value = 0L;
      boolean negative = false;
      if ((this.types.rep[repOffset] & 1) > 0) {
         while(true) {
            try {
               bufLength = this.netIn.read();
               break;
            } catch (SQLnetBreakException var10) {
               this.sqlnet.reset();
            }
         }

         if ((bufLength & 128) > 0) {
            bufLength &= 127;
            negative = true;
         }

         if (bufLength < 0) {
            DBError.check_error((short)103);
         }

         if (bufLength == 0) {
            return 0L;
         }

         if (repOffset == 1 && bufLength > 2 || repOffset == 2 && bufLength > 4) {
            DBError.check_error((short)105);
         }
      } else if (repOffset == 1) {
         bufLength = 2;
      } else if (repOffset == 2) {
         bufLength = 4;
      }

      tmpBuffer = new byte[bufLength];

      while(true) {
         try {
            if (this.netIn.read(tmpBuffer) < 0) {
               DBError.check_error((short)103);
            }
            break;
         } catch (SQLnetBreakException var9) {
            this.sqlnet.reset();
         }
      }

      for(int i = 0; i < tmpBuffer.length; ++i) {
         short tmpByte;
         if ((this.types.rep[repOffset] & 2) > 0) {
            tmpByte = (short)(tmpBuffer[tmpBuffer.length - 1 - i] & 255);
         } else {
            tmpByte = (short)(tmpBuffer[i] & 255);
         }

         value |= (long)(tmpByte << 8 * (tmpBuffer.length - 1 - i));
      }

      value &= -1L;
      if (negative) {
         value *= -1L;
      }

      return value;
   }

   public long get4Bytes(byte[] buffer) throws SQLException, IOException {
      byte[] tmpBuffer = new byte[4];
      long value = this.buffer2Value(tmpBuffer, (byte)2, new ByteArrayInputStream(buffer));
      return value;
   }

   private long buffer2Value(byte[] tmpBuffer, byte repOffset, ByteArrayInputStream in) throws SQLException, IOException {
      int bufLength = 0;
      long value = 0L;
      boolean negative = false;
      if ((this.types.rep[repOffset] & 1) > 0) {
         bufLength = in.read();
         if ((bufLength & 128) > 0) {
            bufLength &= 127;
            negative = true;
         }

         if (bufLength < 0) {
            DBError.check_error((short)103);
         }

         if (bufLength == 0) {
            return 0L;
         }

         if (repOffset == 1 && bufLength > 2 || repOffset == 2 && bufLength > 4) {
            DBError.check_error((short)105);
         }
      } else if (repOffset == 1) {
         bufLength = 2;
      } else if (repOffset == 2) {
         bufLength = 4;
      }

      tmpBuffer = new byte[bufLength];
      if (((InputStream)in).read(tmpBuffer) < 0) {
         DBError.check_error((short)103);
      }

      for(int i = 0; i < tmpBuffer.length; ++i) {
         short tmpByte;
         if ((this.types.rep[repOffset] & 2) > 0) {
            tmpByte = (short)(tmpBuffer[tmpBuffer.length - 1 - i] & 255);
         } else {
            tmpByte = (short)(tmpBuffer[i] & 255);
         }

         value |= (long)(tmpByte << 8 * (tmpBuffer.length - 1 - i));
      }

      value &= -1L;
      if (negative) {
         value *= -1L;
      }

      return value;
   }
}
