package oracle.jdbc.ttc7;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import oracle.sqlnet.SQLnetBufferedOutputStream;

class TTCOutBuffer {
   private static final boolean DEBUG = false;
   private boolean dataSent = false;
   private TTCTypeRep types;
   private byte[] buffer;
   private int bufLength;
   ByteArrayOutputStream outBuffer;
   private SQLnetBufferedOutputStream netOut;

   TTCOutBuffer(int len, SQLnetBufferedOutputStream netOut, TTCTypeRep types) {
      this.outBuffer = new ByteArrayOutputStream(len);
      this.dataSent = false;
      this.netOut = netOut;
      this.types = types;
   }

   TTCOutBuffer(SQLnetBufferedOutputStream netOut, TTCTypeRep types) {
      this.dataSent = true;
      this.netOut = netOut;
      this.types = types;
   }

   public void add(byte value) throws IOException {
      if (this.outBuffer != null) {
         this.outBuffer.write(value);
      } else {
         this.netOut.write(value);
      }

   }

   public void add(short value) throws IOException {
      byte[] tmpBuffer = new byte[2];
      byte bytes = this.value2Buffer(value, tmpBuffer, (byte)1);
      this.add(tmpBuffer, 0, bytes);
   }

   public void add(int value) throws IOException {
      byte[] tmpBuffer = new byte[4];
      byte bytes = this.value2Buffer(value, tmpBuffer, (byte)2);
      this.add(tmpBuffer, 0, bytes);
   }

   public void addPtr(byte value) throws IOException {
      if ((this.types.rep[3] & 1) > 0) {
         this.add(value);
      } else {
         byte[] tmpBuffer = new byte[4];
         byte bytes = this.value2Buffer(value, tmpBuffer, (byte)3);
         this.add(tmpBuffer, 0, bytes);
      }

   }

   public void add(byte[] value) throws IOException {
      if (this.outBuffer != null) {
         this.outBuffer.write(value);
      } else {
         this.netOut.write(value);
      }

   }

   public void add(byte[] value, int offset, int bytes) throws IOException {
      if (this.outBuffer != null) {
         this.outBuffer.write(value, offset, bytes);
      } else {
         this.netOut.write(value, offset, bytes);
      }

   }

   public void add(long[] value) throws IOException {
      for(int i = 0; i < value.length; ++i) {
         this.add((int)(value[i] & -1L));
      }

   }

   public byte[] getData() throws IOException {
      return this.outBuffer.toByteArray();
   }

   public void pushData() throws IOException {
      if (!this.dataSent && this.outBuffer != null) {
         this.netOut.write(this.outBuffer.toByteArray(), 0, this.outBuffer.size());
         this.dataSent = true;
      }

   }

   public void print() {
      System.out.println(" ---- Marshaled buffer    ---- ");
      System.out.println("buffer bytes used = " + this.bufLength);
      if (this.bufLength > 0) {
         System.out.print("buffer: ");
         System.out.println();
      }

      System.out.println(" ---- --------------------- ---- ");
   }

   public void printASCII() {
      System.out.println(" ---- Marshaled buffer in ASCII ---- ");
      System.out.println("buffer bytes used = " + this.bufLength);
      if (this.bufLength > 0) {
         System.out.print("buffer: ");
         System.out.println();
      }

      System.out.println(" ---- -------------------------- ---- ");
   }

   private byte value2Buffer(int value, byte[] tmpBuffer, byte repOffset) throws IOException {
      boolean zeros = true;
      byte bytes = 0;

      for(int i = tmpBuffer.length - 1; i >= 0; --i) {
         tmpBuffer[bytes] = (byte)(value >> 8 * i & 255);
         if ((this.types.rep[repOffset] & 1) > 0) {
            if (!zeros || tmpBuffer[bytes] != 0) {
               zeros = false;
               ++bytes;
            }
         } else {
            ++bytes;
         }
      }

      if ((this.types.rep[repOffset] & 1) > 0) {
         this.add(bytes);
      }

      if ((this.types.rep[repOffset] & 2) > 0) {
         this.reverseArray(tmpBuffer, bytes);
      }

      return bytes;
   }

   private void reverseArray(byte[] tmpBuffer, byte bytes) {
      for(int i = 0; i < bytes / 2; ++i) {
         byte tmp = tmpBuffer[i];
         tmpBuffer[i] = tmpBuffer[bytes - 1 - i];
         tmpBuffer[bytes - 1 - i] = tmp;
      }

   }
}
