package oracle.jdbc.ttc7;

import java.io.IOException;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBType;
import oracle.jdbc.oracore.JavaConversion;
import oracle.sqlnet.SQLnet;
import oracle.sqlnet.SQLnetBufferedInputStream;
import oracle.sqlnet.SQLnetBufferedOutputStream;

public class TTIMsg {
   private static final boolean DEBUG = false;
   private static final boolean DEBUG1 = false;
   public static final byte TTIPRO = 1;
   public static final byte TTIDTY = 2;
   public static final byte TTIFUN = 3;
   public static final byte TTIOER = 4;
   public static final byte TTIRXH = 6;
   public static final byte TTIRXD = 7;
   public static final byte TTIRPA = 8;
   public static final byte TTISTA = 9;
   public static final byte TTIIOV = 11;
   protected byte ttcCode;
   protected TTCOutBuffer metaData;
   protected TTCOutBuffer ptrData;
   protected TTCInBuffer inData;
   protected TTCTypeRep types;
   protected SQLnet sqlnet;
   protected SQLnetBufferedInputStream netIn;
   protected SQLnetBufferedOutputStream netOut;
   protected JavaConversion conv;

   public TTIMsg(TTCTypeRep types, SQLnet sqlnet, JavaConversion conv) {
      this.sqlnet = sqlnet;
      this.types = types;
      this.conv = conv;
      this.netIn = sqlnet.GetBufferedInputStream();
      this.netOut = sqlnet.GetBufferedOutputStream();
   }

   protected void initBuffers(int mdLen, int pdLen) {
      this.initOutBuffers(mdLen, pdLen);
      this.initInBuffers();
   }

   public void initOutBuffers(int mdLen, int pdLen) {
      this.metaData = new TTCOutBuffer(this.netOut, this.types);
      this.ptrData = new TTCOutBuffer(pdLen, this.netOut, this.types);
   }

   public void initInBuffers() {
      this.inData = new TTCInBuffer(this.types, this.sqlnet);
   }

   protected void marshalSB1(byte value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

   }

   protected void marshalUB1(short value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add((byte)(value & 255));
      } else {
         this.metaData.add((byte)(value & 255));
      }

   }

   protected void marshalSB2(short value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

   }

   protected void marshalUB2(int value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add((short)(value & '\uffff'));
      } else {
         this.metaData.add((short)(value & '\uffff'));
      }

   }

   protected void marshalSB4(int value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

   }

   protected void marshalUB4(long value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add((int)(value & -1L));
      } else {
         this.metaData.add((int)(value & -1L));
      }

   }

   protected void marshalSWORD(int value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

   }

   protected void marshalUWORD(long value, boolean ptr) throws IOException {
      if (ptr) {
         this.metaData.addPtr((byte)1);
         this.ptrData.add((int)(value & -1L));
      } else {
         this.metaData.add((int)(value & -1L));
      }

   }

   protected void marshalB1Array(byte[] value, boolean ptr) throws IOException {
      if (ptr) {
         if (value.length == 0) {
            this.metaData.addPtr((byte)0);
         } else {
            this.metaData.addPtr((byte)1);
            this.ptrData.add(value);
         }
      } else if (value.length > 0) {
         this.metaData.add(value);
      }

   }

   protected void marshalB1Array(byte[] value, int off, int len, boolean ptr) throws IOException {
      if (ptr) {
         if (value.length == 0) {
            this.metaData.addPtr((byte)0);
         } else {
            this.metaData.addPtr((byte)1);
            this.ptrData.add(value, off, len);
         }
      } else if (value.length > 0) {
         this.metaData.add(value, off, len);
      }

   }

   protected void marshalUB4Array(long[] value, boolean ptr) throws IOException {
      if (ptr) {
         if (value.length == 0) {
            this.metaData.addPtr((byte)0);
         } else {
            this.metaData.addPtr((byte)1);
            this.ptrData.add(value);
         }
      } else if (value.length > 0) {
         this.metaData.add(value);
      }

   }

   public void marshalO2U(boolean notnull) throws IOException {
      if (notnull) {
         this.metaData.addPtr((byte)1);
      } else {
         this.metaData.addPtr((byte)0);
      }

   }

   public void marshalNULLPTR() throws IOException {
      this.metaData.addPtr((byte)0);
   }

   public void marshalPTR() throws IOException {
      this.metaData.addPtr((byte)1);
   }

   protected void marshalByte(byte value, boolean ptr) throws SQLException, IOException {
      byte tmpRep = this.types.getRep((byte)0);
      this.types.setRep((byte)0, (byte)0);
      if (ptr) {
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

      this.types.setRep((byte)0, tmpRep);
   }

   protected void marshalShort(short value, boolean ptr) throws SQLException, IOException {
      byte tmpRep = this.types.getRep((byte)1);
      this.types.setRep((byte)1, (byte)0);
      if (ptr) {
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

      this.types.setRep((byte)1, tmpRep);
   }

   protected void marshalInt(int value, boolean ptr) throws SQLException, IOException {
      byte tmpRep = this.types.getRep((byte)2);
      this.types.setRep((byte)2, (byte)0);
      if (ptr) {
         this.ptrData.add(value);
      } else {
         this.metaData.add(value);
      }

      this.types.setRep((byte)2, tmpRep);
   }

   protected void marshalOAC(DBType value, TTCOutBuffer buffer) throws IOException {
      byte flags = 1;
      byte precision = 0;
      byte scale = 0;
      byte contflags = 0;
      byte total_elems = 0;
      if (value.type == 96) {
         flags = (byte)(flags | 32);
      }

      buffer.add((byte)(value.type & 255));
      buffer.add(flags);
      buffer.add(precision);
      buffer.add(scale);
      if (value.type == 102) {
         buffer.add(1);
      } else {
         buffer.add(value.max_length);
      }

      buffer.add(total_elems);
      buffer.add(contflags);
   }

   protected void marshalOACArray(DBType[] value, boolean ptr) throws IOException {
      if (ptr) {
         if (value.length == 0) {
            this.metaData.addPtr((byte)0);
         } else {
            this.metaData.addPtr((byte)1);

            for(int i = 0; i < value.length; ++i) {
               this.marshalOAC(value[i], this.ptrData);
            }
         }
      } else if (value.length > 0) {
         for(int i = 0; i < value.length; ++i) {
            this.marshalOAC(value[i], this.metaData);
         }
      }

   }

   public byte unmarshalSB1() throws SQLException, IOException {
      short value = this.inData.getByte();
      return (byte)value;
   }

   public short unmarshalUB1() throws SQLException, IOException {
      short value = this.inData.getByte();
      return value;
   }

   public short unmarshalSB2() throws SQLException, IOException {
      int value = this.inData.get2Bytes();
      return (short)value;
   }

   public int unmarshalUB2() throws SQLException, IOException {
      int value = this.inData.get2Bytes();
      return value;
   }

   public int unmarshalSB4() throws SQLException, IOException {
      long value = this.inData.get4Bytes();
      return (int)value;
   }

   public int unmarshalSB4(byte[] buffer) throws SQLException, IOException {
      long value = this.inData.get4Bytes(buffer);
      return (int)value;
   }

   public long unmarshalUB4() throws SQLException, IOException {
      long value = 0L;
      value = this.inData.get4Bytes();
      return value;
   }

   public int unmarshalSWORD() throws SQLException, IOException {
      return this.unmarshalSB4();
   }

   public long unmarshalUWORD() throws SQLException, IOException {
      return this.unmarshalUB4();
   }

   public byte[] unmarshalNBytes(int n) throws SQLException, IOException {
      return this.inData.getNBytes(n);
   }

   public int unmarshalNBytes(byte[] buf, int off, int n) throws SQLException, IOException {
      int bytes;
      for(bytes = 0; bytes < n; bytes += this.inData.getNBytes(buf, off + bytes, n - bytes)) {
      }

      return bytes;
   }

   public byte[] unmarshalTEXT() throws SQLException, IOException {
      return this.inData.getBytesBefore00();
   }

   public byte[] unmarshalTEXT(int bytes) throws SQLException, IOException {
      return this.inData.getBytesBefore00(bytes);
   }

   public void send() throws IOException {
      this.metaData.pushData();
      this.ptrData.pushData();
   }

   protected void reset() throws SQLException, IOException {
      this.sqlnet.Break();
      this.unmarshalUB1();
      TTIoer oer1 = new TTIoer(this.types, this.sqlnet, this.conv);
      oer1.unmarshal();
   }

   public void print() throws SQLException {
      this.metaData.print();
      this.ptrData.print();
   }

   public void printASCII() {
      this.metaData.printASCII();
      this.ptrData.printASCII();
   }
}
