package oracle.jdbc.ttc7;

import java.io.IOException;
import java.sql.SQLException;
import oracle.jdbc.oracore.JavaConversion;
import oracle.sqlnet.SQLnet;

public class TTIoer extends TTIMsg {
   private final boolean DEBUG = false;
   private long curRowNumber;
   private int retCode;
   private int arrayElemWError;
   private int arrayElemErrno;
   private int currCursorID;
   private short errorPosition;
   private short sqlType;
   private byte oerFatal;
   private byte flags;
   private byte userCursorOpt;
   private short upiParam;
   private short warningFlag;
   private TTIrid rid;
   private int osError;
   private short stmtNumber;
   private short callNumber;
   private int pad1;
   private long successIters;
   private int errorLength;
   private byte[] errorMsg;
   public static final int ORA1403 = 1403;

   public TTIoer(TTCTypeRep types, SQLnet sqlnet, JavaConversion conv) {
      super(types, sqlnet, conv);
      this.rid = new TTIrid(types, sqlnet, conv);
      ((TTIMsg)this).initInBuffers();
   }

   public void unmarshal() throws IOException, SQLException {
      this.curRowNumber = ((TTIMsg)this).unmarshalUB4();
      this.retCode = ((TTIMsg)this).unmarshalUB2();
      this.arrayElemWError = ((TTIMsg)this).unmarshalUB2();
      this.arrayElemErrno = ((TTIMsg)this).unmarshalUB2();
      this.currCursorID = ((TTIMsg)this).unmarshalUB2();
      this.errorPosition = ((TTIMsg)this).unmarshalSB2();
      this.sqlType = ((TTIMsg)this).unmarshalUB1();
      this.oerFatal = ((TTIMsg)this).unmarshalSB1();
      this.flags = ((TTIMsg)this).unmarshalSB1();
      this.userCursorOpt = ((TTIMsg)this).unmarshalSB1();
      this.upiParam = ((TTIMsg)this).unmarshalUB1();
      this.warningFlag = ((TTIMsg)this).unmarshalUB1();
      this.rid.unmarshal();
      this.osError = ((TTIMsg)this).unmarshalSWORD();
      this.stmtNumber = ((TTIMsg)this).unmarshalUB1();
      this.callNumber = ((TTIMsg)this).unmarshalUB1();
      this.pad1 = ((TTIMsg)this).unmarshalUB2();
      this.successIters = ((TTIMsg)this).unmarshalUB4();
      if (this.retCode != 0) {
         this.errorLength = ((TTIMsg)this).unmarshalUB1();
         this.errorMsg = ((TTIMsg)this).unmarshalNBytes(this.errorLength);
      }

   }

   public void print() throws SQLException {
      System.out.println("  Current Row Number    :" + this.curRowNumber);
      System.out.println("  Returned Code         :" + this.retCode);
      System.out.println("  Array Element w/error :" + this.arrayElemWError);
      System.out.println("  Array Element errno   :" + this.arrayElemErrno);
      System.out.println("  Current Cursor ID     :" + this.currCursorID);
      System.out.println("  Error Position        :" + this.errorPosition);
      System.out.println("  SQL command type      :" + this.sqlType);
      System.out.println("  Fatal                 :" + this.oerFatal);
      System.out.println("  Various flags         :" + this.flags);
      System.out.println("  User cursor options   :" + this.userCursorOpt);
      System.out.println("  UPI parameter that \n  generated the error   :" + this.upiParam);
      System.out.println("  Warning flags         :" + this.warningFlag);
      System.out.println("      Row ID structure  :");
      this.rid.print();
      System.out.println("  Operating System Error:" + this.osError);
      System.out.println("  Statemente number     :" + this.stmtNumber);
      System.out.println("  Procedure call number :" + this.callNumber);
      System.out.println("  Pad1                  :" + this.pad1);
      System.out.println("  Successful iterations :" + this.successIters);

      try {
         if (this.retCode != 0) {
            System.out.println("Error Message: " + super.conv.CharBytesToString(this.errorMsg, this.errorLength));
         }
      } catch (SQLException var1) {
      }

   }

   public void processError() throws SQLException {
      if (this.retCode != 0) {
         throw new SQLException(super.conv.CharBytesToString(this.errorMsg, this.errorLength), "", this.retCode);
      }
   }

   public int getCurRowNumber() throws SQLException {
      return (int)this.curRowNumber;
   }

   public int getRetCode() {
      return this.retCode;
   }
}
