package oracle.jdbc.ttc7;

import java.io.IOException;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.oracore.JavaConversion;
import oracle.sqlnet.SQLnet;

public class TTIpro extends TTIMsg {
   private final boolean DEBUG = false;
   private short svrCharSet;
   private short svrCharSetElem;
   private byte svrFlags;
   private byte[] proSvrStr;
   private byte proSvrVer;
   private byte oVersion = -1;
   private boolean svrInfoAvailable = false;
   private byte[] proCliVer = new byte[]{4, 3, 2, 1, 0};
   private byte[] proCliStr = new byte[]{74, 97, 118, 97, 95, 84, 84, 67, 45, 55, 46, 50, 46, 48, 0};
   private final int metaDataSize = 21;

   public TTIpro(TTCTypeRep types, SQLnet sqlnet) throws SQLException, IOException {
      super(types, sqlnet, (JavaConversion)null);
      ((TTIMsg)this).initBuffers(21, 0);
      super.ttcCode = 1;
      this.marshal();
   }

   public void receive() throws SQLException, IOException {
      if (((TTIMsg)this).unmarshalSB1() != 1) {
         DBError.check_error((short)1);
      }

      this.proSvrVer = ((TTIMsg)this).unmarshalSB1();
      ((TTIMsg)this).unmarshalSB1();
      this.proSvrStr = ((TTIMsg)this).unmarshalTEXT(50);
      this.oVersion = this.getOracleVersion();
      this.svrCharSet = (short)((TTIMsg)this).unmarshalUB2();
      this.svrFlags = (byte)((TTIMsg)this).unmarshalUB1();
      if ((this.svrCharSetElem = (short)((TTIMsg)this).unmarshalUB2()) > 0) {
         ((TTIMsg)this).unmarshalNBytes(this.svrCharSetElem * 5);
      }

      this.svrInfoAvailable = true;
   }

   public byte getOracleVersion() {
      if (this.oVersion == -1 && this.proSvrStr != null) {
         byte pro = this.proSvrStr[this.proSvrStr.length - 5];
         if (pro >= 48 && pro <= 57) {
            this.oVersion = (byte)(pro - 48);
         }
      }

      return this.oVersion;
   }

   public short getCharacterSet() {
      return this.svrCharSet;
   }

   public byte getFlags() {
      return this.svrFlags;
   }

   public void printServerInfo() {
      System.out.println("\n ---- Server's Information ---- ");
      if (this.svrInfoAvailable) {
         int i = 0;
         System.out.println("Protocol version :" + this.proSvrVer);
         System.out.print("Protocol string  :");

         while(i < this.proSvrStr.length) {
            System.out.print((char)this.proSvrStr[i++]);
         }

         System.out.println();
         System.out.println("Caracter Set ID  :" + this.svrCharSet);
         System.out.println("Remote flags     :" + this.svrFlags);
         System.out.println("Number of Elements in Server's Character Set Graph :" + this.svrCharSetElem);
         System.out.println("Don't expect the graph, we threw it away :-)");
      } else {
         System.out.println(" Not Available !!");
      }

      System.out.println(" ---- -------------------- ---- ");
   }

   private void marshal() throws SQLException, IOException {
      ((TTIMsg)this).marshalUB1((short)super.ttcCode, false);
      ((TTIMsg)this).marshalB1Array(this.proCliVer, false);
      ((TTIMsg)this).marshalB1Array(this.proCliStr, false);
   }
}
