package oracle.jdbc.ttc7;

import java.io.IOException;
import java.sql.SQLException;
import oracle.jdbc.dbaccess.DBData;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.dbaccess.DBItem;
import oracle.jdbc.dbaccess.DBType;
import oracle.jdbc.oracore.JavaConversion;
import oracle.sqlnet.SQLnet;

public class TTIrxd extends TTIMsg {
   private static final boolean DEBUG = false;
   private static final boolean FIX = false;
   public int unmarshaledRXDs;
   public int marshaledRXDs;
   int unmarshaledCols;
   int indicator;
   boolean streamingOn = false;
   TTIfun msg;
   private DBType[] inTypes;
   private DBData[] inValues;
   private DBType[] outTypes;
   private DBData[] outValues;
   private int inDepth;
   private int outDepth;
   public static final int TTCC_MXL = 252;
   public static final int TTCC_ESC = 253;
   public static final int TTCC_LNG = 254;
   public static final int TTCC_ERR = 255;
   public static final int TTCC_MXIN = 64;

   public TTIrxd(TTCTypeRep types, SQLnet sqlnet, JavaConversion conv, DBType[] inTypes, DBData[] inValues, int inDepth, DBType[] outTypes, DBData[] outValues, int outDepth) {
      super(types, sqlnet, conv);
      this.inTypes = inTypes != null ? inTypes : new DBType[0];
      this.outTypes = outTypes != null ? outTypes : new DBType[0];
      this.inValues = inValues != null ? inValues : new DBData[0];
      this.outValues = outValues != null ? outValues : new DBData[0];
      this.inDepth = this.inValues.length == 0 ? 0 : inDepth;
      this.outDepth = this.outValues.length == 0 ? 0 : outDepth;
      ((TTIMsg)this).initBuffers(0, 0);
   }

   public void marshal() throws IOException {
      if (this.inDepth > 0 && this.inValues.length > 0) {
         int[] streamColumnsPos = new int[this.inValues.length];
         int streamColumns = 0;

         for(int i = 0; i < this.inDepth; ++i) {
            ((TTIMsg)this).marshalSB1((byte)7, false);

            for(int j = 0; j < this.inValues.length; ++j) {
               if (this.inTypes[j] != null) {
                  if (this.inTypes[j].is_stream) {
                     if (!this.inValues[j].items[i].is_null && this.inValues[j].items[i].stream_value != null) {
                        streamColumnsPos[streamColumns++] = j;
                     } else {
                        ((TTIMsg)this).marshalUB1((short)253, false);
                        ((TTIMsg)this).marshalUB1((short)1, false);
                     }
                  } else {
                     this.putColumnValue(this.inValues[j].items[i], this.inTypes[j]);
                  }
               }
            }

            if (streamColumns > 0) {
               for(int k = 0; k < streamColumns; ++k) {
                  this.putStream(this.inValues[streamColumnsPos[k]].items[i]);
               }
            }
         }

      }
   }

   public void putColumnValue(DBItem item, DBType itemType) throws IOException {
      if (itemType.type == 102) {
         ((TTIMsg)this).marshalUB1((short)1, false);
         ((TTIMsg)this).marshalUB1((short)0, false);
      } else {
         if (item.data_size > 64) {
            int offset = 0;
            ((TTIMsg)this).marshalUB1((short)254, false);

            do {
               int bytesLeft = item.data_size - offset;
               int len = bytesLeft > 64 ? 64 : bytesLeft;
               ((TTIMsg)this).marshalUB1((short)(len & 255), false);
               ((TTIMsg)this).marshalB1Array(item.byte_value, offset, len, false);
               offset += len;
            } while(offset < item.data_size);

            ((TTIMsg)this).marshalSB1((byte)0, false);
         } else {
            ((TTIMsg)this).marshalUB1((short)(item.data_size & 255), false);
            ((TTIMsg)this).marshalB1Array(item.byte_value, false);
         }

      }
   }

   public void putStream(DBItem item) throws IOException {
      byte[] buffer = new byte[64];
      int bytes = 0;
      int bytesRead = 0;
      int bytesToRead = 0;
      int totalBytesRead = 0;
      ((TTIMsg)this).marshalUB1((short)254, false);

      do {
         bytesRead = 0;

         for(bytes = 0; bytesRead < 64 && totalBytesRead < item.data_size; totalBytesRead += bytes) {
            if (64 - bytesRead > item.data_size - totalBytesRead) {
               bytesToRead = item.data_size - totalBytesRead;
            } else {
               bytesToRead = 64 - bytesRead;
            }

            if ((bytes = item.stream_value.read(buffer, bytesRead, bytesToRead)) < 0) {
               break;
            }

            bytesRead += bytes;
         }

         ((TTIMsg)this).marshalUB1((short)(bytesRead & 255), false);
         ((TTIMsg)this).marshalB1Array(buffer, 0, bytesRead, false);
      } while(bytes > -1 && totalBytesRead < item.data_size);

      ((TTIMsg)this).marshalSB1((byte)0, false);
   }

   public boolean unmarshal() throws SQLException, IOException {
      if (this.outDepth > 0 && this.outValues.length > 0) {
         if (this.streamingOn) {
            this.streamingOn = false;
            this.processIndicator(this.outValues[this.unmarshaledCols++].items[this.unmarshaledRXDs]);
         }

         for(int i = this.unmarshaledCols; i < this.outValues.length; ++i) {
            if (this.outTypes[i] == null) {
               ++this.unmarshaledCols;
            } else {
               this.getColumnValue(this.outValues[i].items[this.unmarshaledRXDs], this.outTypes[i]);
               if (this.outTypes[i].is_stream && this.streamingOn) {
                  return false;
               }

               ++this.unmarshaledCols;
            }
         }

         ++this.unmarshaledRXDs;
         this.unmarshaledCols = 0;
         return true;
      } else {
         return true;
      }
   }

   public void getColumnValue(DBItem item, DBType type) throws SQLException, IOException {
      short ind = 0;
      short bytes = 0;
      short len = 0;
      if (type.is_stream) {
         this.streamingOn = true;
      } else {
         if (item.byte_value == null) {
            DBError.check_error((short)1);
         }

         if (item.byte_value.length != 0) {
            bytes = ((TTIMsg)this).unmarshalUB1();
            if (bytes < 0) {
               DBError.check_error((short)1);
            }
         }

         item.original_size = 0;
         item.data_size = 0;
         item.is_null = false;
         if (!(item.is_null = this.escapeSequenceNull(bytes))) {
            if (bytes == 254) {
               while((len = ((TTIMsg)this).unmarshalUB1()) > 0) {
                  ((TTIMsg)this).unmarshalNBytes(item.byte_value, item.data_size, len);
                  item.data_size += len;
               }
            } else {
               ((TTIMsg)this).unmarshalNBytes(item.byte_value, item.data_size, bytes);
               item.data_size = bytes;
            }
         }

         this.processIndicator(item);
      }
   }

   public boolean escapeSequenceNull(int bytes) throws SQLException {
      boolean is_null = false;
      switch (bytes) {
         case 0:
            is_null = true;
            break;
         case 253:
            DBError.check_error((short)1);
         case 255:
            DBError.check_error((short)1);
         case 254:
      }

      return is_null;
   }

   public void processIndicator(DBItem item) throws SQLException, IOException {
      short ind = 0;
      ind = ((TTIMsg)this).unmarshalSB2();
      if (!item.is_null) {
         if (ind == 0) {
            item.original_size = item.data_size;
         } else if (ind != -2 && ind <= 0) {
            item.original_size = 65536 + ind;
         } else {
            item.original_size = ind;
         }
      }

   }
}
