package oracle.sqlnet;

import java.util.Vector;

public final class NVPair {
   public static int RHS_NONE;
   public static int RHS_ATOM = 1;
   public static int RHS_LIST = 2;
   public static int LIST_REGULAR = 3;
   public static int LIST_COMMASEP = 4;
   private String _name;
   private int _rhsType;
   private String _atom;
   private Vector _list;
   private int _listType;
   private NVPair _parent;

   public NVPair(String name) {
      this._name = name;
      this._atom = null;
      this._list = null;
      this._listType = LIST_REGULAR;
      this._parent = null;
      this._rhsType = RHS_NONE;
   }

   public NVPair(String name, String atom) {
      this(name);
      this.setAtom(atom);
   }

   public NVPair(String name, NVPair child) {
      this(name);
      this.addListElement(child);
   }

   public String getName() {
      return this._name;
   }

   public void setName(String name) {
      this._name = name;
   }

   public NVPair getParent() {
      return this._parent;
   }

   private void _setParent(NVPair parent) {
      this._parent = parent;
   }

   public int getRHSType() {
      return this._rhsType;
   }

   public int getListType() {
      return this._listType;
   }

   public void setListType(int type) {
      this._listType = type;
   }

   public String getAtom() {
      return this._atom;
   }

   public void setAtom(String atom) {
      this._rhsType = RHS_ATOM;
      this._atom = atom;
      this._list = null;
   }

   public int getListSize() {
      return this._list == null ? 0 : this._list.size();
   }

   public NVPair getListElement(int pos) {
      return this._list == null ? null : (NVPair)this._list.elementAt(pos);
   }

   public void addListElement(NVPair pair) {
      if (this._list == null) {
         this._rhsType = RHS_LIST;
         this._list = new Vector(3, 5);
         this._atom = null;
      }

      this._list.addElement(pair);
      pair._setParent(this);
   }

   public void removeListElement(int pos) {
      if (this._list != null) {
         this._list.removeElementAt(pos);
         if (this.getListSize() == 0) {
            this._list = null;
            this._rhsType = RHS_NONE;
         }
      }

   }

   public String toString() {
      String out = "(" + this._name + "=";
      if (this._rhsType == RHS_ATOM) {
         out = out + this._atom;
      } else if (this._rhsType == RHS_LIST) {
         if (this._listType == LIST_REGULAR) {
            for(int i = 0; i < this.getListSize(); ++i) {
               out = out + this.getListElement(i).toString();
            }
         } else if (this._listType == LIST_COMMASEP) {
            out = out + " (";

            for(int i = 0; i < this.getListSize(); ++i) {
               NVPair listElem = this.getListElement(i);
               out = out + listElem.getName();
               if (i != this.getListSize() - 1) {
                  out = out + ", ";
               }
            }

            out = out + ")";
         }
      }

      out = out + ")";
      return out;
   }

   public void println() {
      System.out.println(this.toString());
   }
}
