package oracle.sqlnet;

import java.util.Vector;

public final class NVTokens {
   public static final int TKN_ERROR = -1;
   public static final int TKN_NONE = 0;
   public static final int TKN_LPAREN = 1;
   public static final int TKN_RPAREN = 2;
   public static final int TKN_COMMA = 3;
   public static final int TKN_EQUAL = 4;
   public static final int TKN_LITERAL = 8;
   public static final int TKN_EOS = 9;
   private static final char TKN_LPAREN_VALUE = '(';
   private static final char TKN_RPAREN_VALUE = ')';
   private static final char TKN_COMMA_VALUE = ',';
   private static final char TKN_EQUAL_VALUE = '=';
   private static final char TKN_BKSLASH_VALUE = '\\';
   private static final char TKN_DQUOTE_VALUE = '"';
   private static final char TKN_SQUOTE_VALUE = '\'';
   private static final char TKN_EOS_VALUE = '%';
   private static final char TKN_SPC_VALUE = ' ';
   private static final char TKN_TAB_VALUE = '\t';
   private static final char TKN_LF_VALUE = '\n';
   private static final char TKN_CR_VALUE = '\r';
   private Vector _tkType = null;
   private Vector _tkValue = null;
   private int _numTokens = 0;
   private int _tkPos = 0;

   private static boolean _isWhiteSpace(char it) {
      return it == ' ' || it == '\t' || it == '\n' || it == '\r';
   }

   private static String _trimWhiteSpace(String it) {
      int length = it.length();
      int start = 0;

      int end;
      for(end = length; start < length && _isWhiteSpace(it.charAt(start)); ++start) {
      }

      while(start < end && _isWhiteSpace(it.charAt(end - 1))) {
         --end;
      }

      return it.substring(start, end);
   }

   public boolean parseTokens(String nvString) {
      this._numTokens = 0;
      this._tkPos = 0;
      this._tkType = new Vector(25, 25);
      this._tkValue = new Vector(25, 25);
      int len = nvString.length();
      char[] input = nvString.toCharArray();
      int pos = 0;

      while(pos < len) {
         while(pos < len && _isWhiteSpace(input[pos])) {
            ++pos;
         }

         if (pos < len) {
            switch (input[pos]) {
               case '(':
                  this._addToken(1, '(');
                  ++pos;
                  continue;
               case ')':
                  this._addToken(2, ')');
                  ++pos;
                  continue;
               case ',':
                  this._addToken(3, ',');
                  ++pos;
                  continue;
               case '=':
                  this._addToken(4, '=');
                  ++pos;
                  continue;
            }

            int startPos = pos;
            int endPos = -1;
            boolean quoted_str = false;
            char quote_char = '"';
            if (input[pos] == '\'' || input[pos] == '"') {
               quoted_str = true;
               quote_char = input[pos];
               ++pos;
            }

            while(pos < len) {
               if (input[pos] == '\\') {
                  pos += 2;
               } else {
                  if (quoted_str) {
                     if (input[pos] == quote_char) {
                        ++pos;
                        endPos = pos;
                        break;
                     }
                  } else if (input[pos] == '(' || input[pos] == ')' || input[pos] == ',' || input[pos] == '=') {
                     endPos = pos;
                     break;
                  }

                  ++pos;
               }
            }

            if (endPos == -1) {
               endPos = pos;
            }

            this._addToken(8, _trimWhiteSpace(nvString.substring(startPos, endPos)));
         }
      }

      this._addToken(9, '%');
      return true;
   }

   public int getToken() {
      if (this._tkType == null) {
         return -1;
      } else {
         return this._tkPos < this._numTokens ? (Integer)this._tkType.elementAt(this._tkPos) : 0;
      }
   }

   public int popToken() {
      int token = 0;
      if (this._tkType == null) {
         token = -1;
      } else if (this._tkPos < this._numTokens) {
         token = (Integer)this._tkType.elementAt(this._tkPos++);
      }

      return token;
   }

   public String getLiteral() {
      String theLiteral = null;
      if (this._tkValue != null && this._tkPos < this._numTokens) {
         theLiteral = (String)this._tkValue.elementAt(this._tkPos);
      }

      return theLiteral;
   }

   public String popLiteral() {
      String theLiteral = null;
      if (this._tkValue != null && this._tkPos < this._numTokens) {
         theLiteral = (String)this._tkValue.elementAt(this._tkPos++);
      }

      return theLiteral;
   }

   public void eatToken() {
      if (this._tkPos < this._numTokens) {
         ++this._tkPos;
      }

   }

   public String toString() {
      if (this._tkType == null) {
         return "*NO TOKENS*";
      } else {
         String out = "Tokens";

         for(int i = 0; i < this._numTokens; ++i) {
            out = out + " : " + this._tkValue.elementAt(i);
         }

         return out;
      }
   }

   public void println() {
      System.out.println(this.toString());
   }

   private void _addToken(int tk, char tk_char) {
      this._addToken(tk, String.valueOf(tk_char));
   }

   private void _addToken(int tk, String tk_val) {
      this._tkType.addElement(new Integer(tk));
      this._tkValue.addElement(tk_val);
      ++this._numTokens;
   }
}
