package oracle.sqlnet;

import java.io.IOException;

public class SQLnetBufferedOutputStream implements SQLnetDef {
   protected SQLnetOutputStream out;
   protected byte[] buf;
   protected int count;

   public SQLnetBufferedOutputStream(SQLnetOutputStream out, int size) {
      this.out = out;
      this.buf = new byte[size];
      this.count = 10;
   }

   public synchronized void write(int b) throws IOException {
      byte[] temp = new byte[1];
      temp[0] = (byte)b;
      this.write(temp);
   }

   public synchronized void write(byte[] b) throws IOException {
      this.write(b, 0, b.length);
   }

   public synchronized void write(byte[] b, int off, int len) throws IOException {
      int avail = this.buf.length - this.count;
      int copy = Math.min(avail, len);
      if (copy > 0) {
         System.arraycopy(b, off, this.buf, this.count, copy);
         this.count += copy;
      }

      if (len > copy) {
         this.out.write(this.buf, this.count, true);
         this.count = 10;
         this.write(b, off + copy, len - copy);
      }

   }

   public synchronized void flush() throws IOException {
      this.out.write(this.buf, this.count, false);
      this.count = 10;
   }

   public synchronized void close() throws IOException {
      this.out.close(this.buf, this.count);
   }
}
