package oracle.sqlnet;

import java.io.IOException;
import java.io.InputStream;

public class SQLnetInputStream implements SQLnetDef {
   private boolean eof;
   private boolean gotReset = false;
   private InputStream soc;

   SQLnetInputStream(InputStream soc) {
      this.soc = soc;
   }

   public synchronized int readPacket(byte[] b) throws IOException {
      int bytesUnread = 8;
      int totalBytesRead = 0;
      if (this.eof) {
         return 0;
      } else {
         while(bytesUnread != 0) {
            int bytesRead;
            if ((bytesRead = this.soc.read(b, totalBytesRead, bytesUnread)) <= 0) {
               return -1;
            }

            totalBytesRead += bytesRead;
            bytesUnread -= bytesRead;
         }

         if (b[4] > 19) {
            throw new IOException("Bad packet type");
         } else {
            bytesUnread = b[0] & 255;
            bytesUnread <<= 8;
            bytesUnread |= b[1] & 255;
            if (bytesUnread > b.length) {
               throw new IOException("SDU mismatch");
            } else {
               int bytesRead;
               for(int var8 = bytesUnread - totalBytesRead; var8 != 0; var8 -= bytesRead) {
                  if ((bytesRead = this.soc.read(b, totalBytesRead, var8)) <= 0) {
                     return -1;
                  }

                  totalBytesRead += bytesRead;
               }

               return b[4];
            }
         }
      }
   }

   private int readDataPacket(byte[] b) throws IOException {
      int packetType = this.readPacket(b);
      int bytesRead;
      if (packetType != 6) {
         bytesRead = -1;
         if (packetType == 12) {
            if (b[10] == 2) {
               this.gotReset = true;
            } else {
               this.gotReset = false;
            }

            throw new SQLnetBreakException();
         }

         if (packetType > 0) {
            throw new IOException("Unexpected packet!");
         }
      } else {
         bytesRead = b[0] & 255;
         bytesRead <<= 8;
         bytesRead |= b[1] & 255;
         int f = b[8] & 255;
         f <<= 8;
         f |= b[9] & 255;
         if ((f & 64) != 0) {
            this.eof = true;
         }
      }

      return bytesRead;
   }

   public void readConnectData(byte[] b) throws IOException {
      int bytesToCopy = b.length;
      int bytesCopied = 0;

      int bytesRead;
      for(byte[] p = new byte[4086]; bytesCopied < bytesToCopy; bytesCopied += bytesRead) {
         bytesRead = this.readDataPacket(p) - 10;
         System.arraycopy(p, 10, b, bytesCopied, bytesRead);
      }

   }

   public int read(byte[] b, int l) throws IOException {
      return this.readDataPacket(b);
   }

   public int available() throws IOException {
      return 0;
   }

   public void close() throws IOException {
   }

   public boolean gotResetPacket() {
      return this.gotReset;
   }

   public void resetComplete() {
      this.gotReset = false;
   }
}
