package oracle.sqlnet;

import java.io.IOException;
import java.io.OutputStream;

public class SQLnetOutputStream implements SQLnetDef {
   private boolean eof;
   private OutputStream soc;

   SQLnetOutputStream(OutputStream soc) {
      this.soc = soc;
   }

   private synchronized void writeDataPacket(byte[] b, int l, boolean moreComing) throws IOException {
      int f = this.eof ? 64 : 0;
      if (moreComing) {
         f |= 32;
      }

      b[0] = (byte)(l / 256);
      b[1] = (byte)(l % 256);
      b[4] = 6;
      b[8] = (byte)(f / 256);
      b[9] = (byte)(f % 256);
      this.soc.write(b, 0, l);
   }

   private void writeConnectData(byte[] b, String conData, int conDataLen) throws IOException {
      int bLen = b.length - 10;
      int getTotal = 0;

      int getLen;
      for(int leftToSend = conDataLen; getTotal < conDataLen; getTotal += getLen) {
         getLen = Math.min(leftToSend, bLen);
         conData.getBytes(getTotal, getLen, b, 10);
         this.writeDataPacket(b, getLen + 10, false);
         leftToSend -= getLen;
      }

   }

   public synchronized void writeMarkerPacket() throws IOException {
      byte[] b = new byte[10];
      b[0] = (byte)(b.length / 256);
      b[1] = (byte)(b.length % 256);
      b[4] = 12;
      b[8] = 0;
      this.soc.write(b, 0, b.length);
   }

   public synchronized void writeMarkerPacket(byte type) throws IOException {
      byte[] b = new byte[11];
      b[0] = (byte)(b.length / 256);
      b[1] = (byte)(b.length % 256);
      b[4] = 12;
      b[8] = 1;
      b[10] = type;
      this.soc.write(b, 0, b.length);
   }

   public void writeConnectPacket(int sduSize, int tduSize, String conData) throws IOException {
      byte[] b = new byte[255];
      int conDataLen = conData == null ? 0 : conData.length();
      int l;
      boolean conDataOflow;
      if (34 + conDataLen <= b.length) {
         l = 34 + conDataLen;
         conDataOflow = false;
      } else {
         l = 34;
         conDataOflow = true;
      }

      b[0] = (byte)(l / 256);
      b[1] = (byte)(l % 256);
      b[4] = 1;
      b[8] = 1;
      b[9] = 51;
      b[10] = 1;
      b[11] = 44;
      b[14] = (byte)(sduSize / 256);
      b[15] = (byte)(sduSize % 256);
      b[16] = (byte)(tduSize / 256);
      b[17] = (byte)(tduSize % 256);
      b[18] = 127;
      b[19] = 8;
      b[22] = 1;
      b[24] = (byte)(conDataLen / 256);
      b[25] = (byte)(conDataLen % 256);
      b[27] = 34;
      b[30] = b[32] = 4;
      if (!conDataOflow && conDataLen > 0) {
         conData.getBytes(0, conDataLen, b, 34);
      }

      this.soc.write(b, 0, l);
      if (conDataOflow) {
         this.writeConnectData(b, conData, conDataLen);
      }

   }

   public void write(byte[] b, int l, boolean m) throws IOException {
      this.writeDataPacket(b, l, m);
   }

   public void close() throws IOException {
      this.eof = true;
   }

   public void close(byte[] b, int l) throws IOException {
      this.eof = true;
      this.write(b, l, false);
   }
}
